﻿// <summary>ソースコード：要素演算データクラス</summary>
// <author>CommonMP</author>
using System;
using System.Collections.Generic;
using System.Text;

using CommonMP.HYSSOP.Interface.HSData;
using CommonMP.HYSSOP.Interface.HSViewer;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.Interface.Controller;

namespace jp.or.jccajsce
{
    /// <summary><para>class outline:</para>
    /// <para>要素内演算中データクラス</para>
    /// </summary>
    /// <remarks><para>history:</para>
    /// <para>[CommonMP][ver 1.0.0][2010/01/25][新規作成]</para>
    /// <para>remarks</para>
    /// <para>演算中の状態、初期条件等を保持するデータクラス</para>
    /// </remarks>
    [Serializable]
    public class McQuasi2DSteadyFlowModelCalInfo : McCalInfo
    {
        /// <summary>Calc()メソッドの呼出し回数</summary>
        public int loopCount = 0;
       
        /// <summary>横断方向座標数</summary>
        public const int FLOW00_MD0 = 500;        
        /// <summary>テーブルデータ数</summary>
        public const int FLOW00_MD1 = 70;        
        /// <summary>横断方向最大断面ブロック数</summary>
        public const int FLOW00_MD2 = 30;
        /// <summary>河道最大断面数</summary>
        public const int FLOW00_MD3 = 1500;
        /// <summary>最大河道数</summary>
        public const int FLOW00_MD4 = 30;


        #region 断面圧縮
        /// <summary>横断方向座標数 (圧縮後の横断座標数)</summary>
        public int[] COMXYZ_NDATA = new int[FLOW00_MD3];
        /// <summary>x座標 (圧縮後の座標)</summary>
        public double[,] COMXYZ_XZ= new double[FLOW00_MD0, FLOW00_MD3];
        /// <summary>y座標 (圧縮後の座標)</summary>
        public double[,] COMXYZ_YZ = new double[FLOW00_MD0, FLOW00_MD3];
        /// <summary>パネル番号 (圧縮後の座標)</summary>
        public int[,] COMXYZ_IFG = new int[FLOW00_MD0, FLOW00_MD3];
        /// <summary>粗度 (圧縮後の座標)</summary>
        public double[,] COMXYZ_SOD = new double[FLOW00_MD0, FLOW00_MD3];
        /// <summary>低水路幅 (圧縮後の座標)</summary>
        public double[] COMXYZ_BBL = new double[FLOW00_MD3];
        /// <summary>低水路満杯水位 (圧縮後の座標)</summary>
        public double[] COMXYZ_HHL = new double[FLOW00_MD3];
        /// <summary>（低水路）平均河床高 (圧縮後の座標)</summary>
        public double[] COMXYZ_BED = new double[FLOW00_MD3];
        #endregion


        #region 計算河道データ
        /// <summary>累加距離 (km)</summary>
        public string[] COMDAN_FL = new string[FLOW00_MD3];
        /// <summary>断面ID </summary>
        public string[] COMDAN_ID = new string[FLOW00_MD3];
        /// <summary>TopoID </summary>
        public string[] COMDAN_TOPO = new string[FLOW00_MD3];
        /// <summary>河川名</summary>
        public string[] COMDAN_RIVR = new string[FLOW00_MD3];
        /// <summary>同河道における先頭断面番号</summary>
        public int[] COMDAN_IDS = new int[20];
        /// <summary>同河道における末尾断面番号</summary>
        public int[] COMDAN_IDE = new int[20];
        /// <summary>河道数</summary>
        public int COMDAN_IRIV = 0;
        /// <summary>総断面数</summary>
        public int COMDAN_IDAN = 0;
        /// <summary>当該断面における接続河道インデックス（接続なしの場合は-1）</summary>
        public int[] COMDAN_RIVC = new int[FLOW00_MD3];
        /// <summary>当該断面の下流断面の断面インデックス（本川最下流断面の場合は-1）</summary>
        public int[] COMDAN_IDL = new int[FLOW00_MD3];
        #endregion


        #region 河道断面特性テーブル
        /// <summary>横断方向分割数</summary>
        public int[] COMHAR_NBUN = new int[FLOW00_MD3];
        /// <summary>水位 (m)</summary>
        public double[,] COMHAR_HH = new double[FLOW00_MD1, FLOW00_MD3];
        /// <summary>河積 (m3)</summary>
        public double[,,] COMHAR_AA = new double[FLOW00_MD1, FLOW00_MD2, FLOW00_MD3];
        /// <summary>径深 (m)</summary>
        public double[,,] COMHAR_RR = new double[FLOW00_MD1, FLOW00_MD2, FLOW00_MD3];
        /// <summary>潤辺 (m)</summary>
        public double[,,] COMHAR_SS = new double[FLOW00_MD1, FLOW00_MD2, FLOW00_MD3]; 
        /// <summary>区間距離 (m)</summary>
        public double[] COMHAR_DX = new double[FLOW00_MD3];
        /// <summary>水面幅</summary>
        public double[,,] COMHAR_BL = new double[FLOW00_MD1, FLOW00_MD2, FLOW00_MD3];
        #endregion
        
        /// <summary>湾曲曲率半径(m)</summary>
        public double[] COMWAN_R = new double[FLOW00_MD3];
        /// <summary>河床勾配</summary>
        public double[] COMTEN_GRD = new double[FLOW00_MD3];
       
        #region 植生データ
        /// <summary>左の距離標から植生の中心位置までの距離 (m)</summary>
        public double[][] COMTDB_TXX = new double[FLOW00_MD3][];
        /// <summary>左の距離標から植生の左端までの距離 (m)</summary>
        public double[][] COMTDB_TXL = new double[FLOW00_MD3][];
        /// <summary>左の距離標から植生の右端までの距離 (m)</summary>
        public double[][] COMTDB_TXR = new double[FLOW00_MD3][];
        /// <summary>植生幅 (m)</summary>
        public double[][] COMTDB_TWW = new double[FLOW00_MD3][];
        /// <summary>植生高さ (m)</summary>
        public double[][] COMTDB_THH = new double[FLOW00_MD3][];
        /// <summary>植生樹冠標高（センター位置）(m)</summary>
        public double[][] COMTDB_TTH = new double[FLOW00_MD3][];
        #endregion


        #region 樹木
        /// <summary>水没樹木の上面粗度係数 (=0.05)</summary>
        public double COMFFF_TSD = 0.0;
        /// <summary>
        /// FT[0]  ：境界混合係数 急拡 (=0.04)
        /// FT[1]  ：境界混合係数 低水路と高水敷境界(b/B＜0.6) (=0.17)
        /// FT[2]  ：境界混合係数 低水路と高水敷境界(0.6=＜b/B＜0.8) (=0.07)
        /// FT[3]  ：境界混合係数 低水路と高水敷境界(0.8=＜b/B) (=0.04)
        /// </summary>
        public double[] COMFFF_FT = new double[4];
        /// <summary>
        /// TAU[0] ：境界混合係数 樹木群境界 堤防に接する (=0.03)
        /// TAU[1] ：境界混合係数 樹木群境界 流れに挟まれる (=0.10)
        /// TAU[2] ：境界混合係数 樹木群境界 流れに挟まれ水没 (=0.17)
        /// </summary>
        public double[] COMFFF_TAU = new double[3];

        #endregion

        
        #region 粗度係数による断面分割
        /// <summary>粗度係数区分の粗度係数の値</summary>
        public double[][] COSODO_FNN = new double[FLOW00_MD3][];
        /// <summary>粗度係数区分の樹木or河道の別（true: 樹木, false: 河道）</summary>
        public bool[][] COSODO_ISTREE = new bool[FLOW00_MD3][];
        /// <summary>粗度係数区分ブロックの樹冠の標高（樹木群がない場合は、-999）</summary>
        public double[][] COSODO_THH = new double[FLOW00_MD3][];
        /// <summary>粗度係数区分ブロックの樹高（樹木群がない場合は、-999）</summary>
        public double[][] COSODO_TTH = new double[FLOW00_MD3][];
        /// <summary>粗度係数区分ブロックの幅（樹木群がない場合は、-999）</summary>
        public double[][] COSODO_BW = new double[FLOW00_MD3][];
        /// <summary>粗度係数区分（左端）の測点インデックス </summary>
        public int[][] COSODO_IDX = new int[FLOW00_MD3][];
        #endregion

        #region 合流条件
        /// <summary>合流時の川幅条件B1 (マニュアル参照)</summary>
        public double[] COMTBL_BB1 = new double[FLOW00_MD4];
        /// <summary>合流時の川幅条件B2 (マニュアル参照)</summary>
        public double[] COMTBL_BB2 = new double[FLOW00_MD4];
        /// <summary>合流時の川幅条件B3 (マニュアル参照)</summary>
        public double[] COMTBL_BB3 = new double[FLOW00_MD4];
        /// <summary>合流時の角度θ1(マニュアル参照)</summary>
        public double[] COMTBL_ANG1 = new double[FLOW00_MD4];
        /// <summary>合流時の角度θ2(マニュアル参照)</summary>
        public double[] COMTBL_ANG2 = new double[FLOW00_MD4];
        #endregion


        #region 断面ブロック境界条件
        /// <summary>横断面ブロック左岸(TREE)潤辺</summary>
        public double[][] COMWET_S0 = new double[FLOW00_MD3][];
        /// <summary>横断面ブロック左岸(WATER)潤辺</summary>
        public double[][] COMWET_S1 = new double[FLOW00_MD3][];
        /// <summary>横断面ブロック右岸(TREE)潤辺</summary>
        public double[][] COMWET_S2 = new double[FLOW00_MD3][];
        /// <summary>横断面ブロック右岸(WATER)潤辺</summary>
        public double[][] COMWET_S3 = new double[FLOW00_MD3][];
        /// <summary>横断面ブロック左岸(TREE)境界混合係数</summary>
        public double[][] COMWET_F0 = new double[FLOW00_MD3][];
        /// <summary>横断面ブロック左岸(WATER)境界混合係数</summary>
        public double[][] COMWET_F1 = new double[FLOW00_MD3][];
        /// <summary>横断面ブロック右岸(TREE)境界混合係数</summary>
        public double[][] COMWET_F2 = new double[FLOW00_MD3][];
        /// <summary>横断面ブロック右岸(WATER)境界混合係数</summary>
        public double[][] COMWET_F3 = new double[FLOW00_MD3][];
        #endregion


        #region 計算結果
        /// <summary>断面ブロック流速 (m/s)</summary>
        public double[,] COMUUU_U = new double[FLOW00_MD1, FLOW00_MD3];
        /// <summary>計算水位 (m)</summary>
        public double[] COMCAL_CAL = new double[FLOW00_MD3];
        /// <summary>水深 (m)</summary>
        public double[] COMCAL_DEP = new double[FLOW00_MD3];
        /// <summary>流量(m3/s)</summary>
        public double[] COMCAL_QQ = new double[FLOW00_MD3];
        /// <summary>流積(m2)</summary>
        public double[] COMCAL_AA = new double[FLOW00_MD3];
        /// <summary>流速(m/s)</summary>
        public double[] COMCAL_UU = new double[FLOW00_MD3];
        /// <summary>フルード数</summary>
        public double[] COMCAL_FRR = new double[FLOW00_MD3];
        /// <summary>水面幅</summary>
        public double[] COMCAL_BBB = new double[FLOW00_MD3];
        /// <summary>エネルギー勾配</summary>
        public double[] COMCAL_EGRD = new double[FLOW00_MD3];
        /// <summary>流量比（計算流量／設定流量）</summary>
        public double[] COMCAL_QRTO = new double[FLOW00_MD3];
        /// <summary>支川合流による水位上昇</summary>
        public double[] COMCAL_DH01 = new double[FLOW00_MD3];
        /// <summary>構造物による水位上昇</summary>
        public double[] COMCAL_DH02 = new double[FLOW00_MD3];
        /// <summary>湾曲水位上昇(m)</summary>
        public double[] COMCAL_DH03 = new double[FLOW00_MD3];
        ///<summary> 砂州水位上昇(m)</summary>
        public double[] COMCAL_DH04 = new double[FLOW00_MD3];
        #endregion

        #region 構造物
        /// <summary>（橋梁）橋脚形状係数</summary>
        public double[] COMBRD_CD = new double[FLOW00_MD3];
        /// <summary>（橋梁）ピア上流側の水路幅</summary>
        public double[] COMBRD_B1 = new double[FLOW00_MD3];
        /// <summary>（橋梁）全水路幅から、ピア幅の総計を削除した幅</summary>
        public double[] COMBRD_B2 = new double[FLOW00_MD3];
        /// <summary>（橋梁）ピア１基の幅</summary>
        public double[] COMBRD_PW = new double[FLOW00_MD3];
        /// <summary>（橋梁）ピア本数</summary>
        public int[] COMBRD_NN = new int[FLOW00_MD3];
        #endregion


        /// <summary><para>method outline:</para>
        /// <para>自己複製</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para> Clone( ) </para>
        /// </example>
        /// <param name="">無し</param>
        /// <returns>同一内容で　別インスタンス</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>収束演算時、特定の時刻に状態を戻す場合等に使用する</para>
        /// </remarks>
        public override McCalInfo Clone()
        {
            McQuasi2DSteadyFlowModelCalInfo csRtn = new McQuasi2DSteadyFlowModelCalInfo();
            csRtn.CopyInfo(this);
            return csRtn;
        }

        /// <summary><para>method outline:</para>
        /// <para>引数で与えられた情報を自分にコピーを行う</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para> bool bRtn = CopyInfo(csOrgInfo) </para>
        /// </example>
        /// <param name="csOrgInfo">コピー元情報</param>
        /// <returns> bool true :正常 , false:異常</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        public override bool CopyInfo(McCalInfo csOrgInfo)
        {
            McQuasi2DSteadyFlowModelCalInfo csOrdDt = (McQuasi2DSteadyFlowModelCalInfo)csOrgInfo;

            this.loopCount = csOrdDt.loopCount;


            Array.Copy(csOrdDt.COMXYZ_NDATA, this.COMXYZ_NDATA, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMXYZ_XZ, 0, this.COMXYZ_XZ, 0, FLOW00_MD0 * FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMXYZ_YZ, 0, this.COMXYZ_YZ, 0, FLOW00_MD0 * FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMXYZ_IFG, 0, this.COMXYZ_IFG, 0, FLOW00_MD0 * FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMXYZ_SOD, 0, this.COMXYZ_SOD, 0, FLOW00_MD0 * FLOW00_MD3);
            Array.Copy(csOrdDt.COMDAN_FL, this.COMDAN_FL, FLOW00_MD3);
            Array.Copy(csOrdDt.COMDAN_ID, this.COMDAN_ID, FLOW00_MD3);
            Array.Copy(csOrdDt.COMDAN_TOPO, this.COMDAN_TOPO, FLOW00_MD3);
            Array.Copy(csOrdDt.COMDAN_RIVR, this.COMDAN_RIVR, 20);
            Array.Copy(csOrdDt.COMDAN_IDS, this.COMDAN_IDS, 20);
            Array.Copy(csOrdDt.COMDAN_IDE, this.COMDAN_IDE, 20);
            this.COMDAN_IRIV = csOrdDt.COMDAN_IRIV;
            this.COMDAN_IDAN = csOrdDt.COMDAN_IDAN;
            Array.Copy(csOrdDt.COMDAN_RIVC, this.COMDAN_RIVC, FLOW00_MD3);
            Array.Copy(csOrdDt.COMDAN_IDL, this.COMDAN_IDL, FLOW00_MD3);

            //分割断面
            Array.Copy(csOrdDt.COMHAR_NBUN, this.COMHAR_NBUN, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMHAR_HH, 0, this.COMHAR_HH, 0, FLOW00_MD1 * FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMHAR_AA, 0, this.COMHAR_AA, 0, FLOW00_MD1 * FLOW00_MD2 * FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMHAR_RR, 0, this.COMHAR_RR, 0, FLOW00_MD1 * FLOW00_MD2 * FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMHAR_DX, 0, this.COMHAR_DX, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMHAR_BL, 0, this.COMHAR_BL, 0, FLOW00_MD1 * FLOW00_MD2 * FLOW00_MD3);

            Buffer.BlockCopy(csOrdDt.COMWAN_R, 0, this.COMWAN_R, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMTEN_GRD, 0, this.COMTEN_GRD, 0, FLOW00_MD3);

            this.COMTDB_TXX = (double[][])csOrdDt.COMTDB_TXX.Clone();
            this.COMTDB_TXL = (double[][])csOrdDt.COMTDB_TXL.Clone();
            this.COMTDB_TXX = (double[][])csOrdDt.COMTDB_TXR.Clone();
            this.COMTDB_TWW = (double[][])csOrdDt.COMTDB_TWW.Clone();
            this.COMTDB_THH = (double[][])csOrdDt.COMTDB_THH.Clone();
            this.COMTDB_TTH = (double[][])csOrdDt.COMTDB_TTH.Clone();

            // 境界混合係数
            this.COMFFF_TSD = csOrdDt.COMFFF_TSD;
            Buffer.BlockCopy(csOrdDt.COMFFF_FT, 0, this.COMFFF_FT, 0, 4);
            Buffer.BlockCopy(csOrdDt.COMFFF_TAU, 0, this.COMFFF_TAU, 0, 3);

            // 断面ブロック分割
            this.COSODO_FNN = (double[][])csOrdDt.COSODO_FNN.Clone();
            this.COSODO_ISTREE = (bool[][])csOrdDt.COSODO_ISTREE.Clone();
            this.COSODO_THH = (double[][])csOrdDt.COSODO_THH.Clone();
            this.COSODO_TTH = (double[][])csOrdDt.COSODO_TTH.Clone();
            this.COSODO_BW = (double[][])csOrdDt.COSODO_BW.Clone();
            this.COSODO_IDX = (int[][])csOrdDt.COSODO_IDX.Clone();

            // 合流条件
            Array.Copy(csOrdDt.COMTBL_BB1, this.COMTBL_BB1, FLOW00_MD4);
            Array.Copy(csOrdDt.COMTBL_BB2, this.COMTBL_BB2, FLOW00_MD4);
            Array.Copy(csOrdDt.COMTBL_BB3, this.COMTBL_BB3, FLOW00_MD4);
            Array.Copy(csOrdDt.COMTBL_ANG1, this.COMTBL_ANG1, FLOW00_MD4);
            Array.Copy(csOrdDt.COMTBL_ANG2, this.COMTBL_ANG2, FLOW00_MD4);

            // 断面ブロック境界
            this.COMWET_S0 = (double[][])csOrdDt.COMWET_S0.Clone();
            this.COMWET_S1 = (double[][])csOrdDt.COMWET_S1.Clone();
            this.COMWET_S2 = (double[][])csOrdDt.COMWET_S2.Clone();
            this.COMWET_S3 = (double[][])csOrdDt.COMWET_S3.Clone();
            this.COMWET_F0 = (double[][])csOrdDt.COMWET_F0.Clone();
            this.COMWET_F1 = (double[][])csOrdDt.COMWET_F1.Clone();
            this.COMWET_F2 = (double[][])csOrdDt.COMWET_F2.Clone();
            this.COMWET_F3 = (double[][])csOrdDt.COMWET_F3.Clone();

            // 計算結果
            Buffer.BlockCopy(csOrdDt.COMUUU_U, 0, this.COMUUU_U, 0, FLOW00_MD1 * FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMCAL_CAL, 0, this.COMCAL_CAL, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMCAL_DEP, 0, this.COMCAL_DEP, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMCAL_QQ, 0, this.COMCAL_QQ, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMCAL_AA, 0, this.COMCAL_AA, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMCAL_UU, 0, this.COMCAL_UU, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMCAL_FRR, 0, this.COMCAL_FRR, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMCAL_BBB, 0, this.COMCAL_BBB, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMCAL_EGRD, 0, this.COMCAL_EGRD, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMCAL_QRTO, 0, this.COMCAL_QRTO, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMCAL_DH01, 0, this.COMCAL_DH01, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMCAL_DH02, 0, this.COMCAL_DH02, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMCAL_DH03, 0, this.COMCAL_DH03, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMCAL_DH04, 0, this.COMCAL_DH04, 0, FLOW00_MD3);
            
            // 構造物
            Buffer.BlockCopy(csOrdDt.COMBRD_CD, 0, this.COMBRD_CD, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMBRD_B1, 0, this.COMBRD_B1, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMBRD_B2, 0, this.COMBRD_B2, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMBRD_PW, 0, this.COMBRD_PW, 0, FLOW00_MD3);
            Buffer.BlockCopy(csOrdDt.COMBRD_NN, 0, this.COMBRD_NN, 0, FLOW00_MD3);

            return true;
        }

    }
}
