﻿using System;
using System.Collections.Generic;
using System.Text;

using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.CoreImpl.Data;
using CommonMP.HYMCO.CoreImpl.Tool;

using System.Collections;
using System.IO;

namespace jp.or.jccajsce
{
    /// <summary>
    /// <para>準二次元不等流計算モデル（演算モデルクラス）JCCA_JSCE_MODEL</para>
    /// </summary>
    /// <remarks><para>history:</para>
    /// <para>[CommonMP][ver 1.0.0][2008/10/01][新規作成]</para>
    /// </remarks>
    public class McQuasi2DSteadyFlowModel : McStateCalModelBase
    {
        /// <summary> 演算データ（キャスト用） </summary>
        // 便利の為、キャスト用に定義しておく
        private McQuasi2DSteadyFlowModelCalInfo ModelCalInfo
        {
            get
            {
                return (McQuasi2DSteadyFlowModelCalInfo)m_csCalInfo;
            }
        }

        public string m_sWorkingDir;
        public string m_sInputFile1;    // 横断ファイル
        public string m_sInputFile2;    // 縦断ファイル
        public string m_sInputFile3;    // 距離標ファイル
        public string m_sOutputFile;

        private InputFiles inputFiles;

        const string calc_log = "00_Calc.log";    // 計算ログファイル

        #region 演算実行前処理関連メソッド

        //=========================
        // 演算実行前処理関連メソッド
        //=========================

        /// <summary><para>method outline:</para>
        /// <para>入力側の接続情報チェック</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>bool bRtn = ReceiveConnectionCheck(ref csErrorInf)</para>
        /// </example>
        /// <param name="csErrorInf">エラー出力</param>
        /// <param name="lInputDataNum">入力情報数</param>
        /// <param name="csInputCellData">演算に必要な入力情報配列</param>
        /// <returns>=true:正常、=false:異常</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>受信するデータが自モデルが期待している情報か否かをチェックする</para>
        /// </remarks>
        protected override bool ReceiveConnectionCheck(ref McStructErrorInfo csErrorInf, long lInputDataNum, McReceiveCellDataIF[] csInputCellData)
        {
            bool bRtn = true;
            for (long lLp = 0; lLp < lInputDataNum; lLp++)
            {
            }
            return bRtn;
        }

        /// <summary><para>method outline:</para>
        /// <para>出力側の接続情報チェック</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>bool bRtn = SendConnectionCheck(ref csErrorInf)</para>
        /// </example>
        /// <param name="csErrorInf">エラー出力</param>
        /// <param name="lOutputDataNum">出力情報数</param>
        /// <param name="csOutputCellData">出力情報配列</param>
        /// <returns>=true:正常、=false:異常</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>送信端子に設定されている伝送データが自モデルが期待している情報か否かをチェックする</para>
        /// </remarks>
        protected override bool SendConnectionCheck(ref McStructErrorInfo csErrorInf, long lOutputDataNum, McSendCellDataIF[] csOutputCellData)
        {
            bool bRtn = true;
            for (long lLp = 0; lLp < lOutputDataNum; lLp++)
            {
            }
            return bRtn;
        }

        /// <summary><para>method outline:</para>
        /// <para>モデルを初期化する</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>Initialize(csInitialData)</para>
        /// </example>
        /// <param name="csInitialData">初期化設定情報</param>
        /// <param name="lInputDataNum">入力情報数</param>
        /// <param name="csInputCellData">演算に必要な入力情報配列</param>
        /// <returns>無し</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        protected override bool Initialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
        {
            string log_file = Path.Combine(m_sWorkingDir, calc_log);
            if (File.Exists(log_file))
            {
                File.Delete(log_file);
            }
            WriteCalcLog("Init:" + ModelCalInfo.loopCount.ToString() + " start at " + DateTime.Now.ToString());
            bool bRtn = false;
            McInitialInfo csInDt = csInitialData as McInitialInfo;
            if (csInDt != null)
            {
                ModelCalInfo.loopCount = 0;

                bRtn = true;
            }

            inputFiles = new InputFiles(m_sInputFile1, m_sInputFile2, m_sInputFile3);

            Set_NETWORK();
            Set_CROSSSECTION();

            return bRtn;
        }

        /// <summary><para>method outline:</para>
        /// <para>モデルを初期化する</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>OnlineInitialize(csInitialData)</para>
        /// </example>
        /// <param name="csInitialData">初期化設定情報</param>
        /// <param name="lInputDataNum">入力情報数</param>
        /// <param name="csInputCellData">演算に必要な入力情報配列</param>
        /// <returns>bool</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        protected override bool OnlineInitialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
        {
            return true;
        }

        #endregion

        #region 演算実行処理関連メソッド

        //=======================
        // 演算実行処理関連メソッド
        //=======================

        /// <summary><para>method outline:</para>
        /// <para>モデル演算</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>long lRtn = Calculate(ref csInputCellData)</para>
        /// </example>
        /// <param name="lInputDataNum">入力情報数</param>
        /// <param name="csInputCellData">演算に必要な入力情報配列</param>
        /// <returns>=0:正常 -1:異常</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        protected override long Calculate(long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
        {
            ModelCalInfo.loopCount++;
            WriteCalcLog("Calc:" + ModelCalInfo.loopCount.ToString() + " start at " + DateTime.Now.ToString());


            for (long lLp = 0; lLp < lInputDataNum; lLp++)
            {
                if (McQuasi2DSteadyFlowModelDefine.RECEIVE_PATTERN_HOSEN_KARYUTAN_SUII_NAME.Equals(csInputCellData[lLp].GetReceivePatternID()))
                {
                    HySCellData d = csInputCellData[lLp].GetInterpolatedCell(0);
                    for (int i = 0; i < inputFiles.throughFile.sections.Count; i++)
                    {
                        ThroughFileSection rs = inputFiles.throughFile.sections[i];
                        if (Int32.Parse(rs.records[0].HonshiCode) == 0)
                        {
                            rs.records[0].WaterLevel = csInputCellData[lLp].Data(d, 0).ToString();
                            break;
                        }
                    }
                    continue;
                }


                for (int IR = 0; IR < 11; IR++)
                {
                    if (McQuasi2DSteadyFlowModelDefine.RECEIVE_PATTERN_HONSHISEN_RYUHAI_NAME_ARRAY[IR].Equals(csInputCellData[lLp].GetReceivePatternID()))
                    {
                        for (int i = 0; i < inputFiles.throughFile.sections.Count; i++)
                        {
                            ThroughFileSection rs = inputFiles.throughFile.sections[i];
                            if (Int32.Parse(rs.records[0].HonshiCode) == IR)
                            {
                                for (int j = 0; j < rs.records.Count; j++)
                                {
                                    HySCellData d = csInputCellData[lLp].GetInterpolatedCell(j);
                                    rs.records[j].Flow = csInputCellData[lLp].Data(d, 0).ToString();
                                }
                                break;
                            }
                        }
                        break;
                    }
                }

            }

            Log_HQRead();

            // 水位計算
            Calc_HWL();

            // 計算結果のログ出力
            Log_Calc();
//            Log_UUU();
            Log_SSSF();
//            Log_QDH01();


            // 演算結果のファイル出力
            File_RESULT();


            return 0;

        }

        /// <summary><para>method outline:</para>
        /// <para>モデル演算結果を外部のエレメントに対して公開する</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>long lRtn = DataFusion( lOutputDataNum, ref csOutputCellData)</para>
        /// </example>
        /// <param name="lOutputDataNum">出力情報数</param>
        /// <param name="csOutputCellData">出力情報配列</param>
        /// <returns>=0:正常 -1:異常</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        protected override long DataFusion(long lOutputDataNum, ref McSendCellDataIF[] csOutputCellData)
        {
            HySCellData[] csSndCellData = null;
            for (long lP = 0; lP < lOutputDataNum; lP++)
            {
                for (int IR = 0; IR < ModelCalInfo.COMDAN_IRIV; IR++)
                {
                    if (McQuasi2DSteadyFlowModelDefine.SEND_PATTERN_HONSHISEN_SUII_NAME_ARRAY[IR].Equals(csOutputCellData[lP].GetSendPatternID()))
                    {
                        int IDS = ModelCalInfo.COMDAN_IDS[IR];
                        int IDE = ModelCalInfo.COMDAN_IDE[IR];
                        csSndCellData = csOutputCellData[lP].PrepareSendCellD1();
                        for (long lLp = 0; lLp <= IDE - IDS; lLp++)
                        {
                            csSndCellData[lLp].m_dData[0] = ModelCalInfo.COMCAL_CAL[lLp + IDS];
                        }
                    }
                }
            }
            return 0;
        }

        #endregion

        #region その他必要なメソッド
        //====================
        // その他必要なメソッド
        //====================
        /// <summary><para>method outline:</para>
        /// <para>プロパティ情報を設定する</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>SetProperty(csCellMdlPropertyInfo)</para>
        /// </example>
        /// <param name="csCellMdlPropertyInfo">セル型プロパティ情報</param>
        /// <returns>無し</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        public override bool SetProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
        {
            bool bRtn = false;

            // プロパティ設定
            McCellModelPropertyInfo csPrptyInfo = csCellMdlPropertyInfo as McCellModelPropertyInfo;
            if (csPrptyInfo != null)
            {
                this.m_csDltTime = new HySTime(csPrptyInfo.GetStepTime());


                bRtn = csPrptyInfo.GetInfo(McQuasi2DSteadyFlowModelDefine.PROPERTY_KEY_WORkING_DIR,
                    ref m_sWorkingDir);
                // 相対パスを絶対パスに変換
                if (m_sWorkingDir == "")
                {
                    m_sWorkingDir = this.GetProjectGroupDirectory().ToString();
                }
                // m_sWorkingDirに絶対パスまたはプロジェクト・ディレクトリの外のパスが含まれていたら
                else if (m_sWorkingDir.IndexOf(":") >= 0 || m_sWorkingDir.IndexOf("..") >= 0)
                {
                    // Do nothing
                }
                // m_sWorkingDirがプロジェクト・ディレクトリ以下のパスならば
                else
                {
                    m_sWorkingDir = this.GetProjectGroupDirectory().ToString() + "\\" + m_sWorkingDir;
                }
                if (!bRtn || !Directory.Exists(m_sWorkingDir))
                {
                    return false;
                }

                bRtn = csPrptyInfo.GetInfo(McQuasi2DSteadyFlowModelDefine.PROPERTY_KEY_INPUT_FILE1,
                    ref m_sInputFile1);
                // 相対パスを絶対パスに変換
                if (m_sInputFile1 == "")
                {
                    m_sInputFile1 = this.GetProjectGroupDirectory().ToString();
                }
                // m_sInputFile1に絶対パスまたはプロジェクト・ディレクトリの外のパスが含まれていたら
                else if (m_sInputFile1.IndexOf(":") >= 0 || m_sInputFile1.IndexOf("..") >= 0)
                {
                    // Do nothing
                }
                // m_sInputFile1がプロジェクト・ディレクトリ以下のパスならば
                else
                {
                    m_sInputFile1 = this.GetProjectGroupDirectory().ToString() + "\\" + m_sInputFile1;
                }
                if (!bRtn || !File.Exists(m_sInputFile1))
                {
                    return false;
                }

                bRtn = csPrptyInfo.GetInfo(McQuasi2DSteadyFlowModelDefine.PROPERTY_KEY_INPUT_FILE2,
                    ref m_sInputFile2);
                // 相対パスを絶対パスに変換
                if (m_sInputFile2 == "")
                {
                    m_sInputFile2 = this.GetProjectGroupDirectory().ToString();
                }
                // m_sInputFile2に絶対パスまたはプロジェクト・ディレクトリの外のパスが含まれていたら
                else if (m_sInputFile2.IndexOf(":") >= 0 || m_sInputFile2.IndexOf("..") >= 0)
                {
                    // Do nothing
                }
                // m_sInputFile2がプロジェクト・ディレクトリ以下のパスならば
                else
                {
                    m_sInputFile2 = this.GetProjectGroupDirectory().ToString() + "\\" + m_sInputFile2;
                }
                if (!bRtn || !File.Exists(m_sInputFile2))
                {
                    return false;
                }

                bRtn = csPrptyInfo.GetInfo(McQuasi2DSteadyFlowModelDefine.PROPERTY_KEY_INPUT_FILE3,
                    ref m_sInputFile3);
                // 相対パスを絶対パスに変換
                if (m_sInputFile3 == "")
                {
                    m_sInputFile3 = this.GetProjectGroupDirectory().ToString();
                }
                // m_sInputFile3に絶対パスまたはプロジェクト・ディレクトリの外のパスが含まれていたら
                else if (m_sInputFile3.IndexOf(":") >= 0 || m_sInputFile3.IndexOf("..") >= 0)
                {
                    // Do nothing
                }
                // m_sInputFile3がプロジェクト・ディレクトリ以下のパスならば
                else
                {
                    m_sInputFile3 = this.GetProjectGroupDirectory().ToString() + "\\" + m_sInputFile3;
                }
                if (!bRtn || !File.Exists(m_sInputFile3))
                {
                    return false;
                }

                bRtn = csPrptyInfo.GetInfo(McQuasi2DSteadyFlowModelDefine.PROPERTY_KEY_OUTPUT_FILE,
                    ref m_sOutputFile);
                // 相対パスを絶対パスに変換
                if (m_sOutputFile == "")
                {
                    m_sOutputFile = this.GetProjectGroupDirectory().ToString();
                }
                // m_sOutputFileに絶対パスまたはプロジェクト・ディレクトリの外のパスが含まれていたら
                else if (m_sOutputFile.IndexOf(":") >= 0 || m_sOutputFile.IndexOf("..") >= 0)
                {
                    // Do nothing
                }
                // m_sOutputFileがプロジェクト・ディレクトリ以下のパスならば
                else
                {
                    m_sOutputFile = this.GetProjectGroupDirectory().ToString() + "\\" + m_sOutputFile;
                }
                if (!bRtn || !Directory.Exists(Path.GetDirectoryName(m_sOutputFile)))
                {
                    return false;
                }
                bRtn = true;
            }
            return bRtn;
        }

        /// <summary><para>method outline:</para>
        /// <para>プロパティ情報を設定する（Ver1.4例外対応）</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>SetProperty(csCellMdlPropertyInfo)</para>
        /// </example>
        /// <param name="csCellMdlPropertyInfo">セル型プロパティ情報</param>
        /// <returns>無し</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        public override bool SetProperty(McCellModelPropertyIF csCellMdlPropertyInfo, ref McStructErrorInfo csErrorInfo)
        {
            bool bRtn = false;

            // プロパティ設定
            McCellModelPropertyInfo csPrptyInfo = csCellMdlPropertyInfo as McCellModelPropertyInfo;
            if (csPrptyInfo != null)
            {
                this.m_csDltTime = new HySTime(csPrptyInfo.GetStepTime());


                bRtn = csPrptyInfo.GetInfo(McQuasi2DSteadyFlowModelDefine.PROPERTY_KEY_WORkING_DIR,
                    ref m_sWorkingDir);
                // 相対パスを絶対パスに変換
                if (m_sWorkingDir == "")
                {
                    m_sWorkingDir = this.GetProjectGroupDirectory().ToString();
                }
                // m_sWorkingDirに絶対パスまたはプロジェクト・ディレクトリの外のパスが含まれていたら
                else if (m_sWorkingDir.IndexOf(":") >= 0 || m_sWorkingDir.IndexOf("..") >= 0)
                {
                    // Do nothing
                }
                // m_sWorkingDirがプロジェクト・ディレクトリ以下のパスならば
                else
                {
                    m_sWorkingDir = this.GetProjectGroupDirectory().ToString() + "\\" + m_sWorkingDir;
                }
                if (!bRtn || !Directory.Exists(m_sWorkingDir))
                {
                    csErrorInfo.AddCheckErrorData(GetID(), McQuasi2DSteadyFlowModelDefine.MODEL_KIND, "結果フォルダがありません。" +
                       "(" + m_sWorkingDir + ")");
                    return false;
                }

                bRtn = csPrptyInfo.GetInfo(McQuasi2DSteadyFlowModelDefine.PROPERTY_KEY_INPUT_FILE1,
                    ref m_sInputFile1);
                // 相対パスを絶対パスに変換
                if (m_sInputFile1 == "")
                {
                    m_sInputFile1 = this.GetProjectGroupDirectory().ToString();
                }
                // m_sInputFile1に絶対パスまたはプロジェクト・ディレクトリの外のパスが含まれていたら
                else if (m_sInputFile1.IndexOf(":") >= 0 || m_sInputFile1.IndexOf("..") >= 0)
                {
                    // Do nothing
                }
                // m_sInputFile1がプロジェクト・ディレクトリ以下のパスならば
                else
                {
                    m_sInputFile1 = this.GetProjectGroupDirectory().ToString() + "\\" + m_sInputFile1;
                }
                if (!bRtn || !File.Exists(m_sInputFile1))
                {
                    csErrorInfo.AddCheckErrorData(GetID(), McQuasi2DSteadyFlowModelDefine.MODEL_KIND, "横断ファイルがありません。" +
                         "(" + m_sInputFile1 + ")");
                    return false;
                }

                bRtn = csPrptyInfo.GetInfo(McQuasi2DSteadyFlowModelDefine.PROPERTY_KEY_INPUT_FILE2,
                    ref m_sInputFile2);
                // 相対パスを絶対パスに変換
                if (m_sInputFile2 == "")
                {
                    m_sInputFile2 = this.GetProjectGroupDirectory().ToString();
                }
                // m_sInputFile2に絶対パスまたはプロジェクト・ディレクトリの外のパスが含まれていたら
                else if (m_sInputFile2.IndexOf(":") >= 0 || m_sInputFile2.IndexOf("..") >= 0)
                {
                    // Do nothing
                }
                // m_sInputFile2がプロジェクト・ディレクトリ以下のパスならば
                else
                {
                    m_sInputFile2 = this.GetProjectGroupDirectory().ToString() + "\\" + m_sInputFile2;
                }
                if (!bRtn || !File.Exists(m_sInputFile2))
                {
                    csErrorInfo.AddCheckErrorData(GetID(), McQuasi2DSteadyFlowModelDefine.MODEL_KIND, "縦断ファイルがありません。" +
                        "(" + m_sInputFile2 + ")");
                    return false;
                }

                bRtn = csPrptyInfo.GetInfo(McQuasi2DSteadyFlowModelDefine.PROPERTY_KEY_INPUT_FILE3,
                    ref m_sInputFile3);
                // 相対パスを絶対パスに変換
                if (m_sInputFile3 == "")
                {
                    m_sInputFile3 = this.GetProjectGroupDirectory().ToString();
                }
                // m_sInputFile3に絶対パスまたはプロジェクト・ディレクトリの外のパスが含まれていたら
                else if (m_sInputFile3.IndexOf(":") >= 0 || m_sInputFile3.IndexOf("..") >= 0)
                {
                    // Do nothing
                }
                // m_sInputFile3がプロジェクト・ディレクトリ以下のパスならば
                else
                {
                    m_sInputFile3 = this.GetProjectGroupDirectory().ToString() + "\\" + m_sInputFile3;
                }
                if (!bRtn || !File.Exists(m_sInputFile3))
                {
                    csErrorInfo.AddCheckErrorData(GetID(), McQuasi2DSteadyFlowModelDefine.MODEL_KIND, "距離標ファイルがありません。" +
                        "(" + m_sInputFile3 + ")");
                    return false;
                }

                bRtn = csPrptyInfo.GetInfo(McQuasi2DSteadyFlowModelDefine.PROPERTY_KEY_OUTPUT_FILE,
                    ref m_sOutputFile);
                // 相対パスを絶対パスに変換
                if (m_sOutputFile == "")
                {
                    m_sOutputFile = this.GetProjectGroupDirectory().ToString();
                }
                // m_sOutputFileに絶対パスまたはプロジェクト・ディレクトリの外のパスが含まれていたら
                else if (m_sOutputFile.IndexOf(":") >= 0 || m_sOutputFile.IndexOf("..") >= 0)
                {
                    // Do nothing
                }
                // m_sOutputFileがプロジェクト・ディレクトリ以下のパスならば
                else
                {
                    m_sOutputFile = this.GetProjectGroupDirectory().ToString() + "\\" + m_sOutputFile;
                }
                if (!bRtn || !Directory.Exists(Path.GetDirectoryName(m_sOutputFile)))
                {
                    csErrorInfo.AddCheckErrorData(GetID(), McQuasi2DSteadyFlowModelDefine.MODEL_KIND, "出力ファイルへのパスが見つかりません。" +
                        "(" + m_sOutputFile + ")");
                    return false;
                }
                bRtn = true;
            }
            return bRtn;
        }

        /// <summary><para>method outline:</para>
        /// <para>演算実行中断中のプロパティ等情報設定</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>bool bRtn = SetOnlineProperty(csCellMdlPropertyInfo)</para>
        /// </example>
        /// <param name="csCellMdlPropertyInfo">プロパティ情報</param>
        /// <returns>無し</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>演算中断中に　プロパティ情報を変更した場合にコールされる</para>
        /// </remarks>
        public override bool SetOnlineProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
        {
            bool bRtn = false;
            // プロパティ設定
            McCellModelPropertyInfo csPrptyInfo = csCellMdlPropertyInfo as McCellModelPropertyInfo;
            if (csPrptyInfo != null)
            {
                bRtn = true;
            }
            return bRtn;
        }

        /// <summary><para>method outline:</para>
        /// <para>計算開始時に動作する処理</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>bool bRtn = ReadyCalculation( )</para>
        /// </example>
        /// <param name="">無し</param>
        /// <returns>true :正常 , false:異常</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>計算開始の　最初に１回だけコールされるメソッドです。不要ならば、本メソッドをオーバーライドする必要はありません。</para>
        /// </remarks>
        public override bool ReadyCalculation()
        {
            return true;
        }
        #endregion

        // ログ出力
        #region LOGGING
        private void WriteCalcLog(string message)
        {
            File.AppendAllText(Path.Combine(m_sWorkingDir, calc_log),
                message + "\r\n", McQuasi2DSteadyFlowModelDefine.FILE_ENCODING);
            McLog.DebugOut(new HySString(this.GetType().FullName),
                new HySString("ReadyCalculation"),
                new HySString(message));
        }
        #endregion
                
        // 河道ネットワーク設定
        #region SET_NETWORK
        void Set_NETWORK()
        {

            ModelCalInfo.COMDAN_IRIV = inputFiles.throughFile.sections.Count; // 河川数（本支川の合計）
            // 断面総数の設定
            ModelCalInfo.COMDAN_IDAN = 0;
            for (int i = 0; i < ModelCalInfo.COMDAN_IRIV; i++)
            {
                ModelCalInfo.COMDAN_IDAN += inputFiles.throughFile.sections[i].records.Count;
            }

            // 断面ごとの河川名、TopoID、断面ID、累加距離の設定
            for (int i = 0; i < inputFiles.crossFile.sections.Count; i++)
            {
                CrossFileSection tr = inputFiles.crossFile.sections[i];
                ModelCalInfo.COMDAN_RIVR[i] = tr.RiverName;
                ModelCalInfo.COMDAN_TOPO[i] = tr.TopoId;
                ModelCalInfo.COMDAN_ID[i] = tr.DanmenId;
                ModelCalInfo.COMDAN_FL[i] = Convert.ToString(tr.getThroughRecord().TotalDistance);
            }

            // 本川支川の先頭断面番号と末尾断面番号の設定
            ModelCalInfo.COMDAN_IDS[0] = 0;
            ModelCalInfo.COMDAN_IDE[0] = inputFiles.throughFile.sections[0].records.Count - 1;
            for (int i = 1; i < ModelCalInfo.COMDAN_IRIV; i++)
            {
                ModelCalInfo.COMDAN_IDS[i] = ModelCalInfo.COMDAN_IDE[i - 1] + 1;
                ModelCalInfo.COMDAN_IDE[i] = ModelCalInfo.COMDAN_IDS[i] + inputFiles.throughFile.sections[i].records.Count - 1;
            }

            // 接続河川の設定（当該断面における接続河川インデックス、接続なしの場合は-1）         
            for (int IR = 0; IR < ModelCalInfo.COMDAN_IRIV; IR++)
            {
                int NS = ModelCalInfo.COMDAN_IDS[IR];
                int NE = ModelCalInfo.COMDAN_IDE[IR];
                int RIVC;

                for (int ID = NS; ID <= NE; ID++)
                {
                    RIVC = inputFiles.throughFile.sections[0].records[ID - NS].connected_RiverCode();
                    if (RIVC != -1 && IR == 0)
                    {
                        ModelCalInfo.COMDAN_RIVC[ID] = RIVC;
                    }
                    else
                    {
                        ModelCalInfo.COMDAN_RIVC[ID] = -1;
                    }
                }
            }

            // 当該断面の下流断面
            ModelCalInfo.COMDAN_IDL[0] = -1;
            for (int IR = 0; IR < ModelCalInfo.COMDAN_IRIV; IR++)
            {
                int NS = ModelCalInfo.COMDAN_IDS[IR];
                int NE = ModelCalInfo.COMDAN_IDE[IR];
                for (int ID = NS; ID <= NE; ID++)
                {
                    if (ID == NS)
                    {
                        string KPost = inputFiles.throughFile.sections[0].records[0].SetsuzokuIchi;
                        int NSC = ModelCalInfo.COMDAN_IDS[0];
                        int NEC = ModelCalInfo.COMDAN_IDE[0];
                        for (int i = NSC; i <= NEC; i++)
                        {
                            if ( ModelCalInfo.COMDAN_RIVC[i] == IR)
                            {
                                ModelCalInfo.COMDAN_IDL[ID] = i;
                                break;
                            }
                        }
                    }
                    else
                    {
                        ModelCalInfo.COMDAN_IDL[ID] = ID - 1;
                    }
                }
            }

            // 境界混合係数の設定
            Set_COEFF();

            // 接続諸元の設定
            for (int IR = 0; IR < ModelCalInfo.COMDAN_IRIV; IR++)
            {
                if (IR == 0)
                {
                    continue;
                }
                double.TryParse(inputFiles.throughFile.sections[IR].records[0].GoryuMaeRiverWidthB1, out ModelCalInfo.COMTBL_BB1[IR]);
                double.TryParse(inputFiles.throughFile.sections[IR].records[0].GoryuMaeRiverWidthB2, out ModelCalInfo.COMTBL_BB2[IR]);
                double.TryParse(inputFiles.throughFile.sections[IR].records[0].GoryuGoRiverWidthB3, out ModelCalInfo.COMTBL_BB3[IR]);
                double.TryParse(inputFiles.throughFile.sections[IR].records[0].GoryuKakudoT1, out ModelCalInfo.COMTBL_ANG1[IR]);
                double.TryParse(inputFiles.throughFile.sections[IR].records[0].GoryuKakudoT2, out ModelCalInfo.COMTBL_ANG2[IR]);
            }

            for (int ID = 0; ID < ModelCalInfo.COMDAN_IDAN; ID++)
            {
                CrossFileSection section = inputFiles.crossFile.sections[ID];

                // 区間距離
                double.TryParse(section.getThroughRecord().SectionDistance, out ModelCalInfo.COMHAR_DX[ID]);
                // 河床勾配
                double.TryParse(section.getThroughRecord().KasyoKoubai, out ModelCalInfo.COMTEN_GRD[ID]);
                // 湾曲曲率半径
                double.TryParse(section.getThroughRecord().WankyokuKyokuritsuHankei, out ModelCalInfo.COMWAN_R[ID]);

                // 橋梁諸元
                double.TryParse(section.getThroughRecord().KyoryoKouryokuKeisu, out ModelCalInfo.COMBRD_CD[ID]);
                double.TryParse(section.getThroughRecord().KyoryoWidth1, out ModelCalInfo.COMBRD_B1[ID]);
                double.TryParse(section.getThroughRecord().KyoryoWidth2, out ModelCalInfo.COMBRD_B2[ID]);
                double.TryParse(section.getThroughRecord().KyoryoPierWidth, out ModelCalInfo.COMBRD_PW[ID]);
                int.TryParse(section.getThroughRecord().KyoryoPierCount, out ModelCalInfo.COMBRD_NN[ID]);
            }

            Log_NETWORK();

                    
        }

        /// <summary>
        /// 境界混合係数のパラメータを設定する。
        /// </summary>
        void Set_COEFF()
        {
            double V = 0.0;
            if (!double.TryParse(inputFiles.throughFile.sections[0].records[0].SodoSuibotuJumoku, out V))
            {
                // INPUT DATA ERROR ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
                throw new Exception("下流端に[粗度係数 水没樹木]が指定されていません 河川名: " + inputFiles.throughFile.sections[0].records[0].RiverName);
                ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            }
            for (int i = 0; i < inputFiles.throughFile.sections.Count; i++)
            {
                if (inputFiles.throughFile.sections[0].records[0].SodoSuibotuJumoku
                    != inputFiles.throughFile.sections[i].records[0].SodoSuibotuJumoku)
                {
                    // INPUT DATA ERROR //////////////////////////////////////////////////////////////////////////////////////
                    throw new Exception("下流端に[粗度係数 水没樹木]が指定されていないか本川と異なる値が指定されています 河川名: "
                        + inputFiles.throughFile.sections[i].records[0].RiverName);
                    //////////////////////////////////////////////////////////////////////////////////////////////////////////
                }
                for (int j = 1; j < inputFiles.throughFile.sections[i].records.Count; j++)
                {
                    if (0 < inputFiles.throughFile.sections[i].records[j].SodoSuibotuJumoku.Length)
                    {
                        // INPUT DATA ERROR /////////////////////////////////////////////////////////
                        throw new Exception("下流端以外に[粗度係数 水没樹木]が指定されています 河川名: "
                            + inputFiles.throughFile.sections[i].records[j].RiverName
                            + " 断面ID: " + inputFiles.throughFile.sections[i].records[j].DanmenId);
                        /////////////////////////////////////////////////////////////////////////////
                    }
                }
            }

            ModelCalInfo.COMFFF_TSD = V;    // 樹木群上面の粗度係数


            if (!double.TryParse(inputFiles.throughFile.sections[0].records[0].KongouKeisu0, out V))
            {
                // INPUT DATA ERROR ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
                throw new Exception("下流端に[混合係数 急拡]が指定されていません 河川名: " + inputFiles.throughFile.sections[0].records[0].RiverName);
                ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            }
            for (int i = 0; i < inputFiles.throughFile.sections.Count; i++)
            {
                if (inputFiles.throughFile.sections[0].records[0].KongouKeisu0
                    != inputFiles.throughFile.sections[i].records[0].KongouKeisu0)
                {
                    // INPUT DATA ERROR //////////////////////////////////////////////////////////////////////////////////////
                    throw new Exception("下流端に[混合係数 急拡]が指定されていないか本川と異なる値が指定されています 河川名: "
                        + inputFiles.throughFile.sections[i].records[0].RiverName);
                    //////////////////////////////////////////////////////////////////////////////////////////////////////////
                }
                for (int j = 1; j < inputFiles.throughFile.sections[i].records.Count; j++)
                {
                    if (0 < inputFiles.throughFile.sections[i].records[j].KongouKeisu0.Length)
                    {
                        // INPUT DATA ERROR /////////////////////////////////////////////////////////
                        throw new Exception("下流端以外に[混合係数 急拡]が指定されています 河川名: "
                            + inputFiles.throughFile.sections[i].records[j].RiverName
                            + " 断面ID: " + inputFiles.throughFile.sections[i].records[j].DanmenId);
                        /////////////////////////////////////////////////////////////////////////////
                    }
                }
            }
            ModelCalInfo.COMFFF_FT[0] = V;  // 混合係数急拡

            if (!double.TryParse(inputFiles.throughFile.sections[0].records[0].KongouKeisu1, out V))
            {
                // INPUT DATA ERROR ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
                throw new Exception("下流端に[混合係数 b/BBB<0.6]が指定されていません 河川名: " + inputFiles.throughFile.sections[0].records[0].RiverName);
                ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            }
            for (int i = 0; i < inputFiles.throughFile.sections.Count; i++)
            {
                if (inputFiles.throughFile.sections[0].records[0].KongouKeisu1
                    != inputFiles.throughFile.sections[i].records[0].KongouKeisu1)
                {
                    // INPUT DATA ERROR //////////////////////////////////////////////////////////////////////////////////////
                    throw new Exception("下流端に[混合係数 b/BBB<0.6]が指定されていないか本川と異なる値が指定されています 河川名: "
                        + inputFiles.throughFile.sections[i].records[0].RiverName);
                    //////////////////////////////////////////////////////////////////////////////////////////////////////////
                }
                for (int j = 1; j < inputFiles.throughFile.sections[i].records.Count; j++)
                {
                    if (0 < inputFiles.throughFile.sections[i].records[j].KongouKeisu1.Length)
                    {
                        // INPUT DATA ERROR /////////////////////////////////////////////////////////
                        throw new Exception("下流端以外に[混合係数 b/BBB<0.6]が指定されています 河川名: "
                            + inputFiles.throughFile.sections[i].records[j].RiverName
                            + " 断面ID: " + inputFiles.throughFile.sections[i].records[j].DanmenId);
                        /////////////////////////////////////////////////////////////////////////////
                    }
                }
            }

            ModelCalInfo.COMFFF_FT[1] = V;  // 混合係数b/B<0.6

            if (!double.TryParse(inputFiles.throughFile.sections[0].records[0].KongouKeisu2, out V))
            {
                // INPUT DATA ERROR ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
                throw new Exception("下流端に[混合係数 0.6≦b/BBB]が指定されていません 河川名: " + inputFiles.throughFile.sections[0].records[0].RiverName);
                ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            }
            for (int i = 0; i < inputFiles.throughFile.sections.Count; i++)
            {
                if (inputFiles.throughFile.sections[0].records[0].KongouKeisu2
                    != inputFiles.throughFile.sections[i].records[0].KongouKeisu2)
                {
                    // INPUT DATA ERROR //////////////////////////////////////////////////////////////////////////////////////
                    throw new Exception("下流端に[混合係数 0.6≦b/BBB]が指定されていないか本川と異なる値が指定されています 河川名: "
                        + inputFiles.throughFile.sections[i].records[0].RiverName);
                    //////////////////////////////////////////////////////////////////////////////////////////////////////////
                }
                for (int j = 1; j < inputFiles.throughFile.sections[i].records.Count; j++)
                {
                    if (0 < inputFiles.throughFile.sections[i].records[j].KongouKeisu2.Length)
                    {
                        // INPUT DATA ERROR /////////////////////////////////////////////////////////
                        throw new Exception("下流端以外に[混合係数 0.6≦b/BBB]が指定されています 河川名: "
                            + inputFiles.throughFile.sections[i].records[j].RiverName
                            + " 断面ID: " + inputFiles.throughFile.sections[i].records[j].DanmenId);
                        /////////////////////////////////////////////////////////////////////////////
                    }
                }
            }

            ModelCalInfo.COMFFF_FT[2] = V;  // 混合係数0.6≦b/B

            if (!double.TryParse(inputFiles.throughFile.sections[0].records[0].KongouKeisu3, out V))
            {
                // INPUT DATA ERROR ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
                throw new Exception("下流端に[混合係数 0.8＜b/BBB]が指定されていません 河川名: " + inputFiles.throughFile.sections[0].records[0].RiverName);
                ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            }
            for (int i = 0; i < inputFiles.throughFile.sections.Count; i++)
            {
                if (inputFiles.throughFile.sections[0].records[0].KongouKeisu3
                    != inputFiles.throughFile.sections[i].records[0].KongouKeisu3)
                {
                    // INPUT DATA ERROR //////////////////////////////////////////////////////////////////////////////////////
                    throw new Exception("下流端に[混合係数 0.8＜b/BBB]が指定されていないか本川と異なる値が指定されています 河川名: "
                        + inputFiles.throughFile.sections[i].records[0].RiverName);
                    //////////////////////////////////////////////////////////////////////////////////////////////////////////
                }
                for (int j = 1; j < inputFiles.throughFile.sections[i].records.Count; j++)
                {
                    if (0 < inputFiles.throughFile.sections[i].records[j].KongouKeisu3.Length)
                    {
                        // INPUT DATA ERROR /////////////////////////////////////////////////////////
                        throw new Exception("下流端以外に[混合係数 0.8＜b/BBB]が指定されています 河川名: "
                            + inputFiles.throughFile.sections[i].records[j].RiverName
                            + " 断面ID: " + inputFiles.throughFile.sections[i].records[j].DanmenId);
                        /////////////////////////////////////////////////////////////////////////////
                    }
                }
            }

            ModelCalInfo.COMFFF_FT[3] = V;  // 混合係数0.8＜b/B

            if (!double.TryParse(inputFiles.throughFile.sections[0].records[0].KongouKeisu4, out V))
            {
                // INPUT DATA ERROR ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
                throw new Exception("下流端に[混合係数 堤防]が指定されていません 河川名: " + inputFiles.throughFile.sections[0].records[0].RiverName);
                ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            }
            for (int i = 0; i < inputFiles.throughFile.sections.Count; i++)
            {
                if (inputFiles.throughFile.sections[0].records[0].KongouKeisu4
                    != inputFiles.throughFile.sections[i].records[0].KongouKeisu4)
                {
                    // INPUT DATA ERROR //////////////////////////////////////////////////////////////////////////////////////
                    throw new Exception("下流端に[混合係数 堤防]が指定されていないか本川と異なる値が指定されています 河川名: "
                        + inputFiles.throughFile.sections[i].records[0].RiverName);
                    //////////////////////////////////////////////////////////////////////////////////////////////////////////
                }
                for (int j = 1; j < inputFiles.throughFile.sections[i].records.Count; j++)
                {
                    if (0 < inputFiles.throughFile.sections[i].records[j].KongouKeisu4.Length)
                    {
                        // INPUT DATA ERROR /////////////////////////////////////////////////////////
                        throw new Exception("下流端以外に[混合係数 堤防]が指定されています 河川名: "
                            + inputFiles.throughFile.sections[i].records[j].RiverName
                            + " 断面ID: " + inputFiles.throughFile.sections[i].records[j].DanmenId);
                        /////////////////////////////////////////////////////////////////////////////
                    }
                }
            }

            ModelCalInfo.COMFFF_TAU[0] = V;     // 混合係数堤防

            if (!double.TryParse(inputFiles.throughFile.sections[0].records[0].KongouKeisu5, out V))
            {
                // INPUT DATA ERROR ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
                throw new Exception("下流端に[混合係数 流れに挟まる]が指定されていません 河川名: " + inputFiles.throughFile.sections[0].records[0].RiverName);
                ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            }
            for (int i = 0; i < inputFiles.throughFile.sections.Count; i++)
            {
                if (inputFiles.throughFile.sections[0].records[0].KongouKeisu5
                    != inputFiles.throughFile.sections[i].records[0].KongouKeisu5)
                {
                    // INPUT DATA ERROR //////////////////////////////////////////////////////////////////////////////////////
                    throw new Exception("下流端に[混合係数 流れに挟まる]が指定されていないか本川と異なる値が指定されています 河川名: "
                        + inputFiles.throughFile.sections[i].records[0].RiverName);
                    //////////////////////////////////////////////////////////////////////////////////////////////////////////
                }
                for (int j = 1; j < inputFiles.throughFile.sections[i].records.Count; j++)
                {
                    if (0 < inputFiles.throughFile.sections[i].records[j].KongouKeisu5.Length)
                    {
                        // INPUT DATA ERROR /////////////////////////////////////////////////////////
                        throw new Exception("下流端以外に[混合係数 流れに挟まる]が指定されています 河川名: "
                            + inputFiles.throughFile.sections[i].records[j].RiverName
                            + " 断面ID: " + inputFiles.throughFile.sections[i].records[j].DanmenId);
                        /////////////////////////////////////////////////////////////////////////////
                    }
                }
            }

            ModelCalInfo.COMFFF_TAU[1] = V;     // 混合係数流れに挟まる

            if (!double.TryParse(inputFiles.throughFile.sections[0].records[0].KongouKeisu6, out V))
            {
                // INPUT DATA ERROR ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
                throw new Exception("下流端に[混合係数 流れに挟まれ水没]が指定されていません 河川名: " + inputFiles.throughFile.sections[0].records[0].RiverName);
                ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            }
            for (int i = 0; i < inputFiles.throughFile.sections.Count; i++)
            {
                if (inputFiles.throughFile.sections[0].records[0].KongouKeisu6
                    != inputFiles.throughFile.sections[i].records[0].KongouKeisu6)
                {
                    // INPUT DATA ERROR //////////////////////////////////////////////////////////////////////////////////////
                    throw new Exception("下流端に[混合係数 流れに挟まれ水没]が指定されていないか本川と異なる値が指定されています 河川名: "
                        + inputFiles.throughFile.sections[i].records[0].RiverName);
                    //////////////////////////////////////////////////////////////////////////////////////////////////////////
                }
                for (int j = 1; j < inputFiles.throughFile.sections[i].records.Count; j++)
                {
                    if (0 < inputFiles.throughFile.sections[i].records[j].KongouKeisu6.Length)
                    {
                        // INPUT DATA ERROR /////////////////////////////////////////////////////////
                        throw new Exception("下流端以外に[混合係数 流れに挟まれ水没]が指定されています 河川名: "
                            + inputFiles.throughFile.sections[i].records[j].RiverName
                            + " 断面ID: " + inputFiles.throughFile.sections[i].records[j].DanmenId);
                        /////////////////////////////////////////////////////////////////////////////
                    }
                }
            }

            ModelCalInfo.COMFFF_TAU[2] = V;     // 混合係数流れに挟まれ水没


        }


        /// <summary>
        /// 河道ネットワークの設定のログ出力
        /// </summary>
        void Log_NETWORK()
        {
            StreamWriter sw = new StreamWriter(Path.Combine(m_sWorkingDir, "01_River.csv"), false, System.Text.Encoding.GetEncoding("shift_jis"));
            sw.WriteLine("[本支川コード]");
            for (int i = 0; i < inputFiles.throughFile.sections.Count; i++)
            {
                sw.WriteLine(inputFiles.throughFile.sections[i].records[0].HonshiCode.ToString() + "," +
                    inputFiles.throughFile.sections[i].records[0].RiverName);
            }
            sw.WriteLine();

            sw.WriteLine("[断面インデックス]");
            sw.WriteLine("断面インデックス,河川名,TopoID,断面ID,累加距離,区間距離,河床勾配,湾曲曲率半径,橋脚形状係数,橋脚上流水路幅,橋脚幅を除いた水路幅,接続河川コード");
            for (int i = 0; i < ModelCalInfo.COMDAN_IDAN; i++)
            {
                sw.Write(i.ToString() + ",");                           // 断面インデックス
                sw.Write(ModelCalInfo.COMDAN_RIVR[i] + ",");            // 河川名
                sw.Write(ModelCalInfo.COMDAN_TOPO[i] + ",");　          // TopoID
                sw.Write(ModelCalInfo.COMDAN_ID[i] + ",");              // 断面ID
                sw.Write(ModelCalInfo.COMDAN_FL[i] + ",");              // 累加距離;
                sw.Write(ModelCalInfo.COMHAR_DX[i].ToString() + ",");   // 区間距離
                sw.Write(ModelCalInfo.COMTEN_GRD[i].ToString() + ",");  // 河床勾配
                sw.Write(ModelCalInfo.COMWAN_R[i].ToString() + ",");    // 湾曲曲率半径
                sw.Write(ModelCalInfo.COMBRD_CD[i].ToString() + ",");   // 橋脚形状係数
                sw.Write(ModelCalInfo.COMBRD_B1[i].ToString() + ",");   // 橋脚上流水路幅
                sw.Write(ModelCalInfo.COMBRD_B2[i].ToString() + ",");  // 橋脚幅を除いた水路幅
                sw.Write(ModelCalInfo.COMDAN_RIVC[i].ToString() + "\n");  // 接続河川コード
            }
            sw.WriteLine();

/*
            sw.WriteLine("[先頭断面インデックスと末尾断面インデックス]");
            for (int i = 0; i < ModelCalInfo.COMDAN_IRIV; i++)
            {
                sw.WriteLine(ModelCalInfo.COMDAN_RIVR[ModelCalInfo.COMDAN_IDS[i]] + "," + ModelCalInfo.COMDAN_IDS[i].ToString() + "," +
                    ModelCalInfo.COMDAN_IDE[i].ToString());
            }
            sw.WriteLine();

            sw.WriteLine("[接続本支川コード]");
            sw.WriteLine("断面インデックス,接続河川コード");
            for (int i = 0; i < ModelCalInfo.COMDAN_IDAN; i++)
            {
                sw.WriteLine(i.ToString() + "," + ModelCalInfo.COMDAN_RIVC[i].ToString());
            }
            sw.WriteLine();

            sw.WriteLine("[下流断面インデックス]");
            sw.WriteLine("断面インデックス,下流断面インデックス");
            for (int i = 0; i < ModelCalInfo.COMDAN_IDAN; i++)
            {
                sw.WriteLine(i.ToString() + "," + ModelCalInfo.COMDAN_IDL[i].ToString());
            }
            sw.WriteLine();
*/
            sw.WriteLine("[境界混合係数]");
            sw.WriteLine("混合係数急拡,混合係数b/B<0.6,混合係数0.6≦b/B,混合係数0.8＜b/B,混合係数堤防,混合係数流れに挟まる,混合係数流れに挟まれ水没");
            sw.Write(ModelCalInfo.COMFFF_TSD.ToString() + ",");
            for (int i = 0; i < 4; i++)
            {
                sw.Write(ModelCalInfo.COMFFF_FT[i].ToString() + ",");
            }

            for (int i = 0; i < 3; i++)
            {
                sw.Write(ModelCalInfo.COMFFF_TAU[i].ToString() + ",");
            }
            sw.WriteLine();
            sw.WriteLine();

            sw.WriteLine("[支川接続諸元]");
            sw.WriteLine("河川名,合流前河川幅B1,合流前河川幅B2,合流後河川幅B3,合流角度θ1,合流角度θ2");
            for (int i = 0; i < ModelCalInfo.COMDAN_IRIV; i++)
            {
                if (i == 0)
                {
                    continue;
                }
                sw.Write(ModelCalInfo.COMDAN_RIVR[ModelCalInfo.COMDAN_IDS[i]] + ",");
                sw.Write(ModelCalInfo.COMTBL_BB1[i].ToString() + ",");
                sw.Write(ModelCalInfo.COMTBL_BB2[i].ToString() + ",");
                sw.Write(ModelCalInfo.COMTBL_BB3[i].ToString() + ",");
                sw.Write(ModelCalInfo.COMTBL_ANG1[i].ToString() + ",");
                sw.Write(ModelCalInfo.COMTBL_ANG2[i].ToString() + "\n");
            }
            sw.Close();

        }

        #endregion

        // 断面設定
        #region SET_CROSSSECTION

        /// <summary>
        /// 断面分割、HARSの計算等の断面設定を行う。
        /// </summary>
        void Set_CROSSSECTION()
        {
            for (int i = 0; i < inputFiles.crossFile.sections.Count; i++)
            {
                CrossFileSection section = inputFiles.crossFile.sections[i];

                // 断面座標の設定
                double[] x = new double[section.records.Count];
                double[] y = new double[section.records.Count];
                int[] panel = new int[section.records.Count];
                double[] sodo = new double[section.records.Count];
                int[] sisui = new int[section.records.Count];
                for (int j = 0; j < section.records.Count; j++)
                {
                    double.TryParse(section.records[j].X, out x[j]);
                    double.TryParse(section.records[j].Z, out y[j]);
                    int.TryParse(section.records[j].Panel, out panel[j]);
                    double.TryParse(section.records[j].Roughness, out sodo[j]);
                    int.TryParse(section.records[j].Shisuiiki, out sisui[j]);
                }
                Log_CROSSSECTION_ORG(i, ref x, ref y, ref panel, ref sodo, ref sisui);

                // 断面座標の圧縮（近傍同士の測点をまとめる。）および急拡・急縮の死水域の設定
                Comp_XY(ref panel, ref x, ref y, ref sodo, ref sisui, i);
                Log_CROSSSECTION_COMP(i);

                // 樹木群による死水域をセットする。
                Set_TREE(i);
                Log_TREESET(i);

                // 粗度区分に従い断面分割を行う。
                Div_ROUGHNESS(i);
                Log_ROUGHNESSDIV(i);

                // 水位・河積・径深・潤辺テーブルを作成する。
                Set_HARS(i);
                Log_HARSSET(i);

                // 低水路幅、低水路平均河床高の設定
                Set_LOWERCHANNEL(i);
                Log_LOWERCHANNEL(i);

            }
        }

        /// <summary>
        /// オリジナルの断面データの読み取り結果をログ出力する。
        /// </summary>
        /// <param name="ID">断面インデックス</param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="panel"></param>
        /// <param name="sodo"></param>
        /// <param name="sisui"></param>
        void Log_CROSSSECTION_ORG(int ID, ref double[] x, ref double[] y, ref int[] panel, ref double[] sodo, ref int[] sisui)
        {
            string log_file = (Path.Combine(m_sWorkingDir, "03_Crosssection_orginal.csv"));
            if (ID == 0 && System.IO.File.Exists(log_file))
            {
                System.IO.File.Delete(log_file);
            }

            StreamWriter sw = new StreamWriter(log_file, true, System.Text.Encoding.GetEncoding("shift_jis"));
            sw.WriteLine("[start]");
            string river_name = ModelCalInfo.COMDAN_RIVR[ID];
            string topo_id = ModelCalInfo.COMDAN_TOPO[ID];
            string section_id = ModelCalInfo.COMDAN_ID[ID];
            sw.WriteLine(river_name + "," + topo_id + "," + section_id);
            for(int i = 0; i < x.Length; i++)
            {
                sw.WriteLine(x[i].ToString() + "," + y[i].ToString() + "," + panel[i].ToString() + "," +
                    sodo[i].ToString() + "," + sisui[i].ToString());
            }
            sw.WriteLine("[end]");
            sw.Close();
        }

        /// <summary>
        /// 断面座標の近傍同士の測点をまとめる（圧縮）。
        /// X座標が昇順になっているかをチェックする。
        /// 急拡・急縮による死水域の測点をカットし、パネル番号(1,4)を再設定
        /// <para name="panal">パネル番号</para>
        /// <param name="x">測点X座標（圧縮前）</param>
        /// <param name="y">測点Y座標（圧縮前）</param>
        /// <param name="sodo">粗度係数（圧縮前）</param>
        /// <param name="ID">河道断面インデックス</param>
        /// </summary>
        int Comp_XY(ref int[] panel, ref double[] x, ref double[] y, ref double[] sodo, ref int[] sisui, int ID)
        {
            int K;
            int IS = -1;
            int IE = -1;
            double EPS = 0.001;
            double DUMX;
            double DUMY;
            string riverName = inputFiles.crossFile.sections[ID].RiverName;
            string danmanID = inputFiles.crossFile.sections[ID].DanmenId;

            try
            {
                string log_file = Path.Combine(m_sWorkingDir, "05_Crosssection_convert.log");
                if(ID == 0 && System.IO.File.Exists(log_file))
                {
                    System.IO.File.Delete(log_file);
                }
                StreamWriter sw = new StreamWriter(log_file, true, System.Text.Encoding.GetEncoding("shift_jis"));

                for(int i = 0; i < y.Length; i++)
                {
                    if(panel[i] == 1) IS = i;
                    if(panel[i] == 4) IE = i;
                }

                // パネル番号1, 4が設定されていないときのエラ－処理
                if(IS == -1 || IE == -1)
                {
                    sw.WriteLine("【エラー】：" + riverName + "の断面" + danmanID + "にパネル番号が設定されていません。");
                    sw.Close();
                    return -1;
                }

                // 急拡・急縮による左右岸の死水域をカットし、パネル番号(1,4)を再セット。
                if(IE != IS)　// ←配列のインデックス・オーバの防ぐため
                {
                    int loop_count = 0;
                    // 左岸側死水域
                    for (int i = IS + 1; i <= IE; i++)
                    {
                        if (sisui[i] == 1)
                        {
                            loop_count++;
                            IS++;
                            panel[i] = 1;
                        }
                        else
                        {
                            if (loop_count > 0)
                            {
                                sw.WriteLine("[" + riverName + " " + danmanID + "]:" + "横断面の左岸側死水域を" + loop_count.ToString() + "測点カットしました。");
                            }
                            break;
                        }
                    }
                    loop_count = 0;
                    // 右岸側死水域
                    for (int i = IE; i >= IS + 1; i--)
                    {
                        if (sisui[i] == 1)
                        {
                            loop_count++;
                            IE--;
                            panel[i - 1] = 4;
                        }
                        else
                        {
                            if (loop_count > 0)
                            {
                                sw.WriteLine("["  + riverName + " " +  danmanID + "]:" + "横断面の右岸側死水域を" + loop_count.ToString() + "測点カットしました。");
                            }
                            break;
                        }
                    }
                }


                // 圧縮した測点データ等を計算用配列に代入する。
                K = 0;
                ModelCalInfo.COMXYZ_XZ[K, ID] = x[IS];
                ModelCalInfo.COMXYZ_YZ[K, ID] = y[IS];
                ModelCalInfo.COMXYZ_SOD[K, ID] = sodo[IS];
                ModelCalInfo.COMXYZ_IFG[K, ID] = panel[IS];
                for (int i = IS + 1; i <= IE; i++)
                {
                    // X座標が昇順に並んでいない時のエラー処理
                    if (x[i] < x[i - 1])
                    {
                        throw new Exception(riverName + "の断面" + danmanID + "のx座標が昇順に並んでいません（" +
                            x[i-1].ToString() + ", " + x[i].ToString() + ")。");
                    }
                    DUMX = Math.Abs(x[i] - ModelCalInfo.COMXYZ_XZ[K, ID]);
                    DUMY = Math.Abs(y[i] - ModelCalInfo.COMXYZ_YZ[K, ID]);
                    if (DUMX >= EPS || DUMY >= EPS)
                    {
                        K++;
                        ModelCalInfo.COMXYZ_XZ[K, ID] = x[i];
                        ModelCalInfo.COMXYZ_YZ[K, ID] = y[i];
                        ModelCalInfo.COMXYZ_SOD[K, ID] = sodo[i];
                        ModelCalInfo.COMXYZ_IFG[K, ID] = panel[i];

                    }
                    ModelCalInfo.COMXYZ_NDATA[ID] = K + 1;      
                    
                }
                

                if (y.Length != ModelCalInfo.COMXYZ_NDATA[ID])
                {
                    sw.Write("[" + riverName + " " + danmanID + "]:横断座標の近似点を圧縮しました。");
                    sw.WriteLine(string.Format("{0,5:D} ==> {1,5:D}", y.Length, ModelCalInfo.COMXYZ_NDATA[ID]));
                }

                if (K >= McQuasi2DSteadyFlowModelCalInfo.FLOW00_MD0)
                {
                    sw.WriteLine("[" + riverName + " " + danmanID + "]: 横断座標数が制限を越えています（" + ModelCalInfo.COMXYZ_NDATA[ID].ToString()
                        + "）。MAX=" + (McQuasi2DSteadyFlowModelCalInfo.FLOW00_MD0 - 1).ToString());
                    sw.Close();
                    return -3;
                }
                sw.Close();
                return 0;
            }

            catch
            {
                return -3;
            }
                 


 
        }

        /// <summary>
        /// 圧縮後の断面データをログ出力する。
        /// </summary>
        /// <param name="ID">断面インデックス</param>
        void Log_CROSSSECTION_COMP(int ID)
        {
            string log_file = (Path.Combine(m_sWorkingDir, "04_Crosssection_convert.csv"));
            if (ID == 0 && System.IO.File.Exists(log_file))
            {
                System.IO.File.Delete(log_file);
            }

            StreamWriter sw = new StreamWriter(log_file, true, System.Text.Encoding.GetEncoding("shift_jis"));
            sw.WriteLine("[start]");
            string river_name = ModelCalInfo.COMDAN_RIVR[ID];
            string topo_id = ModelCalInfo.COMDAN_TOPO[ID];
            string section_id = ModelCalInfo.COMDAN_ID[ID];
            sw.WriteLine(river_name + "," + topo_id + "," + section_id);

            int NDATA = ModelCalInfo.COMXYZ_NDATA[ID];
            for (int K = 0; K < NDATA; K++)
            {
                sw.WriteLine(ModelCalInfo.COMXYZ_XZ[K, ID].ToString() + "," + ModelCalInfo.COMXYZ_YZ[K, ID].ToString() + "," + 
                    ModelCalInfo.COMXYZ_IFG[K, ID].ToString() + "," + ModelCalInfo.COMXYZ_SOD[K, ID].ToString());
            }
            sw.WriteLine("[end]");
            sw.Close();
        }

        /// <summary>
        /// 樹木群による死水域をセットする。
        /// </summary>
        /// <para name="section">横断面データ</para>
        void Set_TREE(int ID)
        {
            CrossFileSection sec = inputFiles.crossFile.sections[ID];

            CrossFileRecord startrec = null;
            CrossFileRecord startrecp = null;

            double waribiki_ratio;
            double.TryParse(sec.getThroughRecord().section.records[0].WaribikiRitsuSo, out waribiki_ratio);
            if (waribiki_ratio < 0.0)
            {
                throw new Exception("割引率に負の値が設定されています。" + sec.getThroughRecord().section.records[0].RiverName);
            }
            double waribiki_factor =  1.0F - 0.01 * waribiki_ratio;

            double x_leftbank = ModelCalInfo.COMXYZ_XZ[0, ID];                                       // 左岸堤防肩X座標
            double x_rightbank = ModelCalInfo.COMXYZ_XZ[ModelCalInfo.COMXYZ_NDATA[ID] - 1, ID];    // 右岸堤防肩X座標

            List<double> center_pos = new List<double>();         // 樹木群センター位置
            List<double> left_pos = new List<double>();           // 樹木群右側位置
            List<double> right_pos = new List<double>();          // 樹木群左側位置
            List<double> tree_height = new List<double>();        // 樹木群高さ
            List<int> tree_density = new List<int>();             // 植生密度
            List<double> tree_width = new List<double>();         // 樹木群幅
            List<double> tree_top_height = new List<double>();    // 樹木樹冠標高

            // 上記リストに代入するための一次変数
            double left_pos_val;
            double right_pos_val;
            double tree_height_val;
            int tree_density_val;

            for (int ii = 0; ii < sec.records.Count; ii++)
            {
                CrossFileRecord rec = sec.records[ii];

                if (rec.EdashitaLength.Length > 0)
                {
                    throw new Exception("枝下長さは使用しませんが指定されています。 河川名: " + rec.section.RiverName + " 断面ID: " + rec.section.DanmenId);
                }

                if (rec.KiType.Length > 0)
                {

                    throw new Exception("樹タイプは使用しませんが指定されています。 河川名: " + rec.section.RiverName + " 断面ID: " + rec.section.DanmenId);
                }

                bool dataExists = false;
                if (rec.ShokuseiHeight.Length > 0)
                {
                    double dummy;
                    if (!double.TryParse(rec.ShokuseiHeight, out dummy))
                    {
                        throw new Exception("植生高さの値が不正です。 河川名: " + rec.section.RiverName + " 断面ID: " + rec.section.DanmenId);
                    }

                    if (dummy > 0.0)
                    {
                        if (0 < rec.ShokuseiDensity.Length && rec.ShokuseiDensity != "0" && rec.ShokuseiDensity != "1" && rec.ShokuseiDensity != "2")
                        {
                             throw new Exception("植生密度の値が不正です。 河川名: " + rec.section.RiverName + " 断面ID: " + rec.section.DanmenId);
                        }
                        dataExists = true;
                    }
                }


                if (dataExists)
                {

                    if (startrec == null)
                    {
                        startrecp = sec.records[0 < ii ? ii - 1 : 0];
                        startrec = rec;
                    }
                    else
                    {
                        bool samerec = (rec.ShokuseiHeight == startrec.ShokuseiHeight && rec.ShokuseiDensity == startrec.ShokuseiDensity);
                        if (!samerec)
                        {
                            double.TryParse(startrecp.X, out left_pos_val);
                            double.TryParse(sec.records[ii - 1].X, out right_pos_val);
                            double.TryParse(sec.records[ii - 1].ShokuseiHeight, out tree_height_val);
                            int.TryParse(sec.records[ii - 1].ShokuseiDensity, out tree_density_val);
                            left_pos.Add(left_pos_val);
                            right_pos.Add(right_pos_val);
                            tree_height.Add(tree_height_val);
                            tree_density.Add(tree_density_val);

                            startrecp = sec.records[0 < ii ? ii - 1 : 0];
                            startrec = rec;
                        }
                    }
                }
                else
                {
                    if (startrec != null)
                    {
                        double.TryParse(startrecp.X, out left_pos_val);
                        double.TryParse(sec.records[ii - 1].X, out right_pos_val);
                        double.TryParse(sec.records[ii - 1].ShokuseiHeight, out tree_height_val);
                        int.TryParse(sec.records[ii - 1].ShokuseiDensity, out tree_density_val);
                        left_pos.Add(left_pos_val);
                        right_pos.Add(right_pos_val);
                        tree_height.Add(tree_height_val);
                        tree_density.Add(tree_density_val);

                        startrecp = null;
                        startrec = null;
                    }
                }

            }
            if (startrec != null)
            {
                double.TryParse(startrecp.X, out left_pos_val);
                double.TryParse(sec.records[sec.records.Count - 1].X, out right_pos_val);
                double.TryParse(sec.records[sec.records.Count - 1].ShokuseiHeight, out tree_height_val);
                int.TryParse(sec.records[sec.records.Count - 1].ShokuseiDensity, out tree_density_val);
                left_pos.Add(left_pos_val);
                right_pos.Add(right_pos_val);
                tree_height.Add(tree_height_val);
                tree_density.Add(tree_density_val);
            }

            // 割引率を考慮する。
            for (int i = 0; i < left_pos.Count; i++)
            {

                if (tree_density[i] != 1)
                {
                    center_pos.Add(0.5 * (left_pos[i] + right_pos[i]));
                    tree_width.Add(right_pos[i] - left_pos[i]);
                }
                else
                {
                    // 左岸堤防に接する場合
                    if (left_pos[i] <= x_leftbank)
                    {
                        left_pos[i] = x_leftbank;
                        right_pos[i] = waribiki_factor * (right_pos[i] - left_pos[i]) + left_pos[i];
                        center_pos.Add(0.5 * (left_pos[i] + right_pos[i]));
                        tree_width.Add(right_pos[i] - left_pos[i]);
                    }
                    // 右岸堤防に接する場合
                    else if (right_pos[i] >= x_rightbank)
                    {
                        right_pos[i] = x_rightbank;
                        left_pos[i] = right_pos[i] - waribiki_factor * (right_pos[i] - left_pos[i]);
                        center_pos.Add(0.5 * (left_pos[i] + right_pos[i]));
                        tree_width.Add(right_pos[i] - left_pos[i]);
                    }
                    else
                    {
                        center_pos.Add(0.5 * (left_pos[i] + right_pos[i]));
                        tree_width.Add(waribiki_factor * (right_pos[i] - left_pos[i]));
                        left_pos[i] = center_pos[i] - 0.5 * tree_width[i];
                        right_pos[i] = center_pos[i] + 0.5 * tree_width[i];
                    }
                }
 
            }

            // 隣接する樹木群を統合する
            for (int i = 0; i < center_pos.Count - 1; i++)
            {
                double EPS = 1.0;   // 隣接の判定（1メートル以内は、接触していると判断する。
                if (Math.Abs(left_pos[i + 1] - right_pos[i]) <= EPS)
                {
                    tree_height[i] = tree_height[tree_width[i] > tree_width[i + 1] ? i : i + 1];  // 幅の広い方の樹木群高さに合わせる。
                    right_pos[i] = right_pos[i + 1];
                    center_pos[i] = 0.5 * (left_pos[i] + right_pos[i]);
                    tree_width[i] = right_pos[i] - left_pos[i];

                    // 統合された要素の削除
                    left_pos.Remove(left_pos[i + 1]);
                    right_pos.Remove(right_pos[i + 1]);
                    center_pos.Remove(center_pos[i + 1]);
                    tree_width.Remove(tree_width[i + 1]);
                    tree_height.Remove(tree_height[i + 1]);
                    i--;
                }
            }


            ModelCalInfo.COMTDB_TXX[ID] = new double[center_pos.Count];      // 樹木群横断位置（センター）
            for (int i = 0; i < center_pos.Count; i++)
            {
                ModelCalInfo.COMTDB_TXX[ID][i] = center_pos[i];
            }
            ModelCalInfo.COMTDB_TXL[ID] = new double[left_pos.Count];         // 樹木群横断位置（右端）
            for (int i = 0; i < left_pos.Count; i++)
            {
                ModelCalInfo.COMTDB_TXL[ID][i] = left_pos[i];
            }
            ModelCalInfo.COMTDB_TXR[ID] = new double[right_pos.Count];        // 樹木群横断位置（左端）
            for (int i = 0; i < right_pos.Count; i++)
            {
                ModelCalInfo.COMTDB_TXR[ID][i] = right_pos[i];
            }
            ModelCalInfo.COMTDB_TWW[ID] = new double[tree_width.Count];      // 樹木群幅
            for (int i = 0; i < tree_width.Count; i++)
            {
                ModelCalInfo.COMTDB_TWW[ID][i] = tree_width[i];
            }
            ModelCalInfo.COMTDB_THH[ID] = new double[tree_height.Count];     // 樹木群高さ
            for (int i = 0; i < tree_height.Count; i++)
            {
                ModelCalInfo.COMTDB_THH[ID][i] = tree_height[i];
            }

            ModelCalInfo.COMTDB_TTH[ID] = new double[center_pos.Count];       // 樹木群樹冠標高
            for (int i = 0; i < center_pos.Count; i++)
            {
                ModelCalInfo.COMTDB_TTH[ID][i] = tree_height[i] + 
                    Y_POS(ref ModelCalInfo.COMXYZ_XZ, ref ModelCalInfo.COMXYZ_YZ, ModelCalInfo.COMXYZ_NDATA[ID], center_pos[i], ID);
            }


            // 樹木群と河道断面がクロスしている場合の処理
            for (int i = 0; i < ModelCalInfo.COMTDB_TXX[ID].Length; i++)
            {
                double left_pos_new;
                double right_pos_new;
                Tree_CROSS(ref ModelCalInfo.COMXYZ_XZ, ref ModelCalInfo.COMXYZ_YZ, ModelCalInfo.COMXYZ_NDATA[ID],
                    left_pos[i], right_pos[i], ModelCalInfo.COMTDB_TTH[ID][i], out left_pos_new, out right_pos_new, ID);
                if (left_pos_new != -999.0)
                {
                    ModelCalInfo.COMTDB_TXL[ID][i] = left_pos_new;
                }
                if (right_pos_new != -999.0)
                {
                    ModelCalInfo.COMTDB_TXR[ID][i] = right_pos_new;
                }
                ModelCalInfo.COMTDB_TXX[ID][i] = 0.5 * (ModelCalInfo.COMTDB_TXL[ID][i] + ModelCalInfo.COMTDB_TXR[ID][i]);
            }

        }

        /// <summary>
        /// X座標位置に対するY座標を補間した値を返す
        /// </summary>
        /// <param name="XX">入力値用テーブル</param>
        /// <param name="YY">出力値用テーブル</param>
        /// <param name="NDATA">テーブルレコード数</param>
        /// <param name="X">入力値</param>
        /// <param name="ID">断面番号</param>
        private double Y_POS(ref double[,] XX, ref double[,] YY, int NDATA, double X, int ID)
        {
            double COEF;
            double DUM;
            double Y = 0.0F;
            double ERR = -999.0;

            for (int I = 1; I < NDATA; I++)
            {
                DUM = (XX[I, ID] - X) * (X - XX[I - 1, ID]);
                if (DUM >= 0.0F)
                {
                    if (XX[I, ID] - XX[I - 1, ID] == 0.0F)
                    {
                        Y = YY[I - 1, ID];
                    }
                    else
                    {
                        COEF = (X - XX[I - 1, ID]) / (XX[I, ID] - XX[I - 1, ID]);
                        Y = YY[I - 1, ID] + (YY[I, ID] - YY[I - 1, ID]) * COEF;
                    }
                    return Y;
                }
            }
            return ERR;

        }

        /// <summary>
        /// 樹木と河道断面の交点を求める
        /// </summary>
        /// <param name="XZ">X座標</param>
        /// <param name="YZ">Y座標</param>
        /// <param name="NDATA">座標数</param>
        /// <param name="XS">植生左</param>
        /// <param name="XE">植生右</param>
        /// <param name="YY">植生樹冠標高</param>
        /// <param name="XCL">交点X座標右側</param>
        /// <param name="XCL">交点X座標左側</param>
        /// <param name="ID">断面インデックス</param>
        private void Tree_CROSS(ref double[,] XZ, ref double[,] YZ, int NDATA, double XS,  double XE,  double HH, out double XCL, out double XCR,  int ID)
        {
            double PLUS;
            double COEF;
            double DUM;
            double XXC = 0.5 * (XS + XE);      // 樹木群センター位置X座標
            double XX;

            // クロスしていない場合は、-999を返す。    
            XCL = -999.0;
            XCR = -999.0;

            for (int I = 1; I < NDATA; I++)
            {
                PLUS = (YZ[I, ID] - HH) * (HH - YZ[I - 1, ID]);
                if (PLUS > 0.0)
                {
                    COEF = (HH - YZ[I - 1, ID]) / (YZ[I, ID] - YZ[I - 1, ID]);
                    XX = XZ[I - 1, ID] + (XZ[I, ID] - XZ[I - 1, ID]) * COEF;
                    DUM = (XE - XX) * (XX - XS);
                    if (DUM > 0.0)
                    {
                        if (XX < XXC)
                        {
                            XCL = XX;
                        }
                        else
                        {
                            XCR = XX;
                        }
                    }
                }
            }
        }


        /// <summary>
        /// 樹木群の設定をログ出力する。
        /// </summary>
        /// <param name="ID"></param>
        void Log_TREESET(int ID)
        {
            StreamWriter sw;
            string log_file = Path.Combine(m_sWorkingDir, "02_Tree.csv");
            if(ID == 0)
            {
                System.IO.File.Delete(log_file);
                sw = new StreamWriter(log_file, false, System.Text.Encoding.GetEncoding("shift_jis"));
                sw.WriteLine("[樹木諸元]");
                sw.WriteLine("断面インデックス,河川名,断面ID,樹木群左端座標,樹木群中心座標,樹木群右端座標,樹木群幅,樹高,樹冠標高");
                sw.Close();
                return;
            }

            sw = new StreamWriter(log_file, true, System.Text.Encoding.GetEncoding("shift_jis"));
            for (int j = 0; j < ModelCalInfo.COMTDB_TXX[ID].Length; j++)
            {
                sw.WriteLine(ID.ToString() + "," + ModelCalInfo.COMDAN_RIVR[ID] + "," + ModelCalInfo.COMDAN_ID[ID] + "," + 
                    ModelCalInfo.COMTDB_TXL[ID][j].ToString() + "," + ModelCalInfo.COMTDB_TXX[ID][j].ToString() + "," + 
                    ModelCalInfo.COMTDB_TXR[ID][j].ToString() + "," + ModelCalInfo.COMTDB_TWW[ID][j].ToString() + "," + 
                    ModelCalInfo.COMTDB_THH[ID][j].ToString() + "," + ModelCalInfo.COMTDB_TTH[ID][j].ToString());
            }
            sw.Close();
        }

        /// <summary>
        /// 粗度区分に従い断面分割する。
        /// </summary>
        void Div_ROUGHNESS(int ID)
        {
            List<double> rough = new List<double>(); // 粗度分割区分の粗度
            List<int> div_node = new List<int>();    // 粗度分割区分の境界測点のインデックス（右側）
            List<bool> isTree = new List<bool>();    // 粗度分割区分の樹木か河道かの別（true: 樹木, false: 河道）

            // 樹木の境界と断面測点をすり合わせる。
            for (int i = 0; i < ModelCalInfo.COMTDB_TXX[ID].Length; i++)
            {
                for (int j = 0; j < ModelCalInfo.COMXYZ_NDATA[ID] - 1; j++)
                {
                    double x_L = ModelCalInfo.COMTDB_TXL[ID][i];
                    double in_out = (x_L - ModelCalInfo.COMXYZ_XZ[j, ID]) * (ModelCalInfo.COMXYZ_XZ[j + 1, ID] - x_L);
                    if (in_out > 0.0F)
                    {
                        // 測点を追加する。
                        Insert_NNode(x_L, j + 1, ID);
                    }

                    double x_R = ModelCalInfo.COMTDB_TXR[ID][i];
                    in_out = (x_R - ModelCalInfo.COMXYZ_XZ[j, ID]) * (ModelCalInfo.COMXYZ_XZ[j + 1, ID] - x_R);
                    if (in_out > 0.0F)
                    {
                        // 測点を追加する。
                        Insert_NNode(x_R, j + 1, ID);
                    }
                }
            }

            // 粗度区分を測点に合わせて分割する。
            bool in_tree = false;   // カーソルが樹木群の中にあるとき真
            bool tree_skip = false; // 樹木を無視する場合は、真
            int ii = 0;  // 樹木群インデックス
            for (int i = 1; i < ModelCalInfo.COMXYZ_NDATA[ID] - 1; i++)
            {
                // 樹木群がある場合
                if (ModelCalInfo.COMTDB_TXX[ID].Length != 0)
                {
                    if (!tree_skip)
                    {

                        if (ModelCalInfo.COMXYZ_XZ[i, ID] >= ModelCalInfo.COMTDB_TXL[ID][ii] &&
                            ModelCalInfo.COMXYZ_XZ[i, ID] <= ModelCalInfo.COMTDB_TXR[ID][ii])
                        {
                            in_tree = true;
                        }
                        else
                        {
                            in_tree = false;
                        }

                        // 樹木群による粗度分割区間の追加
                        if (!in_tree)
                        {
                            if (ModelCalInfo.COMXYZ_XZ[i, ID] <= ModelCalInfo.COMTDB_TXL[ID][ii] &&
                                ModelCalInfo.COMXYZ_XZ[i + 1, ID] >= ModelCalInfo.COMTDB_TXL[ID][ii])
                            {
                                div_node.Add(i + 1);
                                rough.Add(ModelCalInfo.COMXYZ_SOD[i + 1, ID]);
                                isTree.Add(false);
                                in_tree = true;
                            }
                        }
                        else
                        {
                            if (ModelCalInfo.COMXYZ_XZ[i, ID] <= ModelCalInfo.COMTDB_TXR[ID][ii] &&
                                ModelCalInfo.COMXYZ_XZ[i + 1, ID] >= ModelCalInfo.COMTDB_TXR[ID][ii])
                            {
                                div_node.Add(i + 1);
                                rough.Add(ModelCalInfo.COMFFF_TSD);
                                isTree.Add(true);
                                in_tree = false;
                                // iiのインクリメントによる配列範囲外エラーを避ける処理
                                if (ii >= ModelCalInfo.COMTDB_TXX[ID].Length - 1)
                                {
                                    tree_skip = true;
                                }
                                else
                                {
                                    ii++;
                                }
                                continue;
                            }
                        }
                    }

                    // 河道断面粗度の変わり目の測点を探す。
                    if (in_tree)
                    {
                        continue;
                    }

                    if (ModelCalInfo.COMXYZ_SOD[i, ID] != ModelCalInfo.COMXYZ_SOD[i + 1, ID])
                    {
                        div_node.Add(i);
                        rough.Add(ModelCalInfo.COMXYZ_SOD[i, ID]);
                        isTree.Add(false);
                    }
                }
                // 樹木群がない場合
                else
                {
                    if (ModelCalInfo.COMXYZ_SOD[i, ID] != ModelCalInfo.COMXYZ_SOD[i + 1, ID])
                    {
                        div_node.Add(i);
                        rough.Add(ModelCalInfo.COMXYZ_SOD[i, ID]);
                        isTree.Add(false);
                    }
                }

            }
            // 河道断面右端の測点を加える。
            div_node.Add(ModelCalInfo.COMXYZ_NDATA[ID] - 1);
            rough.Add(ModelCalInfo.COMXYZ_SOD[ModelCalInfo.COMXYZ_NDATA[ID] - 1, ID]);
            if (ModelCalInfo.COMTDB_TXX[ID].Length != 0)
            {
                if (ModelCalInfo.COMTDB_TXR[ID][ModelCalInfo.COMTDB_TXR[ID].Length - 1] >= ModelCalInfo.COMXYZ_XZ[ModelCalInfo.COMXYZ_NDATA[ID] - 1, ID])
                {
                    isTree.Add(true);
                }
                else
                {
                    isTree.Add(false);
                }
            }
            else
            {
                isTree.Add(false);
            }

            // 境界区分の測点インデックスの重複を削除
            if (div_node.Count > 1)
            {
                for (int i = 0; i < div_node.Count - 1; i++)
                {
                    if (div_node[i] == div_node[i + 1])
                    {
                        div_node.RemoveAt(i + 1);
                        rough.RemoveAt(i + 1);
                        isTree.RemoveAt(i + 1);
                    }
                }
            }


            // 分割区分の粗度と測点インデックス、樹木・河道の別を配列に格納する。
            ModelCalInfo.COSODO_FNN[ID] = new double[rough.Count];
            for (int k = 0; k < rough.Count; k++)
            {
                ModelCalInfo.COSODO_FNN[ID][k] = rough[k];
            }
            ModelCalInfo.COSODO_IDX[ID] = new int[div_node.Count];
            for (int k = 0; k < div_node.Count; k++)
            {
                ModelCalInfo.COSODO_IDX[ID][k] = div_node[k];
            }
            ModelCalInfo.COSODO_ISTREE[ID] = new bool[isTree.Count];
            for (int k = 0; k < isTree.Count; k++)
            {
                ModelCalInfo.COSODO_ISTREE[ID][k] = isTree[k];
            }

            // 樹冠標高、樹高、樹木群幅を断面ブロック・インデックスで参照できる配列を作成
            ModelCalInfo.COSODO_THH[ID] = new double[isTree.Count];
            ModelCalInfo.COSODO_TTH[ID] = new double[isTree.Count];
            ModelCalInfo.COSODO_BW[ID] = new double[isTree.Count];

            int N_TREE = 0;     // 樹木群インデックス
            for(int k = 0; k < isTree.Count; k++)
            {
                if(!ModelCalInfo.COSODO_ISTREE[ID][k])
                {
                    ModelCalInfo.COSODO_THH[ID][k] = -999.0;
                    ModelCalInfo.COSODO_TTH[ID][k] = -999.0;
                    ModelCalInfo.COSODO_BW[ID][k] = -999.0;
                }
                else
                {
                    ModelCalInfo.COSODO_THH[ID][k] = ModelCalInfo.COMTDB_TTH[ID][N_TREE];
                    ModelCalInfo.COSODO_TTH[ID][k] = ModelCalInfo.COMTDB_THH[ID][N_TREE];
                    ModelCalInfo.COSODO_BW[ID][k] = ModelCalInfo.COMTDB_TWW[ID][N_TREE];
                    N_TREE++;
                }
            }

            // 断面分割数を参照できる配列に格納する。
            ModelCalInfo.COMHAR_NBUN[ID] = ModelCalInfo.COSODO_IDX[ID].Length;
        }

        /// <summary>
        /// 断面に測点を挿入する。
        /// </summary>
        /// <param name="x">挿入する箇所のX座標</param>
        /// <param name="n">挿入する箇所の測点インデックス</param>
        /// <param name="secID">挿入する箇所の断面インデックス</param>
        void Insert_NNode(double x, int n, int ID)
        {
            int NDATA = ModelCalInfo.COMXYZ_NDATA[ID];
            // 挿入する座標を直線補間して求める。
            double y = Y_POS(ref ModelCalInfo.COMXYZ_XZ, ref ModelCalInfo.COMXYZ_YZ, NDATA, x, ID);

            // 挿入する測点より右側の測点を１つ右側にずらし、挿入点をあける。
            for (int i = NDATA; i > n; i--)
            {
                ModelCalInfo.COMXYZ_XZ[i, ID] = ModelCalInfo.COMXYZ_XZ[i - 1, ID];
                ModelCalInfo.COMXYZ_YZ[i, ID] = ModelCalInfo.COMXYZ_YZ[i - 1, ID];
                ModelCalInfo.COMXYZ_SOD[i, ID] = ModelCalInfo.COMXYZ_SOD[i - 1, ID];
                ModelCalInfo.COMXYZ_IFG[i, ID] = ModelCalInfo.COMXYZ_IFG[i - 1, ID];
            }
            // 測点を挿入する。
            ModelCalInfo.COMXYZ_XZ[n, ID] = x;
            ModelCalInfo.COMXYZ_YZ[n, ID] = y;
            ModelCalInfo.COMXYZ_SOD[n, ID] = ModelCalInfo.COMXYZ_SOD[n - 1, ID];

            // 測点数を一つ増やす。
            ModelCalInfo.COMXYZ_NDATA[ID]++;
        }

        /// <summary>
        /// 粗度区分による断面分割のログ出力
        /// </summary>
        /// <param name="ID"></param>
        void Log_ROUGHNESSDIV(int ID)
        {
            string log_file = Path.Combine(m_sWorkingDir, "06_Crosssection_block.csv");
            if (ID == 0 && System.IO.File.Exists(log_file))
            {
                System.IO.File.Delete(log_file);
            }
            StreamWriter sw = new StreamWriter(log_file, true, System.Text.Encoding.GetEncoding("shift_jis"));
            sw.WriteLine("[断面分割毎の諸元]");
            sw.WriteLine(ModelCalInfo.COMDAN_RIVR[ID] + " " + ModelCalInfo.COMDAN_ID[ID] + "," + "測点数（" + ModelCalInfo.COMXYZ_NDATA[ID].ToString() + "）");
            sw.Write("測点インデックス（右側）,");
            for (int j = 0; j < ModelCalInfo.COSODO_IDX[ID].Length; j++)
            {
                sw.Write(ModelCalInfo.COSODO_IDX[ID][j].ToString() + ",");
            }
            sw.WriteLine();
            sw.Write("粗度,");
            for (int j = 0; j < ModelCalInfo.COSODO_IDX[ID].Length; j++)
            {
                sw.Write(ModelCalInfo.COSODO_FNN[ID][j].ToString() + ",");

            }
            sw.WriteLine();
            sw.Write("測点座標（右側）,");
            for (int j = 0; j < ModelCalInfo.COSODO_IDX[ID].Length; j++)
            {
                sw.Write(ModelCalInfo.COMXYZ_XZ[ModelCalInfo.COSODO_IDX[ID][j], ID].ToString() + ",");
            }
            sw.WriteLine();
            sw.Write("ISTREE,");
            for (int j = 0; j < ModelCalInfo.COSODO_IDX[ID].Length; j++)
            {
                sw.Write(ModelCalInfo.COSODO_ISTREE[ID][j].ToString() + ",");
            }
            sw.WriteLine();
            sw.Write("樹冠標高,");
            for (int j = 0; j < ModelCalInfo.COSODO_IDX[ID].Length; j++)
            {
                sw.Write(ModelCalInfo.COSODO_TTH[ID][j].ToString() + ",");
            }
            sw.WriteLine();
            sw.Write("樹木群樹高,");
            for (int j = 0; j < ModelCalInfo.COSODO_IDX[ID].Length; j++)
            {
                sw.Write(ModelCalInfo.COSODO_THH[ID][j].ToString() + ",");
            }
            sw.WriteLine();
            sw.Write("樹木群幅,");
            for (int j = 0; j < ModelCalInfo.COSODO_IDX[ID].Length; j++)
            {
                sw.Write(ModelCalInfo.COSODO_BW[ID][j].ToString("0.000") + ",");
            }
            sw.WriteLine();
            sw.WriteLine();
            sw.Close();
        }

        /// <summary>
        /// 水位・河積・径深・潤辺テーブルを作成する。
        /// </summary>
        /// <param name="secID">断面インデックス</param>
        void Set_HARS(int ID)
        {
            double Zmin = ModelCalInfo.COMXYZ_YZ[0, ID];
            for (int i = 0; i < ModelCalInfo.COMXYZ_NDATA[ID]; i++)
            {
                Zmin = Math.Min(Zmin, ModelCalInfo.COMXYZ_YZ[i, ID]);
            }

            // 水位テーブルの作成
            for (int i = 0; i < McQuasi2DSteadyFlowModelCalInfo.FLOW00_MD1; i++)
            {
                ModelCalInfo.COMHAR_HH[i, ID] = Zmin + 0.5 * (double)i;
            }

            // 河積、径深、潤辺、水面幅テーブルの作成
            double AAA;
            double RRR;
            double SSS;
            double WWW;


            for (int j = 0; j < ModelCalInfo.COSODO_IDX[ID].Length; j++)     // 分割断面インデックス
            {
                for (int k = 0; k < McQuasi2DSteadyFlowModelCalInfo.FLOW00_MD1; k++)   // 水位インデックス
                {
                    HARP_HWL(ID, j, ModelCalInfo.COMHAR_HH[k, ID], out AAA, out RRR, out SSS, out WWW);
                    ModelCalInfo.COMHAR_AA[k, j, ID] = AAA;
                    ModelCalInfo.COMHAR_RR[k, j, ID] = RRR;
                    ModelCalInfo.COMHAR_SS[k, j, ID] = SSS;
                    ModelCalInfo.COMHAR_BL[k, j, ID] = WWW;
                }
            }

        }

        /// <summary>
        /// 当該水位における分割断面ごとの河積・潤辺・径深・水面幅を求める
        /// </summary>
        /// <param name="ID">断面インデックス</param>
        /// <param name="IBUN">分割断面インデックス</param>
        /// <param name="HWL">水位(m)</param>
        /// <param name="AAA">河積(m2)</param>
        /// <param name="RRR">径深(m)</param>
        /// <param name="SSS">潤辺(m)</param>
        /// <param name="WWW">水面幅(m)</param>
        void HARP_HWL(int ID, int IB, double HWL, out double AAA, out double RRR, out double SSS, out double WWW)
        {
            // 分割断面が樹木群でない場合
            if (!ModelCalInfo.COSODO_ISTREE[ID][IB])
            {
                double Y1;
                double Y2;
                double YL;
                double PLUS;
                double COEF;
                double WW;
                double HH;
                double AA;
                double SS;
                double X = 0;
                int NS = (IB == 0) ? 0 : ModelCalInfo.COSODO_IDX[ID][IB - 1];    // 測点の先頭インデックス
                int NE = ModelCalInfo.COSODO_IDX[ID][IB];                          // 測点の末尾インデックス
                int NDATA = ModelCalInfo.COMXYZ_NDATA[ID];


                // 注 =================================================
                //      オバーハングは不可   Xn+1 ≧ Xn  であること。
                //      このルーティンではチェックを行なわない。
                //    =================================================
                AAA = 0.0;
                SSS = 0.0;
                WWW = 0.0;
                RRR = 0.0;

                for (int L = NS + 1; L <= NE; L++)
                {
                    Y1 = ModelCalInfo.COMXYZ_YZ[L - 1, ID];
                    Y2 = ModelCalInfo.COMXYZ_YZ[L, ID];
                    YL = Math.Min(Y1, Y2);
                    PLUS = (Y2 - HWL) * (HWL - Y1);

                    if (PLUS > 0.0)
                    {
                        //交差
                        COEF = (HWL - Y1) / (Y2 - Y1);
                        X = ModelCalInfo.COMXYZ_XZ[L - 1, ID] + (ModelCalInfo.COMXYZ_XZ[L, ID] - ModelCalInfo.COMXYZ_XZ[L - 1, ID]) * COEF;
                        if (Y2 > Y1)
                        {
                            WW = X - ModelCalInfo.COMXYZ_XZ[L - 1, ID];
                            HH = HWL - Y1;
                        }
                        else
                        {
                            WW = ModelCalInfo.COMXYZ_XZ[L, ID] - X;
                            HH = HWL - Y2;
                        }
                        AA = WW * HH * 0.5;
                        SS = Math.Sqrt(WW * WW + HH * HH);
                    }
                    else
                    {
                        //交差しない
                        if (HWL > YL)
                        {
                            WW = ModelCalInfo.COMXYZ_XZ[L, ID] - ModelCalInfo.COMXYZ_XZ[L - 1, ID];
                            SS = Math.Sqrt(WW * WW + (Y2 - Y1) * (Y2 - Y1));
                            AA = ((HWL - Y1) + (HWL - Y2)) * WW * 0.5;
                        }
                        else
                        {
                            WW = 0.0F;
                            AA = 0.0F;
                            SS = 0.0F;
                        }
                    }
                    AAA += AA;
                    SSS += SS;
                    WWW += WW;
                }

                if (AAA < 0.0)
                {
                    AAA = 0.0F;
                }
                if (SSS < 0.0)
                {
                    SSS = 0.0F;
                }
                if (WWW < 0.0)
                {
                    WWW = 0.0F;
                }

                if (SSS > 0.0)
                {
                    RRR = AAA / SSS;
                }
                return;
            }
            // 分割断面が樹木群の場合
            else
            {
                int NS = (IB == 0) ? 0 : ModelCalInfo.COSODO_IDX[ID][IB - 1];       // 測点の先頭インデックス
                int NE = ModelCalInfo.COSODO_IDX[ID][IB];                             // 測点の末尾インデックス
                double X_L = ModelCalInfo.COMXYZ_XZ[NS, ID];                             // 分割断面左端X座標
                double X_R = ModelCalInfo.COMXYZ_XZ[NE, ID];                             // 分割断面右端X座標
                double TTH = 0.0F;                                                       // 樹冠標高
                int N_TREE = 0;                                                         // 樹木群インデックス

                for (int i = 0; i < ModelCalInfo.COMTDB_TTH[ID].Length; i++)
                {
                    if (X_L < ModelCalInfo.COMTDB_TXX[ID][i] && ModelCalInfo.COMTDB_TXX[ID][i] < X_R)
                    {
                        TTH = ModelCalInfo.COMTDB_TTH[ID][i];
                        N_TREE = i;
                    }
                }

                WWW = ModelCalInfo.COMTDB_TWW[ID][N_TREE];

                if (HWL > TTH)
                {
                    AAA = WWW * (HWL - TTH);
                    SSS = WWW;
                    RRR = AAA / SSS;
                }
                else
                {
                    AAA = 0.0F;
                    SSS = 0.0F;
                    RRR = 0.0F;
                    WWW = 0.0F;
                }
            }
        }

        /// <summary>
        /// 水位・河積・径深・潤辺・水面幅のテーブルをログ出力する。
        /// </summary>
        /// <param name="ID"></param>
        void Log_HARSSET(int ID)
        {
            string log_file = Path.Combine(m_sWorkingDir, "07_HARSB.csv");
            if (ID == 0 && System.IO.File.Exists(log_file))
            {
                System.IO.File.Delete(log_file);
            }
            StreamWriter sw = new StreamWriter(log_file, true, System.Text.Encoding.GetEncoding("shift_jis"));
            sw.WriteLine(ModelCalInfo.COMDAN_RIVR[ID] + " " +  ModelCalInfo.COMDAN_ID[ID]);

            for (int k = 0; k < ModelCalInfo.COSODO_IDX[ID].Length; k++)
            {
                sw.WriteLine("分割断面N：," + k.ToString());
                sw.Write("水位H：,");
                for (int j = 0; j < McQuasi2DSteadyFlowModelCalInfo.FLOW00_MD1; j++)
                {
                    sw.Write(ModelCalInfo.COMHAR_HH[j, ID].ToString() + ",");
                }
                sw.WriteLine();
                sw.Write("河積A：,");
                for (int j = 0; j < McQuasi2DSteadyFlowModelCalInfo.FLOW00_MD1; j++)
                {
                    sw.Write(ModelCalInfo.COMHAR_AA[j, k, ID].ToString() + ",");
                }
                sw.WriteLine();
                sw.Write("径深R：,");
                for (int j = 0; j < McQuasi2DSteadyFlowModelCalInfo.FLOW00_MD1; j++)
                {
                    sw.Write(ModelCalInfo.COMHAR_RR[j, k, ID].ToString() + ",");
                }
                sw.WriteLine();
                sw.Write("潤辺S：,");
                for (int j = 0; j < McQuasi2DSteadyFlowModelCalInfo.FLOW00_MD1; j++)
                {
                    sw.Write(ModelCalInfo.COMHAR_SS[j, k, ID].ToString() + ",");
                }
                sw.WriteLine();
                sw.Write("水面幅B：,");
                for (int j = 0; j < McQuasi2DSteadyFlowModelCalInfo.FLOW00_MD1; j++)
                {
                    sw.Write(ModelCalInfo.COMHAR_BL[j, k, ID].ToString() + ",");
                }
                sw.WriteLine();
                sw.WriteLine();
            }
            sw.WriteLine();
            sw.Close();
        }

        
        /// <summary>
        /// 低水路幅、低水路満杯水位、低水路平均河床高をセットする。
        /// </summary>
        /// <param name="ID"></param>
        void Set_LOWERCHANNEL(int ID)
        {

            double ini = 9999999.9;
            double y1 = ini;
            double y2 = ini;
            double y3 = ini;
            double y4 = ini;

            // エラーログ
            string log_file = (Path.Combine(m_sWorkingDir, "Err_LowerChannel.log"));
            if (ID == 0 && System.IO.File.Exists(log_file))
            {
                System.IO.File.Delete(log_file);
            }
            StreamWriter sw;
            int NDATA = ModelCalInfo.COMXYZ_NDATA[ID];
            for (int i = 0; i < NDATA; i++)
            {
                if (ModelCalInfo.COMXYZ_IFG[i, ID] == 1)
                {
                    y1= ModelCalInfo.COMXYZ_YZ[i, ID];
                }
                if (ModelCalInfo.COMXYZ_IFG[i, ID] == 2)
                {
                    y2 = ModelCalInfo.COMXYZ_YZ[i, ID];
                }
                if (ModelCalInfo.COMXYZ_IFG[i, ID] == 3)
                {
                    y3 = ModelCalInfo.COMXYZ_YZ[i, ID];
                }
                if(ModelCalInfo.COMXYZ_IFG[i, ID] == 4)
                {
                    y4 = ModelCalInfo.COMXYZ_YZ[i, ID];
                }
            }
            
            // 低水路満杯水位の設定
            if (y2 != ini && y3 != ini)
            {
                ModelCalInfo.COMXYZ_HHL[ID] = Math.Min(y2, y3);
            }
            else
            // パネル2,3が設定されていないときは、パネル1,4を代用
            {
                ModelCalInfo.COMXYZ_HHL[ID] = Math.Min(y1, y4);
            }

            // 低水路平均河床高の設定
            int NBUN = ModelCalInfo.COMHAR_NBUN[ID];
            double HHL = ModelCalInfo.COMXYZ_HHL[ID];
            double AA;
            double BB;
            double DUM;
            double SumAA = 0.0;
            double SumBB = 0.0;
            for (int IB = 0; IB < NBUN; IB++)
            {
                HARP_HWL(ID, IB, HHL, out AA, out DUM, out DUM, out BB);
                SumAA += AA;
                SumBB += BB;
            }
            if (SumBB <= 0.0)
            {
                ModelCalInfo.COMXYZ_BBL[ID] = 0.0;
                sw = new StreamWriter(log_file, true, System.Text.Encoding.GetEncoding("shift_jis"));
                sw.WriteLine(ModelCalInfo.COMDAN_RIVR[ID] + " " + ModelCalInfo.COMDAN_ID[ID] + "において、低水路幅が設定されてません。");
                sw.Close();
            }
            else
            {
                ModelCalInfo.COMXYZ_BBL[ID] = SumBB;
                ModelCalInfo.COMXYZ_BED[ID] = HHL - SumAA / SumBB;
            }
 
        }

        /// <summary>
        /// 低水路幅、低水路満杯水位、低水路平均河床高の設定結果をログ出力する。
        /// </summary>
        /// <param name="ID"></param>
        void Log_LOWERCHANNEL(int ID)
        {
            StreamWriter sw;
            string log_file = Path.Combine(m_sWorkingDir, "08_LowerChannel.csv");
            if (ID == 0)
            {
                sw = new StreamWriter(log_file, false, System.Text.Encoding.GetEncoding("shift_jis"));
                sw.WriteLine("断面インデックス,河川名,断面ID,低水路幅,低水路満杯水位,低水路平均河床高");
                sw.Close();
                return;
            }
            sw = new StreamWriter(log_file, true, System.Text.Encoding.GetEncoding("shift_jis"));
            sw.WriteLine(ID.ToString() + "," + ModelCalInfo.COMDAN_RIVR[ID] + "," + ModelCalInfo.COMDAN_ID[ID] + "," +
                ModelCalInfo.COMXYZ_BBL[ID].ToString() + "," + ModelCalInfo.COMXYZ_HHL[ID].ToString() + "," +
                ModelCalInfo.COMXYZ_BED[ID].ToString());
            sw.Close();
        }

        #endregion

        // 水位計算
        #region Calc_HWL

        /// <summary>
        /// 水位計算を行う。
        /// </summary>
        /// <returns></returns>
        bool Calc_HWL()
        {
            // 断面流量のセット
            Set_QQ();

            // 本川出発水位の設定
            double.TryParse(inputFiles.throughFile.sections[0].records[0].WaterLevel, out ModelCalInfo.COMCAL_CAL[0]);

            double H0;       // 下流断面水位
            double H1;       // 当該断面水位

            double EGrad = 0.0;      // エネルギー勾配（初期値）
            double DGrad = 0.0;      // エネルギー勾配（収束確認用）

            double QRatio;           // 流量比（収束確認用）

            double E0;               // 下流断面速度水頭
            double E1;               // 下流断面底面摩擦
            double E2;               // 下流断面樹木摩擦

            double E00;              // 当該断面速度水頭
            double E11;              // 当該断面底面摩擦
            double E22;              // 当該断面樹木摩擦

            double FS0;              // 下流断面水頭
            double FS1;              // 当該断面水頭

            double DX;               // 区間距離

            int ID0 = 0;    // 下流断面インデックス

            for (int IR = 0; IR < ModelCalInfo.COMDAN_IRIV; IR++)
            {
                int NS = ModelCalInfo.COMDAN_IDS[IR];
                int NE = ModelCalInfo.COMDAN_IDE[IR];

                // 不等流水位収束判定変数
                double FS_DIF0 = 0.0;       // 初期水頭差
                double Hold = 0.0;          // 収束計算用水位
                double Hnew = 0.0;          // 収束計算用水位
                int ConvStep = 0;           // 収束計算ステップ
                
                H0 = ModelCalInfo.COMCAL_CAL[NS];
                for (int ID = NS; ID <= NE; ID++)
                {
                    // 最下流端断面の水位設定
                    if (ID == 0)
                    {

                        // エネルギー勾配初期値計算（上流断面の値を代用）
                        H1 = H0 + 0.5F;
                        EGrad = Calc_GRAD(ID + 1, H1);                     

                        // 断面ブロック流速の計算
                        Calc_UUU(ID, H0, EGrad, out DGrad, out QRatio);

                        // フルード数による水位補正・登録
                        Adjust_HH(ID, ref H0);
                  
                        continue;
                    }
                        
                    // 当該断面の水位計算
                    DX = ModelCalInfo.COMHAR_DX[ID];            // 下流断面との距離
                    ID0 = ModelCalInfo.COMDAN_IDL[ID];          // 下流断面の断面インデックス
                    H0 = ModelCalInfo.COMCAL_CAL[ID0];          // 下流断面の水位

                    // 区間距離が設定されていないときは、下流断面の値をそのまま使う。
                    if (DX <= 0.0)
                    {
                        
                        H1 = H0;
                        EGrad = ModelCalInfo.COMCAL_EGRD[ID0];
                        Calc_UUU(ID, H1, EGrad, out DGrad, out QRatio);
                        ModelCalInfo.COMCAL_QRTO[ID] = QRatio;
                       
                    }
                    else
                    {



                        Calc_TERM(ID0, H0, out E0, out E1, out E2);
                        FS0 = H0 + E0 + (E1 + E2) * DX * 0.5;

                        double Delta_H = 0.5;       // 初期水位の設定
                        H1 = H0 + Delta_H;
                        EGrad = Calc_GRAD(ID, H1);  // エネルギー勾配の初期値設定

                        double Hmax0 = ModelCalInfo.COMHAR_HH[McQuasi2DSteadyFlowModelCalInfo.FLOW00_MD1 - 1, ID];
                        double Hmin0 = ModelCalInfo.COMHAR_HH[0, ID] + 1.0E-04;
                        double Hmax = Hmax0;
                        double Hmin = Hmin0;
                        double H1pre = 0.0;         // 前回収束ループ時の水位


                        // 不等流水位収束ループ（断面流速分布を無視）
                        for (int i_H = 0; i_H < 30; i_H++)
                        {
                            if (Hmin <= Hmin0)
                            {
                                Hmin = Hmin0;
                            }
                            if (Hmax >= Hmax0)
                            {
                                Hmax = Hmax0;
                            }


                            Calc_UUU(ID, H1, EGrad, out DGrad, out QRatio);

                            Calc_TERM(ID, H1, out E00, out E11, out E22);
                            FS1 = H1 + E00 - (E11 + E22) * DX * 0.5;

                            // 不等流水位の収束判定
                            if (Chk_HH(i_H, FS0, FS1, Hmin, ref FS_DIF0, ref Hold, ref H1, ref Hnew, ref ConvStep))
                            {
                                break;
                            }

                            if (i_H > 0 && H1 == H1pre)
                            {
                                break;
                            }

                            H1pre = H1;

                        }
                    }

                    // フルード数による水位補正・登録
                    Adjust_HH(ID, ref H1);


                    // 支川合流による水位上昇の計算
                    if (ModelCalInfo.COMDAN_RIVC[ID0] != -1 || ID == NS)
                    {
                        int RivCODE;    // 本支川コード
                        int IDD1;        // 本川の上流断面インデックス
                        int IDD2;        // 支川の上流断面インデックス
                        int IDD3;        // 本川の下流断面インデックス

                        IDD3 = ID0;
                        // 本川の場合
                        if (IR == 0)
                        {
                            RivCODE = ModelCalInfo.COMDAN_RIVC[ID0];
                            IDD1 = ID;
                            IDD2 = ModelCalInfo.COMDAN_IDS[RivCODE];
                        }
                        // 支川の場合
                        else
                        {
                            RivCODE = IR;
                            IDD1 = ID0 + 1;
                            IDD2 = ID;
                        }

                        double BB1 = ModelCalInfo.COMTBL_BB1[RivCODE];
                        double BB2 = ModelCalInfo.COMTBL_BB2[RivCODE];
                        double BB3 = ModelCalInfo.COMTBL_BB3[RivCODE];
                        double ANG1 = ModelCalInfo.COMTBL_ANG1[RivCODE];
                        double ANG2 = ModelCalInfo.COMTBL_ANG1[RivCODE];
                        double QQ1 = ModelCalInfo.COMCAL_QQ[IDD1];
                        double QQ2 = ModelCalInfo.COMCAL_QQ[IDD2];
                        double QQ3 = ModelCalInfo.COMCAL_QQ[IDD3];
                        double DEP = ModelCalInfo.COMCAL_DEP[IDD1];

                        double DH01 = Calc_DH01(BB1, BB2, BB3, ANG1, ANG2, QQ1, QQ2, QQ3, DEP);
                        H1 += DH01;
                        ModelCalInfo.COMCAL_DH01[ID] = DH01;

                        // 水位の修正・登録
                        EGrad = DGrad;
                        Calc_UUU(ID, H1, EGrad, out DGrad, out QRatio);
                        Adjust_HH(ID, ref H1);
                          
                    }


                    // 橋脚による水位上昇の計算
                    if(ModelCalInfo.COMCAL_FRR[ID0] <= 0.6)
                    {
                        double DH02 = 0.0;
                        double CD = ModelCalInfo.COMBRD_CD[ID0];
                        double DEP = ModelCalInfo.COMCAL_DEP[ID0];
                        double B1 = ModelCalInfo.COMBRD_B1[ID0];
                        double B2 = ModelCalInfo.COMBRD_B2[ID0];
                        double Q = ModelCalInfo.COMCAL_QQ[ID0];
                        if(CD * B1 > 0.0)
                        {
                            DH02 = Calc_DH02(Q, CD, DEP, B1, B2);
                            H1 += DH02;
                            ModelCalInfo.COMCAL_DH02[ID] = DH02;

                            // 水位の修正・登録
                            EGrad = DGrad;
                            Calc_UUU(ID, H1, EGrad, out DGrad, out QRatio);
                            Adjust_HH(ID, ref H1);

                        }


                    }   // IDループ


                }   // IRループ

            }

            // 湾曲による局所的な水位上昇の計算
            Calc_DH03();


            // 砂州による局所的な水位上昇の計算
            Calc_DH04();


            return true;
        }

        /// <summary>
        /// 断面流量をセットする。
        /// </summary>
        void Set_QQ()
        {
            for (int ID = 0; ID < ModelCalInfo.COMDAN_IDAN; ID++)
            {
                double.TryParse(inputFiles.crossFile.sections[ID].getThroughRecord().Flow, out ModelCalInfo.COMCAL_QQ[ID]);
            }
        }

        /// <summary>
        /// フルード数 を求める
        /// </summary>
        /// <param name="ID">断面番号</param>
        /// <param name="HH">水位</param>
        /// <param name="QQ">流量</param>
        /// <param name="FR">フルード数</param>
        /// <param name="V">流速</param>
        /// <param name="ASUM">流積</param>
        /// <param name="RC">合成径深</param>
        /// <param name="WWW">水面幅</param>
        private void Calc_FROUDE(int ID, double HH, double QQ, out double FR, out double V, out double ASUM, out double RC, out double WWW)
        {
            const double GRV = 9.8F;
            double AAA;
            double RRR;
            double SSS;
            double BBB;
            int NBUN = ModelCalInfo.COMHAR_NBUN[ID];
            RC = 0.0;
            WWW = 0.0;
            double RSUM = 0.0;
            ASUM = 0.0F;

            for (int IB = 0; IB < NBUN; IB++)
            {
                HARP_HWL(ID, IB, HH, out AAA, out RRR, out SSS, out BBB);
                ASUM += AAA;
                RSUM += RRR;
                WWW += BBB;
                RC += AAA * Math.Pow(RRR, 2.0 / 3.0);
            }
            RC = Math.Pow((RC / ASUM), 1.5);
            V = QQ / ASUM;
            FR = V / Math.Sqrt(GRV * RC);
            return;
        }

        /// <summary>
        /// エネルギー勾配を計算する
        /// </summary>
        /// <param name="ID">当該断面インデックス</param>
        /// <param name="HH">当該断面水位</param>
        /// <returns>エネルギー勾配</returns>
        double Calc_GRAD(int ID, double HH)
        {

            const double GRV = 9.8;                           // 重力加速度
            int IDL = ModelCalInfo.COMDAN_IDL[ID];
            int NBUN = ModelCalInfo.COMHAR_NBUN[ID];
            int NBUNL = ModelCalInfo.COMHAR_NBUN[IDL];

            double H0 = ModelCalInfo.COMCAL_CAL[IDL];
            double Q0 = ModelCalInfo.COMCAL_QQ[IDL];
            double Q1 = ModelCalInfo.COMCAL_QQ[ID];
            double AA = 0.0F;
            double ASUM = 0.0F;
            double ASUML = 0.0F;
            double DUMMY;

            double DX = ModelCalInfo.COMHAR_DX[ID];

            // エラーログ
            string log_file = (Path.Combine(m_sWorkingDir, "Err_EGrad.log"));
            if (ID == 0 && System.IO.File.Exists(log_file))
            {
                System.IO.File.Delete(log_file);
            }
            StreamWriter sw;

            if(DX <= 0.0)
            {
                sw = new StreamWriter(log_file, true, Encoding.GetEncoding("shift_jis"));
                string Message = ModelCalInfo.COMDAN_RIVR[ID] + " " + ModelCalInfo.COMDAN_ID[ID] + "において区間距離が設定されていません。エネルギー勾配を0に設定します。";
                sw.WriteLine(Message);
                sw.Close();
                return 0.0;
            }

            for (int IB = 0; IB < NBUNL; IB++)
            {
                HARP_HWL(IDL, IB, H0, out AA, out DUMMY, out DUMMY, out DUMMY);
                ASUML += AA;
            }
            for (int IB = 0; IB < NBUN; IB++)
            {
                HARP_HWL(ID, IB, HH, out AA, out DUMMY, out DUMMY, out DUMMY);
                ASUM += AA;
            }

            if(ASUM <= 0.0)
            {
                string Message = ModelCalInfo.COMDAN_RIVR[ID] + " " + ModelCalInfo.COMDAN_ID[ID] + "において流積が0を下回っています。";
                throw new Exception(Message);
            }

            return (HH - H0 + ((Math.Pow(Q1 / ASUM, 2.0) - Math.Pow(Q0 / ASUML, 2.0)) / (2.0 * GRV))) / DX;

        }


        /// <summary>
        /// 断面ブロック流速を求める。
        /// </summary>
        /// <param name="ID">当該断面ID</param> 
        /// <param name="HH">当該断面水位</param>
        /// <param name="Hupper">上限水位</param>
        /// <param name="Grad">エネルギー勾配初期値</param>
        void Calc_UUU(int ID, double HH, double InitGrad, out double DGrad, out double QRatio)
        {
            double Q_EPS = 0.01;
            double QDIF, QDIF0 = 0.0;

            int NBUN = ModelCalInfo.COMHAR_NBUN[ID];
            double Grad = InitGrad;                         // エネルギー勾配;
            double Grad0;                                   // 断面ブロックの流速差を無視した場合のエネルギー勾配
            double GradL = 0.0F, GradU = 0.0F;
            double QQ = ModelCalInfo.COMCAL_QQ[ID];
            double[] UUU0 = new double[NBUN];             // 断面ブロック流速初期値（収束計算のため）

            int CStep = 0;                              // 収束ステップ
            QRatio = 0.0F;



            // 初期値を設定する。
            if (Grad <= 0.0F)
            {
                for (int IB = 0; IB < NBUN; IB++)
                {
                    ModelCalInfo.COMUUU_U[IB, ID] = 1.0E-1;
                }
                Set_SSSF(ID, HH);
                Grad0 = Grad;
            }
            else
            {
                Set_SSSF(ID, HH);
                Set_UUU0(ID, HH, ref Grad);
                Grad0 = Grad;
            }

            for (int IB = 0; IB < NBUN; IB++)
            {
                UUU0[IB] = ModelCalInfo.COMUUU_U[IB, ID];
            }


            // 断面ブロック流速の収束ループ
            for (int i_qchk = 0; i_qchk < 10; i_qchk++)
            {

                double[,] JMAT = new double[NBUN, NBUN];
                double[] BMAT = new double[NBUN];
                double[] RMAT = new double[NBUN];



                // 断面ブロック流速分布の収束ループ
                for (int i_uchk = 0; i_uchk < 30; i_uchk++)
                {
                    // ヤコビ行列を設定する。
                    Set_JACOBI(ID, HH, Grad, ref JMAT, ref BMAT);

                    // ヤコビ行列を解く。
                    Solve_JACOBI(ref JMAT, ref BMAT, ref RMAT);
                    
                    // ブロック断面流速の収束判定
                    double Norm = 0.0;
                    for (int IB = 0; IB < NBUN; IB++)
                    {
                        Norm += RMAT[IB] * RMAT[IB];
                    }
                    Norm = Math.Sqrt(Norm) / NBUN;
                    if (Norm < 0.01)
                    {
                        break;
                    }

                    // 次のループのための漸化式を解く  
                    for (int IB = 0; IB < NBUN; IB++)
                    {
                        ModelCalInfo.COMUUU_U[IB, ID] -= RMAT[IB];
                        if (ModelCalInfo.COMUUU_U[IB, ID] <= 0)
                        {
                            ModelCalInfo.COMUUU_U[IB, ID] = UUU0[IB];
                        }
                    }

                }

                // 流量の収束判定
                QDIF = Calc_QDif(ID, HH, out QRatio);
                if (Math.Abs(QRatio - 1) <= Q_EPS)
                {
                    break;
                }

                if (i_qchk == 0)
                {
                    QDIF0 = QDIF;
                    CStep = 0;
                    if (QDIF0 > 0.0F)
                    {
                        GradU = Grad;
                    }
                    else
                    {
                        GradL = Grad;
                    }
                    continue;
                }

                if (CStep == 0)
                {
                    if (QDIF0 * QDIF > 0.0F)
                    {
                        if (QDIF0 < 0.0F)
                        {
                            Grad = 1.5F * Grad;
                        }
                        else
                        {
                            Grad = 0.5F * Grad;
                        }
                    }
                    else
                    {
                        if (QDIF0 < 0.0F)
                        {
                            GradU = Grad;
                        }
                        else
                        {
                            GradL = Grad;
                        }
                        Grad = 0.5F * (GradL + GradU);
                        CStep = 1;
                    }
                    continue;
                }

                if (CStep == 1)
                {
                    if (QDIF < 0.0F)
                    {
                        GradL = Grad;

                    }
                    else
                    {
                        GradU = Grad;
                    }
                    Grad = 0.5F * (GradL + GradU);
                }

            }

            // 流量比誤差が規定内に収まらなかった場合は、初期値を用いる。
            if (Math.Abs(QRatio - 1) > Q_EPS)
            {
                for (int IB = 0; IB < NBUN; IB++)
                {
                    ModelCalInfo.COMUUU_U[IB, ID] = UUU0[IB];
                }
                ModelCalInfo.COMCAL_EGRD[ID] = Grad0;
                DGrad = Grad0;
            }

            ModelCalInfo.COMCAL_EGRD[ID] = Grad;
            DGrad = Grad;


        }

        /// <summary>
        /// 断面ブロック流速の収束計算のための初期値をセットする。
        /// </summary>
        /// <param name="ID">断面インデックス</param>
        /// <param name="QQ">流量</param>
        /// <param name="HH">水位</param>
        /// <param name="Grad">エネルギー勾配</param>
        void Set_UUU0(int ID, double HH, ref double Grad)
        {
            const double GRV = 9.8F;
            const double EPS = 0.005F;
            double F0S0, F1S1, F2S2, F3S3;
            double QRatio;
            double QDIF0 = 999.0F, QDIF;
            double GradL = 0.0F, GradU = 0.0F;
            int CStep = 0;
            int NBUN = ModelCalInfo.COMHAR_NBUN[ID];    // 断面ブロック数
            double[] A = new double[NBUN];

            for (int i_Grad = 0; i_Grad < 30; i_Grad++)
            {
                for (int IB = 0; IB < NBUN; IB++)
                {
                    double R, SB, W;
                    HARP_HWL(ID, IB, HH, out A[IB], out R, out SB, out W);
                    double n = ModelCalInfo.COSODO_FNN[ID][IB];
                    F0S0 = ModelCalInfo.COMWET_F0[ID][IB] * ModelCalInfo.COMWET_S0[ID][IB];
                    F1S1 = ModelCalInfo.COMWET_F1[ID][IB] * ModelCalInfo.COMWET_S1[ID][IB];
                    F2S2 = ModelCalInfo.COMWET_F2[ID][IB] * ModelCalInfo.COMWET_S2[ID][IB];
                    F3S3 = ModelCalInfo.COMWET_F3[ID][IB] * ModelCalInfo.COMWET_S3[ID][IB];
                    

                    if (R <= 0.0F)
                    {
                        ModelCalInfo.COMUUU_U[IB, ID] = 0.0F;
                    }
                    else
                    {
                        ModelCalInfo.COMUUU_U[IB, ID] = Math.Sqrt(A[IB] * Grad / (n * n * SB / Math.Pow(R, 1.0 / 3.0) + (F0S0 + F2S2) / GRV));
                    }

                }

                // 収束判定
                QDIF = Calc_QDif(ID, HH, out QRatio);
                if(Math.Abs(QRatio - 1) < EPS)
                {
                    break;
                }

                if (i_Grad == 0)
                {
                    QDIF0 = QDIF;
                    CStep = 0;
                    if (QDIF0 > 0.0F)
                    {
                        GradU = Grad;
                    }
                    else
                    {
                        GradL = Grad;
                    }
                    continue;
                }

                if (CStep == 0)
                {
                    if (QDIF0 * QDIF > 0.0F)
                    {
                        if (QDIF0 < 0.0F)
                        {
                            Grad = 1.5F * Grad;
                        }
                        else
                        {
                            Grad = 0.5F * Grad;
                        }
                    }
                    else
                    {
                        if (QDIF0 < 0.0F)
                        {
                            GradU = Grad;
                        }
                        else
                        {
                            GradL = Grad;
                        }
                        Grad = 0.5F * (GradL + GradU);
                        CStep = 1;
                    }
                    continue;
                }

                if (CStep == 1)
                {
                    if (QDIF < 0.0F)
                    {
                        GradL = Grad;

                    }
                    else
                    {
                        GradU = Grad;
                    }
                    Grad = 0.5F * (GradL + GradU);
                }

            }


        }
        
        /// <summary>
        /// 断面ブロックの境界混合係数（F0～F3）と潤辺長さ（S0～S3を求める。）
        /// </summary>
        /// <param name="ID">断面インデックス</param>
        /// <param name="IB">断面ブロック・インデックス</param>
        void Set_SSSF(int ID, double HH)
        {
            int NBUN = ModelCalInfo.COMHAR_NBUN[ID];        // 断面ブロック数
            int NDATA = ModelCalInfo.COMXYZ_NDATA[ID];      // 測点の個数

            double DUMMY;                                    // ダミー変数
            double BB;                                       // 水面幅計算用変数
            double B = 0.0;                                 // 水面幅
            for (int IB = 0; IB < NBUN; IB++)
            {
                HARP_HWL(ID, IB, HH, out DUMMY, out DUMMY, out DUMMY, out BB);
                B += BB;
            }

            
            double b = ModelCalInfo.COMXYZ_BBL[ID];          // 低水路幅
            double f1;                                       // 境界混合係数（低水路・高水敷境界）
            ModelCalInfo.COMWET_S0[ID] = new double[NBUN];
            ModelCalInfo.COMWET_S1[ID] = new double[NBUN];
            ModelCalInfo.COMWET_S2[ID] = new double[NBUN];
            ModelCalInfo.COMWET_S3[ID] = new double[NBUN];
            ModelCalInfo.COMWET_F0[ID] = new double[NBUN];
            ModelCalInfo.COMWET_F1[ID] = new double[NBUN];
            ModelCalInfo.COMWET_F2[ID] = new double[NBUN];
            ModelCalInfo.COMWET_F3[ID] = new double[NBUN];

            if(B <= 0.0)
            {
                throw new Exception(ModelCalInfo.COMDAN_RIVR[ID] + " " + ModelCalInfo.COMDAN_ID[ID] + "の水面幅が0以下の値になっています。");
            }
            if(b/B < 0.6)
            {
                f1 = ModelCalInfo.COMFFF_FT[1];
            }
            else if (0.6 <= b / B && b / B < 0.8)
            {
                f1 = ModelCalInfo.COMFFF_FT[2];
            }
            else
            {
                f1 = ModelCalInfo.COMFFF_FT[3];
            }


            for (int IB = 0; IB < NBUN; IB++)
            {
                // 断面ブロックが１つのとき（左端ブロックと右端ブロックの区別がない。）
                if (NBUN == 1)
                {
                    // 樹木群との境界をもたないので
                    ModelCalInfo.COMWET_S0[ID][0] = 0.0;
                    ModelCalInfo.COMWET_S2[ID][0] = 0.0;
                    ModelCalInfo.COMWET_S1[ID][0] = Math.Max(0.0, HH - ModelCalInfo.COMXYZ_YZ[0, ID]);
                    ModelCalInfo.COMWET_S3[ID][0] = Math.Max(0.0, HH - ModelCalInfo.COMXYZ_YZ[NDATA - 1, ID]);
                    ModelCalInfo.COMWET_F0[ID][0] = 0.0;
                    ModelCalInfo.COMWET_F2[ID][0] = 0.0;
                    ModelCalInfo.COMWET_F1[ID][0] = ModelCalInfo.COMFFF_FT[0];
                    ModelCalInfo.COMWET_F3[ID][0] = ModelCalInfo.COMFFF_FT[0];
                    break;
                }

                // 左端ブロックの場合
                if (IB == 0)
                {
                    // 当該ブロックが樹木ブロックでない場合
                    if (!ModelCalInfo.COSODO_ISTREE[ID][0])
                    {
                        // 左側境界は樹木群と接しないので
                        ModelCalInfo.COMWET_S0[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_S1[ID][IB] = Math.Max(0.0F, HH - ModelCalInfo.COMXYZ_YZ[0, ID]);
                        ModelCalInfo.COMWET_F0[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_F1[ID][IB] = ModelCalInfo.COMFFF_FT[0];
                        // 右側境界が樹木ブロックでない場合
                        if (!ModelCalInfo.COSODO_ISTREE[ID][1])
                        {
                            ModelCalInfo.COMWET_S2[ID][IB] = 0.0;
                            ModelCalInfo.COMWET_S3[ID][IB] = Math.Max(0.0F, HH - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][0], ID]);
                            ModelCalInfo.COMWET_F2[ID][IB] = 0.0;
                            ModelCalInfo.COMWET_F3[ID][IB] = f1; 
                        }
                        // 右側境界が樹木ブロックの場合
                        else
                        {
                            // 水位が右側境界の樹冠より高い場合（樹木群が水没）
                            if (HH > ModelCalInfo.COSODO_THH[ID][1])
                            {
                                ModelCalInfo.COMWET_S3[ID][IB] = HH - ModelCalInfo.COSODO_THH[ID][1];
                                ModelCalInfo.COMWET_S2[ID][IB] = ModelCalInfo.COSODO_THH[ID][1] - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][0], ID];
                                ModelCalInfo.COMWET_F3[ID][IB] = ModelCalInfo.COMFFF_TAU[2];
                                double Bw = ModelCalInfo.COSODO_BW[ID][IB + 1];      // 樹木群の幅
                                double Hw = ModelCalInfo.COSODO_TTH[ID][IB + 1];     // 樹木群の樹高
                                if (Bw / Hw <= 1.0F)
                                {
                                    ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[0];
                                }
                                else if (1.0F < Bw / Hw && Bw / Hw < 34.5F)
                                {
                                    ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[0] * Math.Pow(Bw / Hw, 0.34);
                                }
                                else
                                {
                                    ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[1];
                                } 
                            }
                            // 水位が右側境界の樹冠より低い場合（樹木群が流れに挟まれる）
                            else
                            {
                                ModelCalInfo.COMWET_S3[ID][IB] = 0.0;
                                ModelCalInfo.COMWET_S2[ID][IB] = Math.Max(0.0F, HH - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][0], ID]);
                                ModelCalInfo.COMWET_F3[ID][IB] = 0.0;
                                ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[1];
                            }
                        }
                    }
                    // 樹木ブロックの場合
                    else
                    {
                        ModelCalInfo.COMWET_S0[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_S2[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_S1[ID][IB] = Math.Max(0.0, HH - ModelCalInfo.COSODO_THH[ID][IB]);
                        ModelCalInfo.COMWET_S3[ID][IB] = Math.Max(0.0, HH - ModelCalInfo.COSODO_THH[ID][IB]);
                        ModelCalInfo.COMWET_F0[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_F2[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_F1[ID][IB] = ModelCalInfo.COMFFF_FT[0];
                        double Bw = ModelCalInfo.COSODO_BW[ID][IB];      // 樹木群の幅
                        if (1.0 - 2.0 * Bw / B < 0.6)
                        {
                            ModelCalInfo.COMWET_F3[ID][IB] = ModelCalInfo.COMFFF_FT[1];
                        }
                        else if (0.6 <= 1.0 - 2.0 * Bw / B && 1.0 - 2.0F * Bw / B < 0.8)
                        {
                            ModelCalInfo.COMWET_F3[ID][IB] = ModelCalInfo.COMFFF_FT[2];
                        }
                        else
                        {
                            ModelCalInfo.COMWET_F3[ID][IB] = ModelCalInfo.COMFFF_FT[3];
                        }
                    }
                }
                // 右端ブロックの場合
                else if (IB == NBUN - 1)
                {
                    // 当該ブロックが樹木ブロックでない場合
                    if (!ModelCalInfo.COSODO_ISTREE[ID][IB])
                    {
                        // 右端境界は、樹木と接しないので
                        ModelCalInfo.COMWET_S2[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_S3[ID][IB] = Math.Max(0.0, HH - ModelCalInfo.COMXYZ_YZ[NDATA - 1, ID]);
                        ModelCalInfo.COMWET_F2[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_F3[ID][IB] = ModelCalInfo.COMFFF_FT[0];
                        // 左側境界が樹木ブロックでない場合
                        if (!ModelCalInfo.COSODO_ISTREE[ID][IB - 1])
                        {
                            ModelCalInfo.COMWET_S0[ID][IB] = 0.0;
                            ModelCalInfo.COMWET_S1[ID][IB] = Math.Max(0.0F, HH - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB - 1], ID]);
                            ModelCalInfo.COMWET_F0[ID][IB] = 0.0;
                            ModelCalInfo.COMWET_F1[ID][IB] = f1;
                        }
                        // 左側境界が樹木ブロックの場合
                        else
                        {
                            // 水位が左側ブロックの樹冠よりも高い場合（樹木群が水没）
                            if (HH > ModelCalInfo.COSODO_THH[ID][IB - 1])
                            {
                                ModelCalInfo.COMWET_S1[ID][IB] = HH - ModelCalInfo.COSODO_THH[ID][IB - 1];
                                ModelCalInfo.COMWET_S0[ID][IB] = ModelCalInfo.COSODO_THH[ID][IB - 1] - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB - 1], ID];
                                ModelCalInfo.COMWET_F1[ID][IB] = ModelCalInfo.COMFFF_TAU[2];
                                double Bw = ModelCalInfo.COSODO_BW[ID][IB - 1];      // 樹木群の幅
                                double Hw = ModelCalInfo.COSODO_TTH[ID][IB - 1];     // 樹木群の樹高
                                if (Bw / Hw <= 1.0)
                                {
                                    ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[0];
                                }
                                else if (1.0 < Bw / Hw && Bw / Hw < 34.5)
                                {
                                    ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[0] * Math.Pow(Bw / Hw, 0.34);
                                }
                                else
                                {
                                    ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[1];
                                }
                            }
                            // 水位が左側ブロックの樹冠よりも低い場合（樹木群が流れに挟まれる）
                            else
                            {
                                ModelCalInfo.COMWET_S1[ID][IB] = 0.0;
                                ModelCalInfo.COMWET_S0[ID][IB] = Math.Max(0.0, HH - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB - 1], ID]);
                                ModelCalInfo.COMWET_F1[ID][IB] = 0.0;
                                ModelCalInfo.COMWET_F0[ID][IB] = ModelCalInfo.COMFFF_FT[1];
                            }
                        }

                    }
                    // 当該ブロックが樹木ブロックの場合
                    else
                    {
                        ModelCalInfo.COMWET_S0[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_S2[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_S1[ID][IB] = Math.Max(0.0, HH - ModelCalInfo.COSODO_THH[ID][IB]);
                        ModelCalInfo.COMWET_S3[ID][IB] = Math.Max(0.0, HH - ModelCalInfo.COSODO_THH[ID][IB]);
                        ModelCalInfo.COMWET_F0[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_F2[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_F3[ID][IB] = ModelCalInfo.COMFFF_FT[0];
                        double Bw = ModelCalInfo.COSODO_BW[ID][IB];      // 樹木群の幅
                        if (1.0F - 2.0F * Bw / B < 0.6F)
                        {
                            ModelCalInfo.COMWET_F1[ID][IB] = ModelCalInfo.COMFFF_FT[1];
                        }
                        else if (0.6 <= 1.0 - 2.0 * Bw / B && 1.0 - 2.0 * Bw / B < 0.8)
                        {
                            ModelCalInfo.COMWET_F1[ID][IB] = ModelCalInfo.COMFFF_FT[2];
                        }
                        else
                        {
                            ModelCalInfo.COMWET_F1[ID][IB] = ModelCalInfo.COMFFF_FT[3];
                        }
                    }
                }
                // 右端・左端ブロックのどちらでもない場合
                else
                {
                    // 樹木ブロックでない場合
                    if (!ModelCalInfo.COSODO_ISTREE[ID][IB])
                    {
                        // 両側とも樹木ブロックの場合
                        if (ModelCalInfo.COSODO_ISTREE[ID][IB - 1] && ModelCalInfo.COSODO_ISTREE[ID][IB + 1])
                        {
                            // 水位が左側境界の樹冠より高い場合（樹木群が水没）
                            if (HH > ModelCalInfo.COSODO_THH[ID][IB - 1])
                            {
                                ModelCalInfo.COMWET_S1[ID][IB] = HH - ModelCalInfo.COSODO_THH[ID][IB - 1];
                                ModelCalInfo.COMWET_S0[ID][IB] = ModelCalInfo.COSODO_THH[ID][IB - 1] - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB - 1], ID];
                                ModelCalInfo.COMWET_F1[ID][IB] = ModelCalInfo.COMFFF_TAU[2];
                                double Bw = ModelCalInfo.COSODO_BW[ID][IB - 1];      // 樹木群の幅
                                double Hw = ModelCalInfo.COSODO_TTH[ID][IB - 1];     // 樹木群の樹高
                                if (Bw / Hw <= 1.0)
                                {
                                    ModelCalInfo.COMWET_F0[ID][IB] = ModelCalInfo.COMFFF_TAU[0];
                                }
                                else if (1.0 < Bw / Hw && Bw / Hw < 34.5)
                                {
                                    ModelCalInfo.COMWET_F0[ID][IB] = ModelCalInfo.COMFFF_TAU[0] * Math.Pow(Bw / Hw, 0.34);
                                }
                                else
                                {
                                    ModelCalInfo.COMWET_F0[ID][IB] = ModelCalInfo.COMFFF_TAU[1];
                                } 
                            }
                            // 水位が左側境界の樹冠より低い場合（樹木群が流れに挟まれる）
                            else
                            {
                                ModelCalInfo.COMWET_S1[ID][IB] = 0.0;
                                ModelCalInfo.COMWET_S0[ID][IB] = Math.Max(0.0F, HH - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB - 1], ID]);
                                ModelCalInfo.COMWET_F1[ID][IB] = 0.0;
                                ModelCalInfo.COMWET_F0[ID][IB] = ModelCalInfo.COMFFF_TAU[1];
                            }
                            // 水位が右側境界の樹冠より高い場合（樹木群が水没）
                            if (HH > ModelCalInfo.COSODO_THH[ID][IB + 1])
                            {
                                ModelCalInfo.COMWET_S3[ID][IB] = HH - ModelCalInfo.COSODO_THH[ID][IB + 1];
                                ModelCalInfo.COMWET_S2[ID][IB] = ModelCalInfo.COSODO_THH[ID][IB + 1] - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB], ID];
                                ModelCalInfo.COMWET_F3[ID][IB] = ModelCalInfo.COMFFF_TAU[2];
                                double Bw = ModelCalInfo.COSODO_BW[ID][IB + 1];      // 樹木群の幅
                                double Hw = ModelCalInfo.COSODO_TTH[ID][IB + 1];     // 樹木群の樹高
                                if (Bw / Hw <= 1.0F)
                                {
                                    ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[0];
                                }
                                else if (1.0 < Bw / Hw && Bw / Hw < 34.5)
                                {
                                    ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[0] * Math.Pow(Bw / Hw, 0.34);
                                }
                                else
                                {
                                    ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[1];
                                }

                            }
                            // 水位が右側境界の樹冠より低い場合（樹木群が流れに挟まれる）
                            else
                            {
                                ModelCalInfo.COMWET_S3[ID][IB] = 0.0;
                                ModelCalInfo.COMWET_S2[ID][IB] = Math.Max(0.0F, HH - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB], ID]);
                                ModelCalInfo.COMWET_F3[ID][IB] = 0.0;
                                ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[1];
                            }
                        }
                        // 左側のみ樹木ブロックの場合
                        else if (ModelCalInfo.COSODO_ISTREE[ID][IB - 1] && !ModelCalInfo.COSODO_ISTREE[ID][IB + 1])
                        {
                            ModelCalInfo.COMWET_S2[ID][IB] = 0.0;
                            ModelCalInfo.COMWET_S3[ID][IB] = Math.Max(0.0F, HH - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB], ID]);
                            ModelCalInfo.COMWET_F2[ID][IB] = 0.0;
                            ModelCalInfo.COMWET_F3[ID][IB] = f1;

                            // 水位が左側境界の樹冠より高い場合（樹木群が水没）
                            if (HH > ModelCalInfo.COSODO_THH[ID][IB - 1])
                            {
                                ModelCalInfo.COMWET_S1[ID][IB] = HH - ModelCalInfo.COSODO_THH[ID][IB - 1];
                                ModelCalInfo.COMWET_S0[ID][IB] = ModelCalInfo.COSODO_THH[ID][IB - 1] - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB - 1], ID];
                                ModelCalInfo.COMWET_F1[ID][IB] = ModelCalInfo.COMFFF_TAU[2];
                                double Bw = ModelCalInfo.COSODO_BW[ID][IB - 1];      // 樹木群の幅
                                double Hw = ModelCalInfo.COSODO_TTH[ID][IB - 1];     // 樹木群の樹高
                                if (Bw / Hw <= 1.0)
                                {
                                    ModelCalInfo.COMWET_F0[ID][IB] = ModelCalInfo.COMFFF_TAU[0];
                                }
                                else if (1.0 < Bw / Hw && Bw / Hw < 34.5)
                                {
                                    ModelCalInfo.COMWET_F0[ID][IB] = ModelCalInfo.COMFFF_TAU[0] * Math.Pow(Bw / Hw, 0.34);
                                }
                                else
                                {
                                    ModelCalInfo.COMWET_F0[ID][IB] = ModelCalInfo.COMFFF_TAU[1];
                                }

                            }
                            // 水位が左側境界の樹冠より低いの場合（樹木群が流れに挟まれる）
                            else
                            {
                                ModelCalInfo.COMWET_S1[ID][IB] = 0.0;
                                ModelCalInfo.COMWET_S0[ID][IB] = Math.Max(0.0F, HH - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB - 1], ID]);
                                ModelCalInfo.COMWET_F1[ID][IB] = 0.0;
                                ModelCalInfo.COMWET_F0[ID][IB] = ModelCalInfo.COMFFF_TAU[1];
                            }
                        }
                        // 右側のみ樹木ブロックの場合
                        else if (!ModelCalInfo.COSODO_ISTREE[ID][IB - 1] && ModelCalInfo.COSODO_ISTREE[ID][IB + 1])
                        {
                            ModelCalInfo.COMWET_S0[ID][IB] = 0.0;
                            ModelCalInfo.COMWET_S1[ID][IB] = Math.Max(0.0F, HH - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB - 1], ID]);
                            ModelCalInfo.COMWET_F0[ID][IB] = 0.0;
                            ModelCalInfo.COMWET_F1[ID][IB] = f1;
                            // 水位が右側境界の樹冠より高い場合（樹木群が水没）
                            if (HH > ModelCalInfo.COSODO_THH[ID][IB + 1])
                            {
                                ModelCalInfo.COMWET_S3[ID][IB] = HH - ModelCalInfo.COSODO_THH[ID][IB + 1];
                                ModelCalInfo.COMWET_S2[ID][IB] = ModelCalInfo.COSODO_THH[ID][IB + 1] - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB], ID];
                                ModelCalInfo.COMWET_F3[ID][IB] = ModelCalInfo.COMFFF_TAU[2];
                                double Bw = ModelCalInfo.COSODO_BW[ID][IB + 1];      // 樹木群の幅
                                double Hw = ModelCalInfo.COSODO_TTH[ID][IB + 1];     // 樹木群の樹高
                                if (Bw / Hw <= 1.0)
                                {
                                    ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[0];
                                }
                                else if (1.0 < Bw / Hw && Bw / Hw < 34.5)
                                {
                                    ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[0] * Math.Pow(Bw / Hw, 0.34);
                                }
                                else
                                {
                                    ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[1];
                                }

                            }
                            // 水位が右側境界の樹冠より低い場合（樹木群が流れに挟まれる）
                            else
                            {
                                ModelCalInfo.COMWET_S3[ID][IB] = 0.0;
                                ModelCalInfo.COMWET_S2[ID][IB] = Math.Max(0.0, HH - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB], ID]);
                                ModelCalInfo.COMWET_F3[ID][IB] = 0.0;
                                ModelCalInfo.COMWET_F2[ID][IB] = ModelCalInfo.COMFFF_TAU[1];
                            }
                        }
                        // 両側とも樹木ブロックでない場合
                        else
                        {
                            ModelCalInfo.COMWET_S0[ID][IB] = 0.0;
                            ModelCalInfo.COMWET_S2[ID][IB] = 0.0;
                            ModelCalInfo.COMWET_S1[ID][IB] = Math.Max(0.0, HH - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB - 1], ID]);
                            ModelCalInfo.COMWET_S3[ID][IB] = Math.Max(0.0, HH - ModelCalInfo.COMXYZ_YZ[ModelCalInfo.COSODO_IDX[ID][IB], ID]);
                            ModelCalInfo.COMWET_F0[ID][IB] = 0.0;
                            ModelCalInfo.COMWET_F2[ID][IB] = 0.0;
                            ModelCalInfo.COMWET_F1[ID][IB] = f1;
                            ModelCalInfo.COMWET_F3[ID][IB] = f1;
                        }
                    }
                    // 樹木ブロックの場合
                    else
                    {
                        ModelCalInfo.COMWET_S0[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_S2[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_S1[ID][IB] = Math.Max(0.0, HH - ModelCalInfo.COSODO_THH[ID][IB]);
                        ModelCalInfo.COMWET_S3[ID][IB] = Math.Max(0.0, HH - ModelCalInfo.COSODO_THH[ID][IB]);
                        ModelCalInfo.COMWET_F0[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_F2[ID][IB] = 0.0;
                        ModelCalInfo.COMWET_F1[ID][IB] = ModelCalInfo.COMFFF_TAU[2];
                        ModelCalInfo.COMWET_F3[ID][IB] = ModelCalInfo.COMFFF_TAU[2];
                    }

                }
            }
        }

        /// <summary>
        /// ヤコビ行列を求める。
        /// </summary>
        /// <param name="ID">断面ID</param>
        /// <param name="H">水位</param>
        /// <param name="GRAD">エネルギー勾配</param>
        /// <param name="JMAT">ヤコビ行列</param>
        /// <param name="BMAT">行列</param>
        void Set_JACOBI(int ID, double H, double GRAD, ref double[,] JMAT, ref double[] BMAT)
        {
            int NBUN = ModelCalInfo.COMHAR_NBUN[ID];   // 当該断面ブロック数
            const double GRV = 9.8F;                    // 重力加速度

            double DUMMY;
            double[] RR = new double[NBUN];              // ブロック断面径深
            double[] AA = new double[NBUN];              // 流積計算用変数
            double[] SS = new double[NBUN];              // ブロック潤辺長
            for (int IB = 0; IB < NBUN; IB++)
            {
                HARP_HWL(ID, IB, H, out AA[IB], out RR[IB], out SS[IB], out DUMMY);
            }

            // 行列の要素を「0」で初期化          
            for (int i = 0; i < NBUN; i++)
            {
                BMAT[i] = 0.0;
                for (int j = 0; j < NBUN; j++)
                {
                    JMAT[i, j] = 0.0;
                }
            }

            // ヤコビ行列の設定
            for (int IB = 0; IB < NBUN; IB++)
            {
                double n = ModelCalInfo.COSODO_FNN[ID][IB];  // 断面ブロック粗度
                double A = AA[IB];                           // 断面ブロック流積
                double R = RR[IB];                           // 断面ブロック径深
                double S = SS[IB];                           // 断面ブロック潤辺長
                double F0S0 = ModelCalInfo.COMWET_F0[ID][IB] * ModelCalInfo.COMWET_S0[ID][IB];
                double F1S1 = ModelCalInfo.COMWET_F1[ID][IB] * ModelCalInfo.COMWET_S1[ID][IB];
                double F2S2 = ModelCalInfo.COMWET_F2[ID][IB] * ModelCalInfo.COMWET_S2[ID][IB];
                double F3S3 = ModelCalInfo.COMWET_F3[ID][IB] * ModelCalInfo.COMWET_S3[ID][IB];

                if (A <= 0.0F)
                {
                    continue;
                }

                // 断面ブロックがひとつの場合
                if (NBUN == 1)
                {
                    double U = ModelCalInfo.COMUUU_U[IB, ID];
                    JMAT[0, 0] = 2.0 * (Math.Pow(n, 2.0) / Math.Pow(R, 1.0 / 3.0) * S * U +
                                (F1S1 + F3S3) * U / GRV);
                    BMAT[0] = Math.Pow(n, 2.0) / Math.Pow(R, 1.0 / 3.0) * S * U * U +
                               (F1S1 + F3S3) * U * U / GRV - A * GRAD;

                    break;
                }

                // 左端ブロックの場合
                if (IB == 0)
                {
                    double U0 = ModelCalInfo.COMUUU_U[IB, ID];
                    double Ub = ModelCalInfo.COMUUU_U[IB + 1, ID];
                    JMAT[IB, IB] = 2.0F * (((Math.Pow(n, 2.0) / Math.Pow(R, 1.0 / 3.0)) * S + (F1S1 + F2S2) / GRV) * U0 +
                        F3S3 * (U0 - Ub) * SIGN(U0, Ub) / GRV);
                    JMAT[IB, IB + 1] = -2.0F * F3S3 * (U0 - Ub) * SIGN(U0, Ub) / GRV;
                    BMAT[IB] = (Math.Pow(n, 2.0) / Math.Pow(R, 1.0 / 3.0)) * S * U0 * U0 +
                               (F1S1 + F2S2) * U0 * U0 / GRV + F3S3 * (U0 - Ub) * (U0 - Ub) * SIGN(U0, Ub) / GRV - A * GRAD;
                }
                // 右端ブロックの場合
                else if (IB == NBUN - 1)
                {
                    double Ua = ModelCalInfo.COMUUU_U[IB - 1, ID];
                    double U0 = ModelCalInfo.COMUUU_U[IB, ID];
                    JMAT[IB, IB] = 2.0F * (((Math.Pow(n, 2.0) / Math.Pow(R, 1.0 / 3.0)) * S + (F0S0 + F3S3) / GRV) * U0 +
                                     F1S1 * (U0 - Ua) * SIGN(U0, Ua)) / GRV;
                    JMAT[IB, IB - 1] = -2.0F * F1S1 * (U0 - Ua) * SIGN(U0, Ua) / GRV;
                    BMAT[IB] = (Math.Pow(n, 2.0) / Math.Pow(R, 1.0 / 3.0)) * S * U0 * U0 +
                               (F0S0 + F3S3) * U0 * U0 / GRV + F1S1 * (U0 - Ua) * (U0 - Ua) * SIGN(U0, Ua) / GRV - A * GRAD;
                }
                // 左端プロックでも、右端ブロックでもない場合
                else
                {
                    double Ua = ModelCalInfo.COMUUU_U[IB - 1, ID];
                    double U0 = ModelCalInfo.COMUUU_U[IB, ID];
                    double Ub = ModelCalInfo.COMUUU_U[IB + 1, ID];
                    JMAT[IB, IB - 1] = -2.0F * F1S1 * (U0 - Ua) * SIGN(U0, Ua) / GRV;
                    JMAT[IB, IB] = 2.0F * (((Math.Pow(n, 2.0) / Math.Pow(R, 1.0 / 3.0)) * S + (F0S0 + F2S2) / GRV) * U0 +
                        F1S1 * (U0 - Ua) * SIGN(U0, Ua) / GRV + F3S3 * (U0 - Ub) * SIGN(U0, Ub) / GRV);
                    JMAT[IB, IB + 1] = -2.0F * F3S3 * (U0 - Ub) * SIGN(U0, Ub) / GRV;
                    BMAT[IB] = (Math.Pow(n, 2.0) / Math.Pow(R, 1.0 / 3.0)) * S * U0 * U0+
                                (F0S0 + F2S2) * U0 * U0 / GRV +
                                F1S1 * (U0 - Ua) * (U0 - Ua) * SIGN(U0, Ua) /GRV + F3S3 * (U0 - Ub) * (U0 - Ub) * SIGN(U0, Ub) / GRV -
                                A * GRAD;
                }

            }

        }

        /// <summary>
        /// 流速差による摩擦力の働く方向を求める。
        /// </summary>
        /// <param name="U0"></param>
        /// <param name="U1"></param>
        /// <returns></returns>
        double SIGN(double U0, double U1)
        {
            return U0 > U1 ? 1.0 : -1.0;
        }

        /// <summary>
        /// 掃き出し法で連立方程式を解く（バンド幅が３の疎行列であることを考慮して、省力化）
        /// </summary>
        /// <parama name="A">ヤコビ行列</parama>
        /// <param name="C"></param>
        /// <param name="X">解ベクトル</param>
        void Solve_SWEEP(ref double[,] A, ref double[] C, ref double[] X)
        {
            int N = A.GetLength(0);
            double[,] B = new double[N, N];

            if (N == 1)
            {
                if(A[0, 0] != 0.0)
                {
                    X[0] = C[0] / A[0, 0];
                }
                return;
            }

            if(N == 2)
            {
                double DET = A[0, 0] * A[1, 1] - A[1, 0] * A[0, 1];
                if(DET != 0.0)
                {
                    X[0] = (A[1, 1] * C[0]  - A[1, 0] * C[1]) / DET; 
                    X[1] = (- A[0, 1] * C[0] + A[0, 0] * C[1]) / DET;
                }
                return;
            }

            // ガウスの消去法で解く（３要素のバンド行列であることに着目して、手順を簡略化）
            if (N > 2)
            {
                // 前進消去により上三角行列をつくる（バンド幅は２になる）。
                for (int i = 0; i < N - 1; i++)
                {
                    if (A[i, i] != 0.0)
                    {
                        double WW = A[i, i];
                        double AA = A[i + 1, i];
                        A[i, i + 1] = A[i, i + 1] / WW;
                        C[i] = C[i] / WW;

                        A[i + 1, i + 1] -= AA * A[i, i + 1];  
                        C[i + 1] -= C[i];
                    }
                }

                // 後退代入により解を求める。
                X[N - 1] = C[N - 1];
                for (int i = N - 2; i >= 0; i--)
                {
                        X[i] =  C[i] -  A[i, i + 1] * X[i + 1];
                }
                return;
            }

           
            return;
        }

        /// <summary>
        /// 0要素で部分行列の分割して連立方程式を解く
        /// </summary>
        /// <parama name="A">ヤコビ行列</parama>
        /// <param name="C"></param>
        /// <param name="X">解ベクトル</param>
        void Solve_JACOBI(ref double[,] A, ref double[] C, ref double[] X)
        {
            int N = A.GetLength(0);

            int head = 0;
            int len = 0;
            List<int> head_list = new List<int>();
            List<int> len_list = new List<int>();
            List<double[,]> AList = new List<double[,]>();
            List<double[]> CList = new List<double[]>();
            List<double[]> XList = new List<double[]>();

            // 行列の初期化
            for (int i = 0; i < N; i++)
            {
                X[i] = 0.0;
            }

            // ヤコビ行列を部分行列に分解
            for (int i = 0; i < N; i++)
            {
                if (C[i] != 0.0)
                {
                    if (len == 0)
                    {
                        head = i;
                        head_list.Add(i);
                    }
                    len++;
                }
                else
                {
                    if (len != 0)
                    {
                        len_list.Add(len);
                        double[,] ASub = new double[len, len];
                        double[] CSub = new double[len];
                        double[] XSub = new double[len];

                        for(int j = 0; j < len; j++)
                        {
                            for(int k = 0; k < len; k++)
                            {
                                ASub[j, k] = A[head + j, head + k];
                            }
                            CSub[j] = C[head + j];
                            XSub[j] = 0.0;
                        }
                        AList.Add(ASub);
                        CList.Add(CSub);
                        XList.Add(XSub);
                        len = 0;
                    }
                }
                
                // 最後尾要素の処理
                if (i == N - 1)
                {
                    len_list.Add(len);
                    double[,] ASub = new double[len, len];
                    double[] CSub = new double[len];
                    double[] XSub = new double[len];

                    for (int j = 0; j < len; j++)
                    {
                        for (int k = 0; k < len; k++)
                        {
                            ASub[j, k] = A[head + j, head + k];
                        }
                        CSub[j] = C[head + j];
                        XSub[j] = 0.0;
                    }
                    AList.Add(ASub);
                    CList.Add(CSub);
                    XList.Add(XSub);
                }

            }

            // 連立一次元方程式を解く
            for (int ii = 0; ii < AList.Count; ii++)
            {
                int NN = AList[ii].GetLength(0);
                double[,] AA = new double[NN, NN];
                double[] CC = new double[NN];
                double[] XX = new double[NN];
                for (int i = 0; i < NN; i++)
                {
                    for (int j = 0; j < NN; j++)
                    {
                        AA[i, j] = AList[ii][i, j];
                    }
                    CC[i] = CList[ii][i];
                }

                Solve_SWEEP(ref AA, ref CC, ref XX);

                int NX = XX.GetLength(0);
                for (int i = 0; i < NX; i++)
                {
                    X[head_list[ii] + i] = XX[i];
                }
            }

            // 解答行列に解を代入
            for (int i = 0; i < head_list.Count; i++)
            {
                for (int j = head_list[i]; j < len_list[i]; j++)
                {
                    X[j] = XList[i][j - head_list[i]];
                }
            }              

        }


        /// <summary>
        /// 流量差を求める。
        /// </summary>
        /// <param name="ID">断面インデックス</param>
        /// <param name="HH">水位</param>
        /// <param name="QRatio">流量比</param>
        /// <returns>流量差</returns>
        double Calc_QDif(int ID, double HH, out double QRatio)
        {
            int NBUN = ModelCalInfo.COMHAR_NBUN[ID];
            double QQ = ModelCalInfo.COMCAL_QQ[ID];
            double QSUM = 0.0;
            double DUMMY;
            double AA;

            for (int IB = 0; IB < NBUN; IB++)
            {
                HARP_HWL(ID, IB, HH, out AA, out DUMMY, out DUMMY, out DUMMY);
                QSUM += AA * ModelCalInfo.COMUUU_U[IB, ID];
            }
            QRatio = QSUM / QQ;
            return QSUM - QQ;
        }


        /// <summary>
        /// 運動量成分を求める。
        /// </summary>
        /// <param name="ID">断面インデックス</param>
        /// <param name="H">水位</param>
        /// <param name="E0">速度水頭</param>
        /// <param name="E1">底面摩擦</param>
        /// <param name="E2">樹木摩擦</param>
        void Calc_TERM(int ID, double H, out double E0, out double E1, out double E2)
        {
            const double GRV = 9.8F;   // 重力加速度
            int NBUN = ModelCalInfo.COMHAR_NBUN[ID];

            double AA;       // 断面ブロック流積
            double RR;       // 断面ブロック径深
            double SS;       // 断面ブロック潤辺
            double DUMMY;    // ダミー変数

            double ASUM = 0.0F;   // 断面流積 

            E0 = 0.0;
            E1 = 0.0;
            E2 = 0.0;

            for (int IB = 0; IB < NBUN; IB++)
            {
                double U = ModelCalInfo.COMUUU_U[IB, ID];
                double n = ModelCalInfo.COSODO_FNN[ID][IB];
                HARP_HWL(ID, IB, H, out AA, out RR, out SS, out DUMMY);
                if (RR <= 0.0F)
                {
                    continue;
                }
                ASUM += AA;
                E0 += U * U * AA;
                E1 += n * n / Math.Pow(RR, 1.0 / 3.0) * U * U * SS;
                          
            }

            if (ASUM <= 0.0F)
            {
                E0 = 0.0;
                E1 = 0.0;
                E2 = 0.0;
                return;
            }

            E0 = E0 / (2.0F * GRV * ASUM);
            E1 = E1 / ASUM;

            for (int IB = 0; IB < NBUN; IB++)
            {
                double U = ModelCalInfo.COMUUU_U[IB, ID];
                double F0S0 = ModelCalInfo.COMWET_F0[ID][IB] * ModelCalInfo.COMWET_S0[ID][IB];
                double F2S2 = ModelCalInfo.COMWET_F2[ID][IB] * ModelCalInfo.COMWET_S2[ID][IB];
                E2 += F0S0 * U * U + F2S2 * U * U;
            }
            E2 = E2 / (GRV * ASUM);
        }


        /// <summary>
        /// 準二次元不等流計算のための次ループの水位セット及び収束判定
        /// </summary>
        /// <param name="loop">収束ループの回数</param>
        /// <param name="FS_DIF"></param>
        /// <param name="FS_DIF0"></param>
        /// <param name="Hmin"></param>
        /// <param name="HH"></param>
        /// <param name="Hmax"></param>
        /// <returns></returns>
        bool Chk_HH(int loop, double FS0, double FS1, double Hmin, ref double DIF0, ref double Hold, ref double HH, ref double Hnew, ref int CStep)
        {
            const double EPS = 0.005F;

            double DIF = FS1 - FS0;

            if (Math.Abs(DIF) <= EPS)
            {
                return true;
            }

            if (loop == 0)
            {
                CStep = 0;
                DIF0 = DIF;
                Hold = HH;
                return false;
            }

            // ステップ１
            if (CStep == 0)
            {
                if (DIF0 * DIF >= 0.0)
                {
                    Hold = HH;
                    HH -= DIF0 >= 0.0F ? 0.1 : -0.1;
                    HH = Math.Max(HH, Hmin + 0.1);
                    Hnew = HH;
                    return false;
                }
                else
                {
                    HH = 0.5 * (Hold + Hnew);
                    CStep = 1;
                    return false;
                }
            }

            // ステップ２
            if (CStep == 1)
            {
                if (DIF0 * DIF > 0.0)
                {
                    Hold = HH;
                }
                else
                {
                    Hnew = HH;
                }
                HH = 0.5F * (Hold + Hnew);
            }
            HH = Math.Max(HH, Hmin + 0.001);
            return false;
        }

        /// <summary>
        /// フルード数により水位を補正する
        /// </summary>
        /// <param name="ID">断面インデックス</param>
        /// <param name="HH">水位</param>
        void Adjust_HH(int ID, ref double HH)
        {
            double QQ = ModelCalInfo.COMCAL_QQ[ID];    // 断面流量
            double FR;                                   // フルード数
            double V;                                    // 断面流速
            double ASUM;                                 // 流積
            double RC;                                   // 合成径深
            double WWW;                                  // 水面幅
            double EGrad;                                // エネルギー勾配（仮定）
            double DGrad;                                // エネルギー勾配（決定）
            double QRatio;
            Calc_FROUDE(ID, HH, QQ, out FR, out V, out ASUM, out RC, out WWW);

            // フルード数0.95に対する水位を求める。
            if (FR >= 0.95F)
            {
                FR = 0.95F;
                Calc_FR2H(ID, FR, ref HH, out V, out ASUM, out RC, out WWW);

                if (ID == 0)
                {
                    EGrad = Calc_GRAD(ID + 1, HH);
                }
                else
                {
                    EGrad = Calc_GRAD(ID, HH);
                }

                Calc_UUU(ID, HH, EGrad, out DGrad, out QRatio);
                ModelCalInfo.COMCAL_QRTO[ID] = QRatio;
            }
            else
            {
                ModelCalInfo.COMCAL_QRTO[ID] = QQ / ModelCalInfo.COMCAL_QQ[ID];
            }

            ModelCalInfo.COMCAL_CAL[ID] = HH;
            ModelCalInfo.COMCAL_DEP[ID] = RC;
            ModelCalInfo.COMCAL_AA[ID] = ASUM;
            ModelCalInfo.COMCAL_FRR[ID] = FR;
            ModelCalInfo.COMCAL_UU[ID] = V;
            ModelCalInfo.COMCAL_BBB[ID] = WWW;
            
        }

        /// <summary>
        /// フルード数に対する水位を求める。
        /// </summary>
        /// <param name="ID">断面インデックス</param>
        /// <param name="FR0">フルード数</param>
        /// <param name="HH">水位</param>
        /// <param name="V">流速</param>
        /// <param name="ASUM">流積</param>
        /// <param name="RC">合成径深</param>
        /// <param name="WWW">水面幅</param>
        void Calc_FR2H(int ID, double FR0, ref double HH, out double V, out double ASUM, out double RC, out double WWW)
        {
            const double EPS = 0.0001F;
            double QQ = ModelCalInfo.COMCAL_QQ[ID];    // 断面流量
            double FR;                                   // フルード数
            double Hmin = ModelCalInfo.COMHAR_HH[0, ID];
            double Hmax = ModelCalInfo.COMHAR_HH[McQuasi2DSteadyFlowModelCalInfo.FLOW00_MD1 - 1, ID];
            double Hbed = Hmin;

            V = 0.0;
            ASUM = 0.0;
            RC = 0.0;
            WWW = 0.0;

            // フルード数0.95に対する水位を求める。!出発水位は固定 150831_JCCAJSCE
            if (ID == 0)
            {
               HH = HH;
            }
            else
            {
             for (int i_fr = 0; i_fr < 100; i_fr++)
             {
                Calc_FROUDE(ID, HH, QQ, out FR, out V, out ASUM, out RC, out WWW);
                if (FR >= FR0)
                {
                    Hmin = HH;
                    HH = 1.1F * (HH - Hbed) + Hbed;
                }
                else
                {
                    Hmax = HH;
                    HH = 0.98F * (HH - Hbed) + Hbed;
                }

                // 収束判定
                if (Math.Abs(FR - FR0) < EPS)
                {
                    break;
                }
              }
            }
        }

        /// <summary>
        /// 合流による水位上昇高を計算する。
        /// </summary>
        /// <param name="BB1"></param>
        /// <param name="BB2"></param>
        /// <param name="BB3"></param>
        /// <param name="ANG1"></param>
        /// <param name="ANG2"></param>
        /// <param name="QQ1"></param>
        /// <param name="QQ2"></param>
        /// <param name="QQ3"></param>
        /// <param name="DEP">合流後水深</param>
        /// <returns></returns>
        double Calc_DH01(double BB1, double BB2, double BB3, double ANG1, double ANG2, double QQ1, double QQ2, double QQ3, double DEP)
        {
            const double GRV = 9.8;

            // 川幅が未定義の場合
            if (BB1 * BB2 * BB3 <= 0.0)
            {
                return 0.0;
            }
            // 流量が未定義の場合
            if (QQ1 * QQ2 <= 0.0)
            {
                return 0.0;
            }

            double BETA = 1.1;
            double COS1 = Math.Cos(2.0 * Math.PI / 360.0 * ANG1);
            double COS2 = Math.Cos(2.0 * Math.PI / 360.0 * ANG2);
            double A1 = (COS2 * (BB2 / BB3) + COS1 * (BB1 / BB3)) / (DEP * DEP * DEP);
            double FR2 = QQ3 * QQ3 / (GRV * BB3 * BB3 * DEP * DEP * DEP);
            double A2 = -(1.0 + 2.0 * BETA * FR2) / DEP;
            double A3 = 2.0 * BETA * ((((QQ2 * QQ2) / (QQ3 * QQ3)) * (BB3 / BB2)) * COS2 + (((QQ1 * QQ1) / (QQ3 * QQ3)) * (BB3 / BB1) * COS1)) * FR2;

            // ニュートン・ラプソン法による収束計算
            double HH0 = DEP;
            double HH1 = 9999.9;
            double EPS = 0.0001;
            for (int i = 0; i < 100; i++)
            {
                double FX = A1 * HH0 * HH0 * HH0 + A2 * HH0 + A3;
                double DFX = 3.0 * A1 * HH0 * HH0 + A2;
                if (DFX == 0.0)
                {
                    HH1 = DEP;
                    break;
                }
                HH1 = HH0 - FX / DFX;
                if (Math.Abs(HH1 - HH0) <= EPS)
                {
                    break;
                }
                HH0 = HH1;
            }

            // 水位上昇の値がマイナスになる場合
            if (HH1 < DEP)
            {
                HH1 = DEP;
            }

            return HH1 - DEP;


        }

        /// <summary>
        /// 橋脚による水位上昇を計算する。
        /// </summary>
        /// <param name="Q">流量</param>
        /// <param name="CD">橋脚の形状係数</param>
        /// <param name="DEP">橋脚の下流側の水深</param>
        /// <param name="B1">橋脚の上流側の水路幅</param>
        /// <param name="B2">全水路幅から橋脚断面幅を引いたもの</param>
        /// <returns>橋脚による堰上げ高</returns>
        double Calc_DH02(double Q, double CD, double DEP, double B1, double B2)
        {
            
            const double GRV = 9.8;
            const double EPS0 = 1.0E-4;
            const double EPS1 = 1.0E-6;

            double DH0 = 0.0;
            double DH1 = 0.0;

            if (CD <= 0.0)
            {
                CD = 1.0;
            }

            if (CD * B2 >= B1)
            {
                return 0.0;
            }

            // 収束計算
            for (int i = 0; i < 100; i++)
            {

                DH1 = Q * Q / (2.0 * GRV) * (Math.Pow(1.0 / (CD * B2 * DEP), 2.0) - Math.Pow(1.0 / (B1 * (DEP + DH0)), 2.0));


                if(Math.Abs(DH1 - DH0) <= EPS0)
                {
                    break;
                }

                if (Math.Abs(DH1 - DH0) / DH1 <= EPS1)
                {
                    break;
                }

                DH0 = DH1;
            }

            return DH1;
        }

        /// <summary>
        /// 湾曲による局所的な水位上昇を計算する。
        /// </summary>
        void Calc_DH03()
        {
            const double GRV = 9.8;
            for (int IR = 0; IR < ModelCalInfo.COMDAN_IRIV; IR++)
            {
                int IS = ModelCalInfo.COMDAN_IDS[IR];
                int IE = ModelCalInfo.COMDAN_IDE[IR];

                int IID = 0;                            // 当該河道の下流端を起点とするインデックス
                int n_ConSN = 0;                        // 連続断面数計算用変数
                int[] ConSN = new int[IE - IS + 1];     // 連続断面数
                double Rpre = 0.0;                      // 1断面下流の湾曲曲率半径
                double SumB = 0.0;                      // 湾曲区間の曲率半径の平均値を求めるための水面幅合計値
                double SumV = 0.0;                      // 湾曲区間の曲率半径の平均値を求めるための流速合計値

                // 湾曲区間における平均曲率半径、平均水面幅、平均流速 
                double[] AveR = new double[IE - IS + 1];
                double[] AveB = new double[IE - IS + 1];
                double[] AveV = new double[IE - IS + 1];
                // 当該断面の水面幅
                double[] BBB = new double[IE - IS + 1];
                double[] RRR = new double[IE - IS + 1];
                // 区間連続断面数を求めるループ
                for (int ID = IS; ID <= IE; ID++)
                {

                    // 隣接する断面の曲率半径が同一の区間を一つの湾曲区間と見なす。
                    double R = ModelCalInfo.COMWAN_R[ID];
                    AveR[IID] = ModelCalInfo.COMWAN_R[ID];
                    AveB[IID] = ModelCalInfo.COMCAL_BBB[ID];
                    AveV[IID] = ModelCalInfo.COMCAL_UU[ID];
                    BBB[IID] = ModelCalInfo.COMCAL_BBB[ID];
                    RRR[IID] = ModelCalInfo.COMWAN_R[ID];


                    if (R == Rpre && R != 0.0f && R / BBB[IID] <= 10.0)
                    {
                        ConSN[IID] = ++n_ConSN;
                    }
                    else
                    {
                        n_ConSN = 0;
                        ConSN[IID] = n_ConSN;
                    }

                    Rpre = R;
                    IID++;
                }

                // 湾曲区間の平均値を求める。
                int i = IE - IS;
                while (i >= 0)
                {
                    if (ConSN[i] == 0)
                    {
                        i--;
                        continue;
                    }
                    for (int j = i; j >= i - ConSN[i]; j--)
                    {
                        SumB += AveB[j];
                        SumV += AveV[j];
                    }
                    AveB[i] = SumB / (ConSN[i] + 1);
                    AveV[i] = SumV / (ConSN[i] + 1);
                    for (int k = i; k >= i - ConSN[i]; k--)
                    {
                        AveB[k] = AveB[i];
                        AveV[k] = AveV[i];
                    }
                    i = i - ConSN[i] - 1;
                    SumB = 0.0;
                    SumV = 0.0;
                }

                // 湾曲による水面上昇を求める。
                IID = 0;
                for (int ID = IS; ID <= IE; ID++)
                {
                    if (RRR[IID] > 9000.0 || RRR[IID] <= 100.0 || RRR[IID] / BBB[IID] >= 10.0)
                    {
                        IID++;
                        continue;
                    }
                    ModelCalInfo.COMCAL_DH03[ID] = 0.5 * AveB[IID] * AveV[IID] * AveV[IID] / (AveR[IID] * GRV);
                    IID++;


                }
            }

        }

        /// <summary>
        /// 砂州による局所的な水位上昇を計算する。
        /// </summary>
        void Calc_DH04()
        {

            for (int ID = 0; ID < ModelCalInfo.COMDAN_IDAN; ID++)
            {
                double GRAD = ModelCalInfo.COMTEN_GRD[ID];

                if (GRAD > 2000.0)
                {
                    ModelCalInfo.COMCAL_DH04[ID] = Calc_DH04_GENTLE(ID);
                }
                else
                {
                    ModelCalInfo.COMCAL_DH04[ID] = Calc_DH04_STEEP(ID);
                }
            }

        }

        /// <summary>
        /// 砂州による局所的な水位上昇を計算する（勾配1/2000以下）
        /// </summary>
        /// <param name="ID">断面インデックス</param>
        /// <returns>水位上昇量</returns>
        double Calc_DH04_GENTLE(int ID)
        {
            double DEP = ModelCalInfo.COMCAL_CAL[ID] - ModelCalInfo.COMXYZ_BED[ID]; ;    // 平均水深（計算水位）
            return 0.025 * DEP;

        }

        /// <summary>
        /// 砂州による局所的な水位上昇を計算する（勾配1/2000より急）
        /// </summary>
        /// <param name="ID">断面インデックス</param>
        /// <returns>水位上昇量</returns>
        double Calc_DH04_STEEP(int ID)
        {
            double HL = ModelCalInfo.COMXYZ_HHL[ID] - ModelCalInfo.COMXYZ_BED[ID];  // 低水路満杯流量時水深（平均年最大流量時相当）
            double B = ModelCalInfo.COMXYZ_BBL[ID];                                 // 低水路満杯流量時水面幅（平均年最大流量時相当）
            double B_HL;                                                            // B/HL
            double GRD = ModelCalInfo.COMTEN_GRD[ID];                               // 河床勾配

            // エラーログ
            string log_file = (Path.Combine(m_sWorkingDir, "Err_DH04Steep.log"));
            if (ID == 0 && System.IO.File.Exists(log_file))
            {
                System.IO.File.Delete(log_file);
            }
            StreamWriter sw;

            // 河床勾配と水位上昇高の関係のデータを格納する配列
            double[,] Tbl_DHL =  
            {{100.0, 200.0, 300.0, 400.0, 500.0, 600.0, 700.0, 800.0, 900.0, 1000.0, 1200.0, 1300.0, 1800.0, 2000.0},
             { 19.8,  14.6,  11.9, 10.05,  8.55,  7.40,  6.40,  5.50,  4.75,   4.05,   3.00,   2.50,   2.50,   2.50},
             { 23.0,  17.5,  14.4, 12.30, 10.50,  9.10,  7.80,  6.80,  5.90,   5.00,   3.80,   3.20,   2.50,   2.50},
             { 25.9,  19.9,  16.6, 14.10, 12.30, 10.80,  9.55,  8.45,  7.50,   6.60,   5.10,   4.50,   2.50,   2.50},
             { 30.5,  23.6,  19.6, 16.80, 14.60, 13.00, 11.60, 10.40,  9.30,   8.40,   6.80,   6.10,   3.10,   2.50},
             { 37.0,  29.7,  25.3, 22.30, 20.00, 18.10, 16.50, 15.20, 14.10,  13.00,  11.20,  10.40,   7.30,   6.40}
            };

            int IT = 0, JT0 = 0, JT1 = 0;     // 河床勾配と水位上昇高の関係のデータを格納する配列Tbl_DHLのインデックス
            double P = 0.0;                   // GRDのポジション（GRDLとGRDHの間）の割合表示

            if (HL <= 0.0)
            {
                sw = new StreamWriter(log_file, true, Encoding.GetEncoding("shift_jjis"));
                sw.WriteLine(ModelCalInfo.COMDAN_RIVR[ID] + " " + ModelCalInfo.COMDAN_ID[ID] + 
                    "で低水路満杯水位が定義されてません。当該断面における砂州による水位上昇を0とします。");
                sw.Close();
                return 0.0;
            }
            else
            {
                B_HL = B / HL;
            }

            if (B_HL < 30.0)
            {
                IT = 1;
            }
            else if (B_HL < 40.0)
            {
                IT = 2;
            }
            else if (B_HL < 50.0)
            {
                IT = 3;
            }
            else if (B_HL < 100.0)
            {
                IT = 4;
            }
            else
            {
                IT = 5;
            }

            // 河床勾配に相当するデータ配列のインデックスJTとPを求める。
            int Tbl_Length = Tbl_DHL.GetLength(1);
            if (GRD >= 2000.0)
            {
                JT0 = Tbl_Length - 1;
                JT1 = JT0;
                P = 0.0;
            }
            else if (GRD <= 100.0)
            {
                JT0 = 0;
                JT1 = 0;
                P = 0.0;
            }
            else
            {
                for (int JT = 0; JT < Tbl_DHL.GetLength(1) - 1; JT++)
                {
                    double GRDL = Tbl_DHL[0, JT];
                    double GRDH = Tbl_DHL[0, JT + 1];
                    double PS = (GRD - GRDL) * (GRDH - GRD);

                    if (PS == 0.0)
                    {
                        JT0 = JT + 1;
                        JT1 = JT0;
                        P = 0.0;
                        break;
                    }
                    else if (PS > 0.0)
                    {
                        P = (GRD - GRDL) / (GRDH - GRDL);
                        JT0 = JT;
                        JT1 = JT + 1;
                        break;
                    }
                }

            }


            double R0 = Tbl_DHL[IT, JT0];
            double R1 = Tbl_DHL[IT, JT1];

            if (GRD < 100.0)
            {
                double RateH04 = 0.00;       // 水位上昇率0（勾配1/100より急または空欄の場合） !150831_JCCAJSCE
            return RateH04 * HL;
            }
            else
            {
            double RateH04 = 0.01 * (R0 + P * (R1 - R0));       // 水位上昇率（勾配1/100より緩）
            return RateH04 * HL;
            }   
        }

        /// <summary>
        /// 流量・出発水位の設定のログ出力
        /// </summary>
        void Log_HQRead()
        {
            String log_file = Path.Combine(m_sWorkingDir, "09_Input_HQ") + "_case" + ModelCalInfo.loopCount.ToString() + ".csv";
            StreamWriter sw = new StreamWriter(log_file, false, System.Text.Encoding.GetEncoding("shift_jis"));
            sw.WriteLine("[下流端水位]");
            sw.WriteLine(inputFiles.throughFile.sections[0].records[0].WaterLevel.ToString());
            sw.WriteLine();
            sw.WriteLine("[流量配分]");
            
            for (int i = 0; i < inputFiles.throughFile.sections.Count; i++)
            {
                sw.WriteLine(ModelCalInfo.COMDAN_RIVR[ModelCalInfo.COMDAN_IDS[i]]);
                sw.WriteLine("断面ID,流量");
                for (int j = 0; j < inputFiles.throughFile.sections[i].records.Count; j++)
                {
                    sw.WriteLine(inputFiles.throughFile.sections[i].records[j].DanmenId.ToString() + "," +
                        inputFiles.throughFile.sections[i].records[j].Flow.ToString());
                }
                sw.WriteLine();
            }
            sw.Close();

        }

        /// <summary>
        /// 演算結果のログ出力     // !150831_JCCAJSCE
        /// </summary>
        void Log_Calc()
        {
            String log_file = Path.Combine(m_sWorkingDir, "10_Output_Calc") + "_case" + ModelCalInfo.loopCount.ToString() + ".csv";
            StreamWriter sw = new StreamWriter(log_file, false, System.Text.Encoding.GetEncoding("shift_jis"));
            sw.WriteLine("[計算結果一覧]");
            sw.WriteLine("断面インデックス,河川名,topoID,断面ID,累加距離,水位 ,DH01,DH02,DH03,DH04,流量,流積,径深,水面幅,流速,フルード数,エネルギー勾配,摩擦速度");
            sw.WriteLine("nd,nd,nd,nd,m,T.P.m,m,m,m,m,m3/s,m2,m,m,m/s,nd,nd,m/s");

            for (int i = 0; i < ModelCalInfo.COMDAN_IDAN; i++)
            {
                    sw.WriteLine(i.ToString() + "," + ModelCalInfo.COMDAN_RIVR[i] + "," + ModelCalInfo.COMDAN_TOPO[i] + "," +
                    ModelCalInfo.COMDAN_RIVR[i] + "," + ModelCalInfo.COMDAN_FL[i] + "," +
                    ModelCalInfo.COMCAL_CAL[i].ToString() + "," + ModelCalInfo.COMCAL_DH01[i].ToString() + "," + 
                    ModelCalInfo.COMCAL_DH02[i].ToString() + "," + ModelCalInfo.COMCAL_DH03[i].ToString() + "," +
                    ModelCalInfo.COMCAL_DH04[i].ToString() + "," + 
                    ModelCalInfo.COMCAL_QQ[i].ToString() + "," + //流量
                    ModelCalInfo.COMCAL_AA[i].ToString() + "," + ModelCalInfo.COMCAL_DEP[i].ToString() + "," +
                    ModelCalInfo.COMCAL_BBB[i].ToString() + "," +
                    ModelCalInfo.COMCAL_UU[i].ToString() + "," +
                    ModelCalInfo.COMCAL_FRR[i].ToString() + "," +
                    ModelCalInfo.COMCAL_EGRD[i].ToString() + "," +
                    Math.Sqrt(9.8 * ModelCalInfo.COMCAL_DEP[i] * Math.Abs(ModelCalInfo.COMCAL_EGRD[i]))); //摩擦速度(径深・エネルギー勾配より)150831_JCCAJSCE
                    //ModelCalInfo.COMCAL_QRTO[i].ToString()); //流量比
            }
            sw.Close();

        }
/*
        /// <summary>
        /// 断面ブロック流速をログ出力する。
        /// </summary>
        void Log_UUU()
        {
            String log_file = Path.Combine(m_sWorkingDir, "11_UUU") + "_case" + ModelCalInfo.loopCount.ToString() + ".csv";
            StreamWriter sw = new StreamWriter(log_file, false, System.Text.Encoding.GetEncoding("shift_jis"));
            for (int i = 0; i < ModelCalInfo.COMDAN_IDAN; i++)
            {
                sw.WriteLine(i.ToString() + "," + ModelCalInfo.COMDAN_RIVR[i] + "," + ModelCalInfo.COMDAN_ID[i]);
                sw.WriteLine("IB,UUU");
                for (int j = 0; j < ModelCalInfo.COMHAR_NBUN[i]; j++)
                {
                    sw.WriteLine(j.ToString() + "," + ModelCalInfo.COMUUU_U[j, i].ToString());                    
                }
                sw.WriteLine();
            }
            sw.Close();
        }
*/
        /// <summary>
        /// ブロック潤辺長・境界混合係数設定ログ出力
        /// </summary>
        void Log_SSSF()
        {
            String log_file = Path.Combine(m_sWorkingDir, "11_Output_Factor") + "_case" + ModelCalInfo.loopCount.ToString() + ".csv";
            StreamWriter sw = new StreamWriter(log_file, false, System.Text.Encoding.GetEncoding("shift_jis"));
            sw.WriteLine("[境界混合係数設定]");
            for (int i = 0; i < ModelCalInfo.COMDAN_IDAN; i++)
            {
                sw.WriteLine(i.ToString() + "," + ModelCalInfo.COMDAN_RIVR[i] + "," +  ModelCalInfo.COMDAN_ID[i]);
                //sw.WriteLine("IB,左岸(TREE)S0,左岸(WATER)S1,右岸(TREE)S2,右岸(TREE)S3,左岸(TREE)F0,左岸(WATER)F1,右岸(TREE)F2,右岸(TREE)F3");
                sw.WriteLine("IB,左岸(TREE)F0,左岸(WATER)F1,右岸(TREE)F2,右岸(TREE)F3");
                for (int j = 0; j < ModelCalInfo.COMHAR_NBUN[i]; j++)
                {
                    sw.WriteLine(j.ToString() + "," + 
                        //ModelCalInfo.COMWET_S0[i][j].ToString() + "," + ModelCalInfo.COMWET_S1[i][j].ToString() + "," + 
                        //ModelCalInfo.COMWET_S2[i][j].ToString() + "," + ModelCalInfo.COMWET_S3[i][j].ToString() + "," + 
                        ModelCalInfo.COMWET_F0[i][j].ToString() + "," + ModelCalInfo.COMWET_F1[i][j].ToString() + "," + 
                        ModelCalInfo.COMWET_F2[i][j].ToString() + "," + ModelCalInfo.COMWET_F3[i][j].ToString());
                }
                sw.WriteLine();
            }
        }
/*
        /// <summary>
        /// 支川合流の流量のログ出力
        /// </summary>
        void Log_QDH01()
        {
            String log_file = Path.Combine(m_sWorkingDir, "16_QDH01") + "_case" + ModelCalInfo.loopCount.ToString() + ".csv";
            StreamWriter sw = new StreamWriter(log_file, false, System.Text.Encoding.GetEncoding("shift_jis"));
            sw.WriteLine("IR,Q1,Q2,Q3");
            for (int IRIV = 0; IRIV < ModelCalInfo.COMDAN_IRIV; IRIV++)
            {
                if (IRIV == 0)
                {
                    continue;
                }
                int ID = ModelCalInfo.COMDAN_IDS[IRIV];
                int IDL = ModelCalInfo.COMDAN_IDL[ID];
                double Q1 = ModelCalInfo.COMCAL_QQ[IDL + 1];
                double Q2 = ModelCalInfo.COMCAL_QQ[ID];
                double Q3 = ModelCalInfo.COMCAL_QQ[IDL];
                sw.WriteLine(IRIV.ToString() + "," + Q1.ToString() + "," + Q2.ToString() + "," + Q3.ToString());
            }
            sw.Close();
        }
*/
        #endregion

        // ファイル出力
        #region File_RESULT


        /// <summary>
        /// 演算結果のファイル出力
        /// </summary>
        void File_RESULT()
        {
            StreamWriter outputWriter = null;
            for (int IR = 0; IR < ModelCalInfo.COMDAN_IRIV; IR++)
            {
                try
                {
                    string channel_name = ModelCalInfo.COMDAN_RIVR[ModelCalInfo.COMDAN_IDS[IR]];    // 河道名
                    String outputFilename = Path.GetFileNameWithoutExtension(m_sOutputFile) + "_" + channel_name + "_" + ModelCalInfo.loopCount.ToString();
                    outputFilename = outputFilename + Path.GetExtension(m_sOutputFile);
                    outputFilename = Path.Combine(Path.GetDirectoryName(m_sOutputFile), outputFilename);

                    outputWriter = new StreamWriter(outputFilename, false, McQuasi2DSteadyFlowModelDefine.FILE_ENCODING);
                    const double GRV = 9.8;

                    /*
                    outputWriter.WriteLine("河川名,topoID,断面ID,累加距離,水位,湾曲,砂州,合計水位,流量,河積,流速,フルード数,水頭");
                    outputWriter.WriteLine("char  ,char  ,char  ,double  ,double,double,double,double,double,double,double,double,double");
                    outputWriter.WriteLine("nd    ,nd    ,nd    ,m       ,T.P.m,m,m,T.P.m,m3/s,m2,m/s,nd,m");
                    */

                    //分割断面出力用
                    int MaxNum = 0;
                    for (int I = 0; I < ModelCalInfo.COMDAN_IDAN; I++)
                    {
                        if (MaxNum < ModelCalInfo.COMHAR_NBUN[I])
                        {
                            MaxNum = ModelCalInfo.COMHAR_NBUN[I];
                        }
                    }

                    StringBuilder str = new StringBuilder();
                    str.Append("河川名,topoID,断面ID,累加距離"
                             + ",水位,水位(湾曲考慮),水位(砂州考慮),合計水位,水頭,水位1,水位2,水位3,水位4,水位5"
                             + ",流量,河積,径深,水面幅,流速,フルード数,エネルギー勾配");
                    for (int I = 0; I < MaxNum; I++)
                    {
                        str.AppendFormat(",分割断面番号{0},X座標(L){0},X座標(R){0}"
                                       + ",分割断面積{0},分割断面径深{0},分割断面流速{0}"
                                       + ",摩擦速度{0},流体力{0},分割断面底面粗度係数{0}"
                                       + ",水-水境界混合係数(L){0},水-水境界混合係数(R){0}"
                                       + ",水-樹木境界混合係数(L){0},水-樹木境界混合係数(R){0}", I + 1);
                    }
                    //str.AppendLine();
                    outputWriter.WriteLine(str.ToString());

                    str.Remove(0, str.Length);
                    str.Append("char  ,char  ,char  ,double"
                             + ",double,double,double,double,double,double,double,double,double,double"
                             + ",double,double,double,double,double,double,double");
                    for (int I = 0; I < MaxNum; I++)
                    {
                        str.Append(",int,double,double"
                                 + ",double,double,double"
                                 + ",double,double,double"
                                 + ",double,double"
                                 + ",double,double");
                    }
                    //str.AppendLine();
                    outputWriter.WriteLine(str.ToString());

                    str.Remove(0, str.Length);
                    str.Append("nd    ,nd    ,nd    ,m"
                             + ",T.P.m,T.P.m,T.P.m,T.P.m,T.P.m,T.P.m,T.P.m,T.P.m,T.P.m,T.P.m"
                             + ",m3/s,m2,m,m,m/s,nd,nd");
                    for (int I = 0; I < MaxNum; I++)
                    {
                        str.Append(",nd,m,m"
                                 + ",m2,m,m/s"
                                 + ",m/s,Kgm,m-1/3s"
                                 + ",nd,nd"
                                 + ",nd,nd");
                    }
                    //str.AppendLine();
                    outputWriter.WriteLine(str.ToString());

                    outputWriter.WriteLine("[start]");
                    //for (int I = 0; I < ModelCalInfo.COMDAN_IDAN; I++) // 2015.05.01河道ごとにファイル出力するように修正
                    for(int I = ModelCalInfo.COMDAN_IDS[IR]; I <= ModelCalInfo.COMDAN_IDE[IR]; I++) 
                    {
                        //if (0 < I)
                        //{
                        //    if (ModelCalInfo.COMDAN_RIVR[I] != ModelCalInfo.COMDAN_RIVR[I - 1]
                        //        || ModelCalInfo.COMDAN_TOPO[I] != ModelCalInfo.COMDAN_TOPO[I - 1])
                        //    {
                        //        outputWriter.WriteLine("[end]");
                        //        outputWriter.WriteLine("[start]");
                        //    }
                        //}

                        outputWriter.Write("{0},", ModelCalInfo.COMDAN_RIVR[I]);
                        outputWriter.Write("{0},", ModelCalInfo.COMDAN_TOPO[I]);
                        outputWriter.Write("{0},", ModelCalInfo.COMDAN_ID[I]);
                        outputWriter.Write("{0},", ModelCalInfo.COMDAN_FL[I]);

                        outputWriter.Write("{0,10:F3},", ModelCalInfo.COMCAL_CAL[I]); //E-5 水位
                        outputWriter.Write("{0,10:F3},", ModelCalInfo.COMCAL_CAL[I] + ModelCalInfo.COMCAL_DH03[I]);//E-6 水位(湾曲考慮)
                        outputWriter.Write("{0,10:F3},", ModelCalInfo.COMCAL_CAL[I] + ModelCalInfo.COMCAL_DH04[I]);//E-7 水位(砂州考慮)
                        outputWriter.Write("{0,10:F3},", ModelCalInfo.COMCAL_CAL[I] + Math.Max(ModelCalInfo.COMCAL_DH03[I], ModelCalInfo.COMCAL_DH04[I]));//E-8 合計水位
                        outputWriter.Write("{0,10:F2},", ModelCalInfo.COMCAL_CAL[I] + ModelCalInfo.COMCAL_UU[I] * ModelCalInfo.COMCAL_UU[I] / (2 * GRV));//E-13 水頭

                        outputWriter.Write(",");// 水位1
                        outputWriter.Write(",");// 水位2
                        outputWriter.Write(",");// 水位3
                        outputWriter.Write(",");// 水位4
                        outputWriter.Write(",");// 水位5

                        outputWriter.Write("{0,10:F3},", ModelCalInfo.COMCAL_QQ[I]);//E-9 流量
                        outputWriter.Write("{0,10:F3},", ModelCalInfo.COMCAL_AA[I]);//E-10 河積
                        outputWriter.Write("{0,10:F3},", ModelCalInfo.COMCAL_DEP[I]); //径深
                        outputWriter.Write("{0,10:F3},", ModelCalInfo.COMCAL_BBB[I]); //水面幅
                        outputWriter.Write("{0,10:F3},", ModelCalInfo.COMCAL_UU[I]);  //E-11 流速
                        outputWriter.Write("{0,10:F3},", ModelCalInfo.COMCAL_FRR[I]); //E-12 フルード数
                        outputWriter.Write("{0,10:F7},", ModelCalInfo.COMCAL_EGRD[I]); //エネルギー勾配

                        for (int J = 0; J < ModelCalInfo.COMHAR_NBUN[I]; J++)
                        {
                            double XBNDL;
                            if (J == 0)
                            {
                                XBNDL = ModelCalInfo.COMXYZ_XZ[0, I];
                            }
                            else
                            {
                                XBNDL = ModelCalInfo.COMXYZ_XZ[ModelCalInfo.COSODO_IDX[I][J - 1], I];
                            }
                            double XBNDR = ModelCalInfo.COMXYZ_XZ[ModelCalInfo.COSODO_IDX[I][J], I];
                            double HH = ModelCalInfo.COMCAL_CAL[I];
                            double AA, RR, SS, BB;
                            HARP_HWL(I, J, HH, out AA, out RR, out SS, out BB);
                            double UU = ModelCalInfo.COMUUU_U[J, I];
                            double FNN = ModelCalInfo.COSODO_FNN[I][J];
                            outputWriter.Write("{0,10:d},", J + 1); //分割断面番号
                            outputWriter.Write("{0,10:F3},", XBNDL); //X座標(L)
                            outputWriter.Write("{0,10:F3},", XBNDR); //X座標(R)
                            outputWriter.Write("{0,10:F3},", AA);//分割断面積
                            outputWriter.Write("{0,10:F3},", RR);//分割断面径深
                            outputWriter.Write("{0,10:F3},", UU);//分割断面流速


                            // 摩擦速度
                            double Ustr = 0.0;
                            if (RR > 0.0)
                            {
                                Ustr = Math.Sqrt(GRV * UU * UU * FNN * FNN / Math.Pow(RR, 1.0 / 3.0));
                            }

                            // 流体力
                            double Flf = 0.0d;
                            if (ModelCalInfo.COSODO_ISTREE[I][J])
                            {
                                double W = ModelCalInfo.COSODO_BW[I][J]; //幅
                                double Htree = ModelCalInfo.COSODO_TTH[I][J]; // 樹高
                                double HtreeTop = ModelCalInfo.COSODO_THH[I][J]; // 樹冠標高
                                double Hbed = HtreeTop - Htree; //河床高
                                int NBUN = ModelCalInfo.COMHAR_NBUN[I];

                                double dH = 0.0; // 流体力を受ける部分の高さ方向の長さ
                                if (HH > HtreeTop)
                                {
                                    dH = HtreeTop - Hbed; //高さ
                                }
                                else
                                {
                                    dH = Math.Max(HH - Hbed, 0.0); //高さ
                                }

                                // 樹木群に作用する流速（左右の断面ブロックの流速を使用）
                                double dU = 0.0;
                                if (NBUN > 1)
                                {
                                    if (J == 0)
                                    {
                                        for (int K = 1; K < NBUN; K++)
                                        {
                                            if (!ModelCalInfo.COSODO_ISTREE[I][K])
                                            {
                                                dU = ModelCalInfo.COMUUU_U[K, I];
                                                break;
                                            }
                                        }
                                    }
                                    else if (J == NBUN - 1)
                                    {
                                        for (int K = NBUN - 1; K >= 0; K--)
                                        {
                                            if (!ModelCalInfo.COSODO_ISTREE[I][K])
                                            {
                                                dU = ModelCalInfo.COMUUU_U[K, I];
                                                break;
                                            }
                                        }
                                    }
                                    else
                                    {
                                        double dUL = 0.0;
                                        int flgL = 0;
                                        for (int K = J - 1; K >= 0; K--)
                                        {
                                            if (!(ModelCalInfo.COSODO_ISTREE[I][K]))
                                            {
                                                dUL = ModelCalInfo.COMUUU_U[K, I];
                                                flgL = 1;
                                                break;
                                            }
                                        }
                                        double dUR = 0.0;
                                        int flgR = 0;
                                        for (int K = J + 1; K < NBUN; K++)
                                        {
                                            if (!ModelCalInfo.COSODO_ISTREE[I][K])
                                            {
                                                dUR = ModelCalInfo.COMUUU_U[K, I];
                                                flgR = 1;
                                                break;
                                            }
                                        }
                                        if (flgL == 1 && flgR == 1)
                                        {
                                            dU = (dUL + dUR) / 2.0;
                                        }
                                        else
                                        {
                                            dU = (double)flgL * dUL + (double)flgR * dUR;
                                        }
                                    }
                                }
                                double Cd = 1.2;
                                double r0 = 102.0;
                                Flf = r0 * Cd * W * dH * dH * dU * dU / 4.0;
                            }

                            double F0 = ModelCalInfo.COMWET_F0[I][J];
                            double F1 = ModelCalInfo.COMWET_F1[I][J];
                            double F2 = ModelCalInfo.COMWET_F2[I][J];
                            double F3 = ModelCalInfo.COMWET_F3[I][J];
                            
                            outputWriter.Write("{0,10:F3},", Ustr); //摩擦速度
                            outputWriter.Write("{0,10:F3},", Flf); //流体力
                            outputWriter.Write("{0,10:F3},", FNN);//分割断面底面粗度係数
                            outputWriter.Write("{0,10:F3},", F1); //水-水境界混合係数(L)
                            outputWriter.Write("{0,10:F3},", F3); //水-水境界混合係数(R)
                            outputWriter.Write("{0,10:F3},", F0); //水-樹木境界混合係数(L)
                            outputWriter.Write("{0,10:F3},", F2); //水-樹木境界混合係数(R

                        }

                        outputWriter.WriteLine("");
                    }
                    outputWriter.WriteLine("[end]");

                }
                finally
                {
                    if (outputWriter != null)
                    {
                        outputWriter.Close();
                    }
                }
            }
        }

        #endregion

        // テキストデータの読み取り
        #region Formatted Text I/O
        class FormattedTextIO
        {
            /// <summary>
            /// 2バイト文字対応版Length
            /// </summary>
            /// <param name="strData">文字列</param>
            /// <returns>バイト長</returns>
            public static int Length(string strData)
            {
                ArrayList byteData = new ArrayList(McQuasi2DSteadyFlowModelDefine.FILE_ENCODING.GetBytes(strData));
                return byteData.Count;
            }

            /// <summary>
            /// 2バイト文字対応版SubString(ファイルからの読込み時用)
            /// </summary>
            /// <param name="strData">文字列</param>
            /// <param name="startIndex">開始位置(0～)</param>
            /// <param name="iLength">文字長</param>
            /// <returns>切り出した文字列</returns>
            public string SubString(string strData, int startIndex, int iLength)
            {
                string ret = "";
                ArrayList byteData = new ArrayList(McQuasi2DSteadyFlowModelDefine.FILE_ENCODING.GetBytes(strData));
                byte[] getData = null;
                getData = new byte[iLength];
                byteData.GetRange(startIndex, iLength).CopyTo(getData);
                ret = McQuasi2DSteadyFlowModelDefine.FILE_ENCODING.GetString(getData);
                return ret;
            }
        }
        class FormattedTextReader : FormattedTextIO
        {
            /// <summary>
            /// 入力用ファイル
            /// </summary>
            StreamReader textFile = null;

            /// <summary>
            /// 文字列切り出し数
            /// </summary>
            int tNum = 0;

            /// <summary>
            /// 切り出しバイト数の配列
            /// </summary>
            int[] N = new int[1000];

            /// <summary>
            /// 切り出した文字列の配列
            /// </summary>
            string[] STR = new string[1000];

            /// <summary>
            /// １行読込文字列
            /// </summary>
            string buffer = null;

            /// <summary>
            /// 読込文字列のバイト長(SJIS)
            /// strLen_1=strLen_2の場合、全角文字が含まれていない
            /// </summary>
            int strLen_1 = 0;

            /// <summary>
            /// 読込文字列の文字数(UNICODE)
            /// strLen_1=strLen_2の場合、全角文字が含まれていない
            /// </summary>
            int strLen_2 = 0;

            /// <summary>
            /// ファイルをREADオープンする
            /// </summary>
            /// <param name="fname">ファイル名</param>
            public FormattedTextReader(Stream stream)
            {
                textFile = new StreamReader(stream, McQuasi2DSteadyFlowModelDefine.FILE_ENCODING);
            }

            /// <summary>
            /// ファイルを１行読込む
            /// 後から、バイト単位で切り出す場合用
            /// </summary>
            public void READLINE()
            {
                if (textFile == null)
                {
                    buffer = "";
                    strLen_1 = 0;
                    strLen_2 = 0;
                    return;
                }
                string line = textFile.ReadLine();
                buffer = line;
                strLen_1 = Length(buffer);
                strLen_2 = buffer.Length;
            }

            /// <summary>
            /// 1行読込んだ文字列から文字列を切り出す
            /// </summary>
            /// <param name="sPtr">開始位置(0～)</param>
            /// <param name="sLen">バイト長</param>
            /// <returns>切り出した文字列</returns>
            public string GET(int sPtr, int sLen)
            {
                int sPos = sPtr;
                int ePos = sPos + sLen - 1;
                int len = Length(buffer);
                if (ePos > len - 1)
                    ePos = len - 1;

                if (strLen_1 != strLen_2)
                    return (SubString(buffer, sPos, ePos - sPos + 1));
                else
                    return (buffer.Substring(sPos, ePos - sPos + 1));
            }

            /// <summary>
            /// 切り出すバイト長を指定して文字列を読込む
            /// 引数=1,10,2,5,1,10の場合
            /// 10,5,5,10の順でバイト長単位で合計4要素切り出す
            /// 取得するためにはGET_INT(),GET_DOUBLE()等で0～3を指定する
            /// </summary>
            /// <param name="num">切り出し数とバイト長の対</param>
            /// <returns>切り出し数</returns>
            public int READ(params int[] num)
            {
                if (textFile == null)
                {
                    buffer = "";
                    strLen_1 = 0;
                    strLen_2 = 0;
                    for (int I = 0; I < 1000; I++)
                        STR[I] = "";
                    return (0);
                }

                string line = textFile.ReadLine();
                buffer = line;

                if (buffer == null) return (0);

                strLen_1 = Length(buffer);
                strLen_2 = buffer.Length;

                if (buffer == "")
                {
                    for (int I = 0; I < 1000; I++)
                        STR[I] = "";
                }
                else
                {
                    char c = buffer[0];
                    if (Char.IsControl(c)) return (0);
                }

                int n = num.Length;
                int pNum = n / 2;
                int ptr = 0;
                for (int i = 0; i < pNum; i++)
                {
                    int n1 = num[i * 2];
                    int n2 = num[i * 2 + 1];
                    for (int j = 1; j <= n1; j++)
                    {
                        N[ptr] = n2;
                        ptr++;
                    }
                }
                int tNum0 = ptr;

                int strLen = strLen_1;
                ptr = 0;
                tNum = 0;
                for (int i = 0; i < tNum0; i++)
                {
                    int sPtr = ptr;
                    int n0 = N[i];
                    if (sPtr < strLen)
                    {
                        if (sPtr + n0 < strLen)
                        {
                            string s = null;
                            if (strLen_1 != strLen_2)
                                s = SubString(buffer, sPtr, n0);
                            else
                                s = buffer.Substring(sPtr, n0);
                            STR[i] = s;
                            ptr += n0;
                        }
                        else
                        {
                            n0 = strLen - sPtr;
                            string s = null;
                            if (strLen_1 != strLen_2)
                                s = SubString(buffer, sPtr, n0);
                            else
                                s = buffer.Substring(sPtr, n0);
                            STR[i] = s;
                            ptr += n0;
                        }
                        tNum++;
                    }
                    else
                    {
                        break;
                    }
                }
                return (tNum);
            }

            /// <summary>
            /// 切り出した文字列配列の１要素を実数として取り出す
            /// </summary>
            /// <param name="no">インデクス(0～)</param>
            /// <returns>文字列を実数に変換したもの</returns>
            public double GET_DOUBLE(int no)
            {
                if (0 <= no && no < tNum)
                    if (STR[no] == "")
                        return (0.0);
                    else if (STR[no].Trim() == "")
                        return (0.0);
                    else
                        return (Convert.ToDouble(STR[no]));
                else
                    return (0.0);
            }

            /// <summary>
            /// 切り出した文字列配列の１要素を整数として取り出す
            /// </summary>
            /// <param name="no">インデクス(0～)</param>
            /// <returns>文字列を整数に変換したもの</returns>
            public int GET_INT(int no)
            {
                if (0 <= no && no < tNum)
                    if (STR[no] == "")
                        return (0);
                    else if (STR[no].Trim() == "")
                        return (0);
                    else
                        return (Convert.ToInt32(STR[no]));
                else
                    return (0);
            }

            /// <summary>
            /// 切り出した文字列配列の１要素を文字列として取り出す
            /// </summary>
            /// <param name="no">インデクス(0～)</param>
            /// <returns>文字列</returns>
            public string GET_STRING(int no)
            {
                if (0 <= no && no < tNum)
                    return (STR[no]);
                else
                    return ("");
            }
        }
        class FormattedTextWriter : FormattedTextIO
        {
            /// <summary>
            /// 出力用ファイル
            /// </summary>
            private StreamWriter writer;
            /// <summary>
            /// ファイルをWRITEオープンする
            /// </summary>
            /// <param name="fname">ファイル名</param>
            public FormattedTextWriter(Stream stream)
            {
                writer = new StreamWriter(stream, McQuasi2DSteadyFlowModelDefine.FILE_ENCODING);
            }
            /// <summary>
            /// スペースをファイル出力する
            /// </summary>
            /// <param name="num">スペース数</param>
            public void WRITE_SPACE(int num)
            {
                for (int i = 0; i < num; i++)
                {
                    writer.Write(" ");
                }
                writer.Flush();
            }
            /// <summary>
            /// 整数を右詰でファイル出力する
            /// </summary>
            /// <param name="val">整数値</param>
            /// <param name="num">バイト数</param>
            public void WRITE_INT(long val, int num)
            {
                string str = val.ToString();
                int strLen = str.Length;
                WRITE_SPACE(num - strLen);
                writer.Write(str);
                writer.Flush();
            }
            /// <summary>
            /// 実数を右詰でファイル出力する
            /// </summary>
            /// <param name="val">実数値</param>
            /// <param name="num1">バイト数</param>
            /// <param name="num2">小数点以下桁数</param>
            public void WRITE_DOUBLE(double val, int num1, int num2)
            {
                string s = num2.ToString();
                string str = String.Format("{0:F" + s + "}", val);
                if (-1.0 < val && val < 0.0)
                {
                    string str1 = str.Replace("-0.", "-.");
                    if (str1 != str)
                        str = str1;
                }
                else if (0.0 <= val && val < 1.0)
                {
                    str = str.Replace("0.", " .");
                }
                int strLen = str.Length;
                WRITE_SPACE(num1 - strLen);
                writer.Write(str);
                writer.Flush();
            }
            /// <summary>
            /// 実数を(E±00)形式で右詰でファイル出力する
            /// </summary>
            /// <param name="val">実数値</param>
            /// <param name="num1">バイト数</param>
            /// <param name="num2">小数点以下桁数</param>
            public void WRITE_DOUBLE_E(double val, int num1, int num2)
            {
                double val2 = 1.0;
                string s = "";
                for (int i = 0; i < num2; i++)
                {
                    s += "0";
                    val2 /= 10;
                }
                string str = "";
                if (val >= 0.1 || Math.Abs(val) < val2)
                {
                    str = String.Format("{0:." + s + "E+00}", val);
                }
                else
                {
                    str = String.Format("{0:." + s + "E-00}", val);
                }
                int strLen = str.Length;
                WRITE_SPACE(num1 - strLen);
                writer.Write(str);
                writer.Flush();
            }
            /// <summary>
            /// 文字列をファイル出力する
            /// </summary>
            /// <param name="str">文字列</param>
            public void WRITE_STRING(string str)
            {
                writer.Write(str);
                writer.Flush();
            }
            /// <summary>
            /// 文字列を改行コード付きでファイル出力する
            /// </summary>
            /// <param name="str">文字列</param>
            public void WRITE_LINE(string str)
            {
                writer.WriteLine(str);
                writer.Flush();
            }

            /// <summary>
            /// 文字列を右詰でファイル出力する
            /// </summary>
            /// <param name="str">文字列</param>
            /// <param name="num">バイト長</param>
            public void WRITE_STRING_RIGHT(string str, int num)
            {
                int strLen = Length(str);
                WRITE_SPACE(num - strLen);
                writer.Write(str);
                writer.Flush();
            }
            /// <summary>
            /// 文字列を左詰でファイル出力する
            /// </summary>
            /// <param name="str">文字列</param>
            /// <param name="num">バイト長</param>
            public void WRITE_STRING_LEFT(string str, int num)
            {
                int strLen = Length(str);
                writer.Write(str);
                WRITE_SPACE(num - strLen);
                writer.Flush();
            }
            /// <summary>
            /// 改行をファイル出力する
            /// </summary>
            public void WRITE_NL()
            {
                writer.WriteLine("");
                writer.Flush();
            }
        }
        #endregion

        // 断面データ
        #region Data

        class CsvFileReader
        {
            private StreamReader sr;
            public CsvFileReader(string filename, Encoding encoding, int headerlines)
            {
                sr = new StreamReader(filename, encoding);
                for (int i = 0; i < headerlines; i++)
                {
                    sr.ReadLine();
                }
            }
            // ファイルを１行読み込み、「,」で分割した文字列の配列を返す。
            private string[] readLine()
            {
                string line = sr.ReadLine();
                while (line != null)
                {
                    if (0 < line.Trim().Length)
                    {
                        return System.Text.RegularExpressions.Regex.Split(line.Trim(), ",");
                    }
                    line = sr.ReadLine();
                }
                return null;
            }
            // ファイルの[xxx]の行から[end]の行まで読み込み、その間の行のカンマ区切りの文字列配列の１行
            // ごとのリストを返す。
            public List<string[]> readSection()
            {
                string[] line = readLine();
                if (line != null)
                {
                    List<string[]> result = new List<string[]>();
                    if (line[0].Trim().StartsWith("[") && line[0].Trim().EndsWith("]"))
                    {
                        line = readLine();
                        while (line[0].Trim() != "[end]")
                        {
                            result.Add(line);
                            line = readLine();
                        }
                    }
                    else
                    {
                        result.Add(line);
                    }
                    return result;
                }
                return null;
            }
            public void Close()
            {
                sr.Close();
            }
        }

        class InputFiles
        {
            public readonly List<string> readerrors = new List<string>();


            public readonly CrossFile crossFile;      // 横断面ファイル
            public readonly ThroughFile throughFile;    // 縦断ファイル
            public readonly SignpostFile signpostFile;   // 距離標ファイル

            public InputFiles(string filename1, string filename2, string filename3)
            {
                crossFile = new CrossFile(this, filename1);
                throughFile = new ThroughFile(this, filename2);
                signpostFile = new SignpostFile(this, filename3);

                //縦断ファイルの本支川を本支川コードの順に並べ替える。
                int[] flag = new int[throughFile.sections.Count];
                bool[] selected = new bool[throughFile.sections.Count];
                ThroughFileSection[] sec = new ThroughFileSection[throughFile.sections.Count];  
                for (int i = 0; i < throughFile.sections.Count; i++)
                {
                    if (Int32.TryParse(throughFile.sections[i].records[0].HonshiCode, out flag[i]))
                    {
                        sec[i] = throughFile.sections[i];
                    }
                }
                Array.Sort(flag);
                for(int i = 0; i < selected.Length; i++)
                {
                    selected[i] = false;
                }
                for (int i = 0; i < throughFile.sections.Count; i++)
                {
                    for (int j = i; j < throughFile.sections.Count; j++)
                    {
                        int flag_org; 
                        if(Int32.TryParse(throughFile.sections[j].records[0].HonshiCode, out flag_org))
                        {
                            if (flag[i] == flag_org && !selected[i])
                            {
                                throughFile.sections[i] = sec[j];
                                break;
                            }
                        }
                    }
                }

                //入力ファイルにて各ファイルの河川順序が同じとなることが前提となっているため、
                //縦断ファイルに準じて、本川支川コードをベースに並べ替える。
                int cindex = 0;
                for (int i = 0; i < throughFile.sections.Count; i++)
                {
                    ThroughFileSection tsec = throughFile.sections[i];
                    SignpostFileSection ssec = signpostFile.sections[i];
                    if (!tsec.records[0].RiverName.Equals(ssec.records[0].RiverName)
                        || !tsec.records[0].TopoId.Equals(ssec.records[0].TopoId))
                    {
                        for (int k = i + 1; k < signpostFile.sections.Count; k++)
                        {
                            ssec = signpostFile.sections[k];
                            if (tsec.records[0].RiverName.Equals(ssec.records[0].RiverName)
                                && tsec.records[0].TopoId.Equals(ssec.records[0].TopoId))
                            {
                                signpostFile.sections.RemoveAt(k);
                                signpostFile.sections.Insert(i, ssec);
                            }
                        }
                    }

                    for (int j = 0; j < tsec.records.Count; j++)
                    {
                        ThroughFileRecord trec = tsec.records[j];
                        SignpostFileRecord srec = ssec.records[j];
                        if (!trec.RiverName.Equals(srec.RiverName)
                            || !trec.TopoId.Equals(srec.TopoId)
                            || !trec.DanmenId.Equals(srec.DanmenId))
                        {
                            for (int k = j + 1; k < ssec.records.Count; k++)
                            {
                                srec = ssec.records[k];
                                if (trec.RiverName.Equals(srec.RiverName)
                                    && trec.TopoId.Equals(srec.TopoId)
                                    && trec.DanmenId.Equals(srec.DanmenId))
                                {
                                    ssec.records.RemoveAt(k);
                                    ssec.records.Insert(j, srec);
                                    break;
                                }
                            }
                        }

                        CrossFileSection csec = crossFile.sections[cindex];
                        if (!trec.RiverName.Equals(csec.RiverName)
                            || !trec.TopoId.Equals(csec.TopoId)
                            || !trec.DanmenId.Equals(csec.DanmenId))
                        {
                            for (int k = cindex + 1; k < crossFile.sections.Count; k++)
                            {
                                csec = crossFile.sections[k];
                                if (trec.RiverName.Equals(csec.RiverName)
                                    && trec.TopoId.Equals(csec.TopoId)
                                    && trec.DanmenId.Equals(csec.DanmenId))
                                {
                                    crossFile.sections.RemoveAt(k);
                                    crossFile.sections.Insert(cindex, csec);
                                    break;
                                }
                            }
                        }
                        cindex++;
                    }
                }

            }
        }

        /// <summary>
        /// 横断ファイルのレコードを保存する
        /// </summary>
        class CrossFileRecord
        {
            /// <summary>
            /// 横断ファイルのセクション（断面）
            /// </summary>
            public CrossFileSection section;
            /// <summary>
            /// X座標
            /// </summary>
            public string X = String.Empty;
            /// <summary>
            /// Z座標
            /// </summary>
            public string Z = String.Empty;
            /// <summary>
            /// パネル
            /// </summary>
            public string Panel = String.Empty;
            /// <summary>
            /// 粗度係数
            /// </summary>
            public string Roughness = String.Empty;
            /// <summary>
            /// 死水域
            /// </summary>
            public string Shisuiiki = String.Empty;
            /// <summary>
            /// 植生高さ
            /// </summary>
            public string ShokuseiHeight = String.Empty;
            /// <summary>
            /// 枝下長さ
            /// </summary>
            public string EdashitaLength = String.Empty;
            /// <summary>
            /// 植生密度
            /// </summary>
            public string ShokuseiDensity = String.Empty;
            /// <summary>
            /// 樹タイプ
            /// </summary>
            public string KiType = String.Empty;
        }

        /// <summary>
        /// 横断ファイルのセクション（断面）を保存する
        /// </summary>
        class CrossFileSection
        {
            /// <summary>
            /// 横断ファイル
            /// </summary>
            public readonly CrossFile file;
            /// <summary>
            /// 河川名
            /// </summary>
            public string RiverName = String.Empty; //nd
            /// <summary>
            /// topoID
            /// </summary>
            public string TopoId = String.Empty; //nd
            /// <summary>
            /// 断面ID
            /// </summary>
            public string DanmenId = String.Empty; //nd
            
            public readonly List<CrossFileRecord> records = new List<CrossFileRecord>();
            
            public CrossFileSection(CrossFile file)
            {
                this.file = file;
            }
            
            private ThroughFileRecord throughRecord = null;
            /// <summary>
            /// 対応する縦断ファイルのレコード
            /// </summary>
            /// <returns></returns>
            public ThroughFileRecord getThroughRecord()
            {
                if (throughRecord == null)
                {
                    for (int i = 0; i < file.inputFiles.throughFile.sections.Count; i++)
                    {
                        ThroughFileSection throughSection = file.inputFiles.throughFile.sections[i];
                        for (int ii = 0; ii < throughSection.records.Count; ii++)
                        {
                            ThroughFileRecord tr = throughSection.records[ii];
                            if (tr.RiverName == this.RiverName && tr.TopoId == this.TopoId && tr.DanmenId == this.DanmenId)
                            {
                                throughRecord = tr;
                                throughRecord.crossedSection = this;
                                break;
                            }
                        }
                        if (throughRecord != null)
                        {
                            break;
                        }
                    }
                }
                return throughRecord;
            }

            private SignpostFileRecord signpostRecord = null;
            /// <summary>
            /// 対応する距離標ファイルのレコード
            /// </summary>
            /// <returns></returns>
            public SignpostFileRecord getSignpostRecord()
            {
                if (signpostRecord == null)
                {
                    for (int i = 0; i < file.inputFiles.signpostFile.sections.Count; i++)
                    {
                        SignpostFileSection ssection = file.inputFiles.signpostFile.sections[i];
                        for (int ii = 0; ii < ssection.records.Count; ii++)
                        {
                            SignpostFileRecord sr = ssection.records[ii];
                            if (sr.RiverName == this.RiverName && sr.TopoId == this.TopoId && sr.DanmenId == this.DanmenId)
                            {
                                signpostRecord = sr;
                                sr.crossedSection = this;
                                break;
                            }
                        }
                        if (signpostRecord != null)
                        {
                            break;
                        }
                    }
                }
                return signpostRecord;
            }

            /// <summary>
            /// 川幅
            /// </summary>
            public double RiverWidth
            {
                get
                {
                    double x1 = 0.0f;
                    double x2 = 0.0f;
                    for (int j = 0; j < records.Count; j++)
                    {
                        if (records[j].Panel == "1")
                        {
                            x1 = double.Parse(records[j].X);
                        }
                        if (records[j].Panel == "4")
                        {
                            x2 = double.Parse(records[j].X);
                        }
                    }
                    return x2 - x1;
                }
            }

        }
        
        /// <summary>
        /// 横断ファイル
        /// </summary>
        class CrossFile
        {
            public CrossFile(InputFiles inputFiles, String filename)
            {
                this.inputFiles = inputFiles;
                CsvFileReader reader = new CsvFileReader(filename, McQuasi2DSteadyFlowModelDefine.FILE_ENCODING, 6);
                try
                {
                    read(reader);
                }
                finally
                {
                    reader.Close();
                }
            }
            public readonly InputFiles inputFiles;
            public List<CrossFileSection> sections = new List<CrossFileSection>();
            private void read(CsvFileReader reader)
            {
                List<string[]> csvsection = reader.readSection();
                while (csvsection != null)
                {
                    CrossFileSection section = new CrossFileSection(this);
                    sections.Add(section);

                    section.RiverName = csvsection[0][0].Trim();
                    section.TopoId = csvsection[0][1].Trim();
                    section.DanmenId = csvsection[0][2].Trim();

                    for (int i = 1; i < csvsection.Count; i++)
                    {
                        CrossFileRecord record = new CrossFileRecord();
                        int j = 0;
                        record.section = section;
                        try
                        {
                            record.X = csvsection[i][j++].Trim();
                            record.Z = csvsection[i][j++].Trim();
                            record.Panel = csvsection[i][j++].Trim();
                            record.Roughness = csvsection[i][j++].Trim();
                            record.Shisuiiki = csvsection[i][j++].Trim();
                            record.ShokuseiHeight = csvsection[i][j++].Trim();
                            record.EdashitaLength = csvsection[i][j++].Trim();
                            record.ShokuseiDensity = csvsection[i][j++].Trim();
                            record.KiType = csvsection[i][j++].Trim();
                        }
                        catch
                        {
                            //ignore!!
                        }
                        section.records.Add(record);
                    }
                    csvsection = reader.readSection();
                }
            }
        }

        /// <summary>
        /// 縦断ファイルのレコード
        /// </summary>
        class ThroughFileRecord
        {
            /// <summary>
            /// 縦断ファイルのセクション（河川)
            /// </summary>
            public ThroughFileSection section;
            /// <summary>
            /// 河川名
            /// </summary>
            public string RiverName = String.Empty;         //nd
            /// <summary>
            /// topoID
            /// </summary>
            public string TopoId = String.Empty;         //nd
            /// <summary>
            /// 断面ID
            /// </summary>
            public string DanmenId = String.Empty;         //nd
            /// <summary>
            /// 累加距離
            /// </summary>
            public string TotalDistance = String.Empty;       //m
            /// <summary>
            /// 区間距離
            /// </summary>
            public string SectionDistance = String.Empty;       //m

            /// <summary>
            /// 計画高水位
            /// </summary>
            public string KeikakudakaSuii = String.Empty;     //T.P.m
            /// <summary>
            /// 計画堤防高
            /// </summary>
            public string KeikakuTeibouDaka = String.Empty;     //T.P.m
            /// <summary>
            /// 余裕高
            /// </summary>
            public string Yoyudaka = String.Empty;         //m
            /// <summary>
            /// セグメント
            /// </summary>
            public string Segment = String.Empty;     //nd
            /// <summary>
            /// 左岸痕跡水位
            /// </summary>
            public string LeftBankKonsekiSuii = String.Empty;   //T.P.m
            /// <summary>
            /// 右岸痕跡水位
            /// </summary>
            public string RightBankKonsekiSuii = String.Empty;   //T.P.m
            /// <summary>
            /// 河床材料
            /// </summary>
            public string KashoZairyo = String.Empty;       //nd
            /// <summary>
            /// 粗度係数
            /// </summary>
            public string Sodo = String.Empty;       //nd
            /// <summary>
            /// 射流コード
            /// </summary>
            public string SyaryuCoord = String.Empty;     //nd

            /// <summary>
            /// 左岸堤防端高
            /// </summary>
            public string LeftBankTeibotanDaka = String.Empty; //T.P.m
            /// <summary>
            /// 右岸堤防端高
            /// </summary>
            public string RightBankTeibotanDaka = String.Empty; //T.P.m
            /// <summary>
            /// 左岸天端高
            /// </summary>
            public string LeftBankTenbaDaka = String.Empty; //m
            /// <summary>
            /// 右岸天端高
            /// </summary>
            public string RightBankTenbaDaka = String.Empty; //m
            /// <summary>
            /// 川幅
            /// </summary>
            public string RiverWidth = String.Empty;           //m
            /// <summary>
            /// 低水幅
            /// </summary>
            public string TeisuiroHaba = String.Empty;       //m
            /// <summary>
            /// 河床高
            /// </summary>
            public string KasyoDaka = String.Empty;         //T.P.m
            /// <summary>
            /// 河床勾配
            /// </summary>
            public string KasyoKoubai = String.Empty;       //nd
            /// <summary>
            /// 左岸堤内地盤高
            /// </summary>
            public string LeftBankTeinaiJibanDaka = String.Empty; //T.P.m
            /// <summary>
            /// 右岸堤内地盤高
            /// </summary>
            public string RightBankTeinakJibanDaka = String.Empty; //T.P.m
            /// <summary>
            /// 河積
            /// </summary>
            public string Kaseki = String.Empty;           //m2

            /// <summary>
            /// 左岸高水敷
            /// </summary>
            public string LeftBankKousuiziki = String.Empty;
            /// <summary>
            /// 右岸高水敷
            /// </summary>
            public string RightBankKousuiziki = String.Empty;

            public string KyoryoKouryokuKeisu = String.Empty;
            /// <summary>
            /// 橋梁のピア上流側の水路幅
            /// </summary>
            public string KyoryoWidth1 = String.Empty;
            /// <summary>
            /// 橋梁の全水路幅からピア幅の総計を削除した幅
            /// </summary>
            public string KyoryoWidth2 = String.Empty;
            /// <summary>
            /// 橋梁のピア幅
            /// </summary>
            public string KyoryoPierWidth = String.Empty;
            /// <summary>
            /// 橋梁のピア数
            /// </summary>
            public string KyoryoPierCount = String.Empty;
            /// <summary>
            /// 潜水橋 橋脚１基の投影幅
            /// </summary>
            public string LowWaterBridgePierProjectedWidth = String.Empty;
            /// <summary>
            /// 潜水橋 橋脚の抗力係数
            /// </summary>
            public string LowWaterBridgePierDrag = String.Empty;
            /// <summary>
            /// 潜水橋 橋脚長さ
            /// </summary>
            public string LowWaterBridgePierLength = String.Empty;
            /// <summary>
            /// 潜水橋 桁厚さ
            /// </summary>
            public string LowWaterBridgeGirderThickness = String.Empty;
            /// <summary>
            /// 潜水橋 桁長さ
            /// </summary>
            public string LowWaterBridgeGirderLength = String.Empty;
            /// <summary>
            /// 潜水橋 桁の抗力係数
            /// </summary>
            public string LowWaterBridgeGirderDrag = String.Empty;
            /// <summary>
            /// 潜水橋 橋脚基数
            /// </summary>
            public string LowWaterBridgePierCount = String.Empty;
            /// <summary>
            /// 堰 堰高
            /// </summary>
            public string WeirHeight = String.Empty;
            /// <summary>
            /// 堰 堰天端幅
            /// </summary>
            public string WeirTopWidth = String.Empty;
            /// <summary>
            /// 堰 上流側勾配
            /// </summary>
            public string WeirInclinationOnUpstream = String.Empty;
            /// <summary>
            /// 堰 下流側勾配
            /// </summary>
            public string WeirInclinationOnTheDownstream = String.Empty;
            /// <summary>
            /// 堰 堰幅
            /// </summary>
            public string WeirWidth = String.Empty;
            /// <summary>
            /// 段差 段差
            /// </summary>
            public string DansaHeight = String.Empty;
            /// <summary>
            /// 段差 幅
            /// </summary>
            public string DansaWidth = String.Empty;

            /// <summary>
            /// 湾曲曲率半径
            /// </summary>
            public string WankyokuKyokuritsuHankei = String.Empty;

            public string SodoSuibotuJumoku = String.Empty;
            /// <summary>
            /// 混合係数 急拡
            /// </summary>
            public string KongouKeisu0 = String.Empty;
            /// <summary>
            /// 混合係数 b/B<0.6;
            /// </summary>
            public string KongouKeisu1 = String.Empty;//
            /// <summary>
            /// 混合係数 0.6≦b/B
            /// </summary>
            public string KongouKeisu2 = String.Empty;//;
            /// <summary>
            /// 混合係数 0.8＜b/B
            /// </summary>
            public string KongouKeisu3 = String.Empty;//;
            /// <summary>
            /// 混合係数 堤防
            /// </summary>
            public string KongouKeisu4 = String.Empty;
            /// <summary>
            /// 混合係数 流れに挟まる
            /// </summary>
            public string KongouKeisu5 = String.Empty;
            /// <summary>
            /// 混合係数 流れに挟まれ水没
            /// </summary>
            public string KongouKeisu6 = String.Empty;
            /// <summary>
            /// 割引率 粗
            /// </summary>
            public string WaribikiRitsuSo = String.Empty;
            /// <summary>
            /// 割引率 密
            /// </summary>
            public string WaribikiRitsuMitsu = String.Empty;
            /// <summary>
            /// 本川支川フラグ
            /// </summary>
            public string HonshiCode = String.Empty;
            /// <summary>
            /// 流量
            /// </summary>
            public string Flow = String.Empty;
            /// <summary>
            /// 出発水位
            /// </summary>
            public string WaterLevel = String.Empty;
            /// <summary>
            /// 接続河川名
            /// </summary>
            public string SetsuzokuRiverName = String.Empty;
            /// <summary>
            /// 接続位置
            /// </summary>
            public string SetsuzokuIchi = String.Empty;
            /// <summary>
            /// 合流前川幅 B1
            /// </summary>
            public string GoryuMaeRiverWidthB1 = String.Empty;
            /// <summary>
            /// 合流前川幅 B1
            /// </summary>
            public string GoryuMaeRiverWidthB2 = String.Empty;
            /// <summary>
            /// 合流後川幅 B3
            /// </summary>
            public string GoryuGoRiverWidthB3 = String.Empty;
            /// <summary>
            /// 合流角度T1
            /// </summary>
            public string GoryuKakudoT1 = String.Empty;
            /// <summary>
            /// 合流角度T2
            /// </summary>
            public string GoryuKakudoT2 = String.Empty;

            public CrossFileSection crossedSection = null;
            /// <summary>
            /// 対応する横断ファイルのセクション(断面）
            /// </summary>
            public CrossFileSection getCrossedSection()
            {
                if (crossedSection == null)
                {
                    for (int i = 0; i < section.file.inputFiles.crossFile.sections.Count; i++)
                    {
                        CrossFileSection csec = section.file.inputFiles.crossFile.sections[i];
                        if (csec.getThroughRecord() == this)
                        {
                            break;
                        }
                    }
                }
                return crossedSection;
            }
            
            /// <summary>
            /// 対応する距離標ファイルのレコード
            /// </summary>
            /// <returns></returns>
            public SignpostFileRecord getSignpostRecord()
            {
                return getCrossedSection().getSignpostRecord();
            }
            
            /// <summary>
            /// 河川名と断面IDから縦断レコードを取得します。
            /// </summary>
            /// <param name="RiverName"></param>
            /// <param name="DanmenId"></param>
            /// <returns></returns>
            private ThroughFileRecord SearchIchiRecord(string RiverName, string DanmenId)
            {
                return section.file.inputFiles.throughFile.SearchRecord(RiverName, DanmenId);
            }

            /// <summary>
            /// 接続位置に指定されている縦断レコードを取得します。
            /// </summary>
            /// <param name="RiverName"></param>
            /// <param name="DanmenId"></param>
            /// <returns></returns>
            public ThroughFileRecord SetsuzokuIchiRecord()
            {
                return SearchIchiRecord(SetsuzokuRiverName, SetsuzokuIchi);
            }

            /// <summary>
            /// 合流点であるかどうかを返します。
            /// </summary>
            /// <returns></returns>
            public bool isBreakPoint()
            {
                for (int i = 0; i < section.file.sections.Count; i++)
                {
                    if (section.file.sections[i] != section)
                    {
                        if (section.file.sections[i].records[0].SetsuzokuIchiRecord() == this
                            || section.file.sections[i].records[section.file.sections[i].records.Count - 1].SetsuzokuIchiRecord() == this)
                        {
                            return true;
                        }
                    }
                }
                return false;
            }
            /// <summary>
            /// 合流点に合流している支川コードを返します。合流点でない場合は、-1を返します。
            /// </summary>
            /// <returns></returns>
            public int connected_RiverCode()
            {
                if(section.file.inputFiles.throughFile.sections.Count == 1){
                    return -1;
                }
                else if(isBreakPoint())
                {
                    for (int i = 1; i < section.file.inputFiles.throughFile.sections.Count; i++)
                    {
                        if (section.file.inputFiles.throughFile.sections[i].records[0].SetsuzokuIchiRecord() == this)
                        {
                            return i;
                        }
                    }
                    throw new Exception("接続河川が定義されていません。: " + this.RiverName + " 断面ID：" + this.DanmenId);
                }
                else
                {
                    return -1;
                }
            }
        }
        
        /// <summary>
        /// 縦断ファイルのセクション（河川）を管理します。
        /// </summary>
        class ThroughFileSection
        {
            public ThroughFile file;
            public List<ThroughFileRecord> records = new List<ThroughFileRecord>();
            internal string SekiFlag()
            {
                return "0";
            }

            internal string KyokyakuFlag()
            {
                return "0";
            }

            internal string SensuikyoFlag()
            {
                return "0";
            }

            internal string WankyokuFlag()
            {
                return "0";
            }
        }

        class ThroughFile
        {
            public ThroughFile(InputFiles inputFiles, String filename)
            {
                this.inputFiles = inputFiles;
                CsvFileReader reader = new CsvFileReader(filename, McQuasi2DSteadyFlowModelDefine.FILE_ENCODING, 3);
                try
                {
                    read(reader);
                }
                finally
                {
                    reader.Close();
                }
            }
            public readonly InputFiles inputFiles;
            public List<ThroughFileSection> sections = new List<ThroughFileSection>();
            private void read(CsvFileReader reader)
            {
                List<string[]> csvsection = reader.readSection();
                while (csvsection != null)
                {
                    ThroughFileSection section = new ThroughFileSection();
                    sections.Add(section);
                    section.file = this;
                    for (int i = 0; i < csvsection.Count; i++)
                    {
                        ThroughFileRecord record = new ThroughFileRecord();
                        record.section = section;
                        try
                        {
                            int j = 0;
                            record.RiverName = csvsection[i][j++].Trim();
                            record.TopoId = csvsection[i][j++].Trim();
                            record.DanmenId = csvsection[i][j++].Trim();
                            record.TotalDistance = csvsection[i][j++].Trim();
                            record.SectionDistance = csvsection[i][j++].Trim();

                            record.KeikakudakaSuii = csvsection[i][j++].Trim();
                            record.KeikakuTeibouDaka = csvsection[i][j++].Trim();
                            record.Yoyudaka = csvsection[i][j++].Trim();
                            record.Segment = csvsection[i][j++].Trim();
                            record.LeftBankKonsekiSuii = csvsection[i][j++].Trim();

                            record.RightBankKonsekiSuii = csvsection[i][j++].Trim();
                            record.KashoZairyo = csvsection[i][j++].Trim();
                            record.Sodo = csvsection[i][j++].Trim();
                            record.SyaryuCoord = csvsection[i][j++].Trim();
                            record.LeftBankTeibotanDaka = csvsection[i][j++].Trim();

                            record.RightBankTeibotanDaka = csvsection[i][j++].Trim();
                            record.LeftBankTenbaDaka = csvsection[i][j++].Trim();
                            record.RightBankTenbaDaka = csvsection[i][j++].Trim();
                            record.RiverWidth = csvsection[i][j++].Trim();
                            record.TeisuiroHaba = csvsection[i][j++].Trim();

                            record.KasyoDaka = csvsection[i][j++].Trim();
                            record.KasyoKoubai = csvsection[i][j++].Trim();
                            record.LeftBankTeinaiJibanDaka = csvsection[i][j++].Trim();
                            record.RightBankTeinakJibanDaka = csvsection[i][j++].Trim();
                            record.Kaseki = csvsection[i][j++].Trim();

                            record.LeftBankKousuiziki = csvsection[i][j++].Trim();
                            record.RightBankKousuiziki = csvsection[i][j++].Trim();

                            record.KyoryoKouryokuKeisu = csvsection[i][j++].Trim();
                            record.KyoryoWidth1 = csvsection[i][j++].Trim();
                            record.KyoryoWidth2 = csvsection[i][j++].Trim();
                            record.KyoryoPierWidth = csvsection[i][j++].Trim();
                            record.KyoryoPierCount = csvsection[i][j++].Trim();
                            record.LowWaterBridgePierProjectedWidth = csvsection[i][j++].Trim();
                            record.LowWaterBridgePierDrag = csvsection[i][j++].Trim();
                            record.LowWaterBridgePierLength = csvsection[i][j++].Trim();
                            record.LowWaterBridgeGirderThickness = csvsection[i][j++].Trim();
                            record.LowWaterBridgeGirderLength = csvsection[i][j++].Trim();
                            record.LowWaterBridgeGirderDrag = csvsection[i][j++].Trim();
                            record.LowWaterBridgePierCount = csvsection[i][j++].Trim();
                            record.WeirHeight = csvsection[i][j++].Trim();
                            record.WeirTopWidth = csvsection[i][j++].Trim();
                            record.WeirInclinationOnUpstream = csvsection[i][j++].Trim();
                            record.WeirInclinationOnTheDownstream = csvsection[i][j++].Trim();
                            record.WeirWidth = csvsection[i][j++].Trim();
                            record.DansaHeight = csvsection[i][j++].Trim();
                            record.DansaWidth = csvsection[i][j++].Trim();
                            record.WankyokuKyokuritsuHankei = csvsection[i][j++].Trim();


                            record.SodoSuibotuJumoku = csvsection[i][j++].Trim();
                            record.KongouKeisu0 = csvsection[i][j++].Trim();
                            record.KongouKeisu1 = csvsection[i][j++].Trim();//b/B<0.6;
                            record.KongouKeisu2 = csvsection[i][j++].Trim();//0.6≦b/B;
                            record.KongouKeisu3 = csvsection[i][j++].Trim();//0.8＜b/B;
                            record.KongouKeisu4 = csvsection[i][j++].Trim();
                            record.KongouKeisu5 = csvsection[i][j++].Trim();
                            record.KongouKeisu6 = csvsection[i][j++].Trim();
                            record.WaribikiRitsuSo = csvsection[i][j++].Trim();
                            record.WaribikiRitsuMitsu = csvsection[i][j++].Trim();
                            record.HonshiCode = csvsection[i][j++].Trim();
                            record.Flow = csvsection[i][j++].Trim();
                            record.WaterLevel = csvsection[i][j++].Trim();
                            record.SetsuzokuRiverName = csvsection[i][j++].Trim();
                            record.SetsuzokuIchi = csvsection[i][j++].Trim();
                            record.GoryuMaeRiverWidthB1 = csvsection[i][j++].Trim();
                            record.GoryuMaeRiverWidthB2 = csvsection[i][j++].Trim();
                            record.GoryuGoRiverWidthB3 = csvsection[i][j++].Trim();
                            record.GoryuKakudoT1 = csvsection[i][j++].Trim();
                            record.GoryuKakudoT2 = csvsection[i][j++].Trim();
                        }
                        catch (Exception ignore)
                        {
                            inputFiles.readerrors.Add(ignore.Message);
                        }
                        section.records.Add(record);
                    }
                    csvsection = reader.readSection();
                }
            }

            public ThroughFileRecord SearchRecord(string RiverName, string DanmenId)
            {
                for (int i = 0; i < sections.Count; i++)
                {
                    ThroughFileSection sec = sections[i];
                    for (int ii = 0; ii < sec.records.Count; ii++)
                    {
                        ThroughFileRecord rec = sec.records[ii];
                        if (rec.RiverName == RiverName && rec.DanmenId == DanmenId)
                        {
                            return rec;
                        }
                    }
                }
                return null;
            }
        }

        /// <summary>
        /// 距離標レコード
        /// </summary>
        class SignpostFileRecord
        {
            public SignpostFileSection section;
            /// <summary>
            /// 河川名
            /// </summary>
            public string RiverName = String.Empty;
            /// <summary>
            /// topoID
            /// </summary>
            public string TopoId = String.Empty;
            /// <summary>
            /// 断面ID
            /// </summary>
            public string DanmenId = String.Empty;
            /// <summary>
            /// 左岸緯度
            /// </summary>
            public string LeftBankLatitude = String.Empty;
            /// <summary>
            /// 左岸経度
            /// </summary>
            public string LeftBankLongtitude = String.Empty;
            /// <summary>
            /// 右岸緯度
            /// </summary>
            public string RightBankLatitude = String.Empty;
            /// <summary>
            /// 右岸経度
            /// </summary>
            public string RightBankLongtitude = String.Empty;
            public CrossFileSection crossedSection;
            public CrossFileSection getCrossedSection()
            {
                if (crossedSection == null)
                {
                    for (int i = 0; i < section.file.inputFiles.crossFile.sections.Count; i++)
                    {
                        CrossFileSection csec = section.file.inputFiles.crossFile.sections[i];
                        if (csec.getSignpostRecord() == this)
                        {
                            break;
                        }
                    }
                }
                return crossedSection;
            }
            public ThroughFileRecord getThroughRecord()
            {
                return getCrossedSection().getThroughRecord();
            }
        }

        class SignpostFileSection
        {
            public SignpostFile file;
            public List<SignpostFileRecord> records = new List<SignpostFileRecord>();
        }

        class SignpostFile
        {
            public SignpostFile(InputFiles inputFiles, String filename)
            {
                this.inputFiles = inputFiles;
                CsvFileReader reader = new CsvFileReader(filename, McQuasi2DSteadyFlowModelDefine.FILE_ENCODING, 3);
                try
                {
                    read(reader);
                }
                finally
                {
                    reader.Close();
                }
            }
            public readonly InputFiles inputFiles;
            public string CoordinateSystem = String.Empty; //nd	
            public string SystemNo = String.Empty; //nd	
            public readonly List<SignpostFileSection> sections = new List<SignpostFileSection>();
            private void read(CsvFileReader reader)
            {
                List<string[]> csvsections = reader.readSection();
                int j = 0;
                CoordinateSystem = csvsections[0][j++].Trim();
                SystemNo = csvsections[0][j++].Trim();

                reader.readSection();//3行はヘッダ
                reader.readSection();//3行はヘッダ
                reader.readSection();//3行はヘッダ

                csvsections = reader.readSection();
                while (csvsections != null)
                {
                    SignpostFileSection section = new SignpostFileSection();
                    sections.Add(section);
                    section.file = this;
                    for (int i = 0; i < csvsections.Count; i++)
                    {
                        SignpostFileRecord record = new SignpostFileRecord();
                        record.section = section;
                        j = 0;
                        record.RiverName = csvsections[i][j++].Trim();
                        record.TopoId = csvsections[i][j++].Trim();
                        record.DanmenId = csvsections[i][j++].Trim();
                        try
                        {
                            record.LeftBankLatitude = csvsections[i][j++].Trim();
                            record.LeftBankLongtitude = csvsections[i][j++].Trim();
                            record.RightBankLatitude = csvsections[i][j++].Trim();
                            record.RightBankLongtitude = csvsections[i][j++].Trim();
                        }
                        catch
                        {
                            //ignore
                        }
                        section.records.Add(record);
                    }
                    csvsections = reader.readSection();
                }
            }
        }

        #endregion
    }
}
